/*=============================================================================
 *     Copyright Texas Instruments 2002. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * An input handler that recognizes debug/warning/error messages from the 
 * console and colorizes them appropriately.
 * 
 * @param console      the console to install into
 */
public function ConsoleInputHandler(console)
  extends ti.swing.console.InputAdapter( console.getInputHandler() )
{
  const var FgColorAttribute  = ti.swing.console.FgColorAttribute;
  const var Color = java.awt.Color;
  
  const var debugColor   = Color.green;
  const var warningColor = Color.yellow;
  const var errorColor   = Color.red;
  
  const var super_append = append;
  
  function write(str)
  {
    super_append( str, 0, str.length() );
  }
  
  /**
   * Append characters to the end of the character stream.  Note that this
   * method is the entry point to a bunch of internal processing that is
   * not thread safe, so it is synchronized.
   * 
   * @param cbuf         the character buffer
   * @param off          the offset into cbuf to first character to append
   * @param len          the number of characters to append
   */
  public function append( cbuf, off, len )
  {
    synchronized( getBufferLock() )
    {
      lock();
      
      for( var i=0; i<len; i++ )
        state = state( "" + cbuf[off+i] );  // XXX hack... interpreter should recognize char[] as a String....
      
      unlock();
    }
  }
  
  // keep track of color-starts, so when we get to end of line, we can apply color if needed
  var regionColor = null;
  var regionStartOffset;
  
  function nullState(c)
  {
    write(c);
    
    if( c == "\n" )
    {
      var offset = getOffset();
      
      if( regionColor != null )
      {
        addRegion( (new FgColorAttribute(regionColor)).
                   getRegion( regionStartOffset, offset - regionStartOffset ) );
        regionColor = null;
      }
      
      // function which returns an intermediate string for
      // a partial match... perhaps this should be expanded
      // to be able to specify a next-state(s), which could
      // make it useful for cases where we are matching
      // different strings that have the same first n chars
      function makeMatcherState( str, idx, color )
      {
        return function(c) {
          
          if( c == str[idx] )
          {
            if( idx == (str.length()-1) )
            {
                // we've matchted all characters in string:
              regionColor = color;
              regionStartOffset = offset;
              
              write(str);
              return nullState;
            }
            
            return makeMatcherState( str, idx+1, color );
          }
          else
          {
            write(str[0..idx-1] + c);
            return nullState;
          }
          
        };
      }
        
      return function(c) {
        
        if( c == "d" )
          return makeMatcherState( "debug", 1, debugColor );
        else if( c == "w" )
          return makeMatcherState( "warning", 1, warningColor );
        else if( c == "e" )
          return makeMatcherState( "error", 1, errorColor );
          
        write(c);
        return nullState;
      };
    }
    
    return nullState;
  }
  
  var state = nullState;
}



/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */
