/*=============================================================================
 *     Copyright Texas Instruments, Inc., 2001.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * The console-plugin is a bit of a special case... it is started from the
 * bootstrap code, and cannot be closed.  Also, it makes no sense to have
 * more than one console, so it is never added to the plugin menu.
 */
registry.register( new (function() extends ti.chimera.Plugin( main, "Console" ) {
  
  pkg.registry.setDefaultPersistentValue( 
    "/Dialogs/Console/bounds",
    new java.awt.Rectangle(330,0,613,450) 
  );
  
  const var OscriptInterpreter = oscript.OscriptInterpreter;
  const var BasicScope = oscript.data.BasicScope;
  
  /**
   * Used to generate a unique name for each view.
   */
  var consoleCnt;
  
  /**
   * Table mapping view name to view.
   */
  var nameToViewTable = null;
  
  services["talkback"].addReportAttachmentFxn( '{
    
    var str = "";
    
    if( nameToViewTable != null )
    {
      for( var name : nameToViewTable.keySet() )
      {
        var view = nameToViewTable.get(name);
        str += "---------------------------\n";
        str += "Console: " + name + "\n\n";
        str += view.getContents();
      }
    }
    
    return new pkg.email.PlainTextAttachment( "console.txt", str );
    
  } );
  
  /**
   * List of all views, first in list is the primary view.
   */
  var viewList;
  
  addResource( new (function() extends ti.chimera.Resource(true) {
    
    public function install()
    {
      consoleCnt = 0;
      nameToViewTable = new java.util.Hashtable();
      viewList = new java.util.LinkedList();
    }
    
    public function uninstall()
    {
      consoleCnt = null;
      nameToViewTable = null;
      viewList = null;
    }
    
    uninstall();  // initialze to in-active state
    
  })() );
  
  /* At system shutdown time, close all console writers
   */
  main.atExit( '{ 
    if( viewList == null )
      return;
    for( var view : viewList.toArray() )
    {
      view.saveHistory();
      try
        view.getConsoleWriter().close();
      catch(java.io.IOException e)
        services["prompt"].showErrorMessage( e.getMessage() );
    }
  } );

  /**
   * The console-view class.  The console creates a single view, which
   * contains, yup you guessed it, the console!
   */
  function ConsoleView(name)
    extends ti.chimera.View( main, name, pkg.util.getImageIcon("Host16.gif") )
  {
    protected var console = pkg.util.makeConsole( this, 200, 80, false );
    
    if( ! registry.exists("/Console/history") )
      registry.link( new ti.chimera.registry.PersistentNode( console.getHistory(), null, null ), "/Console/history" );
    
    console.setHistory( registry.resolve("/Console/history").getValue() );
    
    // install ConsoleInputHandler to colorize error/warning/debug msgs:
    console.setInputHandler( new pkg.ConsoleInputHandler(console) );
    
    public function getComponent()
    {
      return console;
    }
    
    public function getConsoleReader()
    {
      return console.getReader();
    }
    
    public function getConsoleWriter()
    {
      return consoleWriter;
    }
    private var consoleWriter = console.getWriter();
    
    public function startLogging(out)
    {
      if( originalInputHandler != null )
        console.setInputHandler(originalInputHandler);
      else
        originalInputHandler = console.getInputHandler();
      console.setInputHandler( new ti.swing.console.LogInputHandler( console, out ) );
    }
    private var originalInputHandler = null;
    
    public function saveHistory()
    {
      registry.resolve("/Console/history").setValue( console.getHistory() );
    }
    
    public function getContents()
    {
      // getData() returns a character array, so use java.lang.String constructor
      // to force conversion to string:
      return new java.lang.String( console.getInputHandler().getData( 0, Integer.MAX_VALUE ) );
    }
    
    /**
     * Create an action for this view.
     * 
     * @param name         the name of the action
     * @param accel        the accelerator key, a single character string
     * @param fxn          the function to call when the action happens
     */
    protected function createAction( name, accel, fxn )
    {
      var a = new pkg.util.ActionAdapter( name, null, null, '{ fxn(); } );
      
      if( accel != null )
      {
        console.getInputMap().put( pkg.util.getAcceleratorKeyStroke(accel), name );
        console.getActionMap().put( name, a );
      }
      
      addAction(a);
    }
    
    // show the help screen:
    addAction( new pkg.util.ActionAdapter( "Help", null, pkg.util.getImageIcon("Help16.gif"),
                                           '{ services["help"].displayHelp(HELP_PATH); } ) );
    
    // this lets the user open a new console:
    addAction( new pkg.util.ActionAdapter( "New Console", null, null, 
                                           '{ services["console"].newConsole(); } ) );
    
    // this lets the user select a file to log output to console to:
    addAction( new pkg.util.ActionAdapter( "Log", null, null, 
                                           '{ services["console"].logToFile( getName(), null ); } ) );
    
    nameToViewTable.put( getName(), this );
    
    
    if( viewList.size() == 0 )
    {
      main.setIn( getConsoleReader() );
      main.setOut( getConsoleWriter() );
      main.setErr( getConsoleWriter() );
    }
    
    viewList.add(this);
  }
  
  /**
   */
  function ScriptConsoleView(name)
    extends ConsoleView(name)
  {
    synchronized(registry)
    {
      var node = registry.resolve("/Console/count");
      node.setValue( node.getValue() + 1 );
    }
    
    addCloseRunnable( '{
      
      synchronized(registry)
      {
        var node = registry.resolve("/Console/count");
        node.setValue( node.getValue() - 1 );
      }
      
      nameToViewTable.remove( getName() );
      console.dispose();
      
      if( viewList != null )
      {
        viewList.remove(this);
        
        if( viewList.size() == 0 )
        {
          main.setIn( new java.io.InputStreamReader(System.in) );
          main.setOut( new java.io.OutputStreamWriter(System.out) );
          main.setErr( new java.io.OutputStreamWriter(System.out) );
        }
        else
        {
          main.setIn( viewList.getFirst().getConsoleReader() );
          main.setOut( viewList.getFirst().getConsoleWriter() );
          main.setErr( viewList.getFirst().getConsoleWriter() );
        }
      }
      
      services["file browser"].removeDragAndDropSupport(console);
      
      saveHistory();
      
    } );
    
    
    var tabCompleter = null;
    
    public function interrupt()
    {
      if( (tabCompleter == null) || !tabCompleter.isPopupVisible() )
        t.interrupt();
    }
    
    //createAction( "Interrupt", "PERIOD", interrupt );
    createAction( "Interrupt", "ESCAPE", interrupt );
    
    var tg = new ThreadGroup(name);
    var t  = new (function() extends Thread( tg, name ) {
      
      public function run()
      {
        // interpreter object, mainly just for backwards compatibility:
        var interpreter = new (function() {
          
          var in  = new java.io.BufferedReader( getConsoleReader() );
          var out = new java.io.PrintWriter( getConsoleWriter(), true );
          
          public function getIn()  { return in; }
          public function getOut() { return out; }
          public function getErr() { return out; }
          
          // display a welcome message:
          out.println( getWelcomeMessage() );
          
        })();
        
        var scope = new BasicScope( oscript.OscriptInterpreter.getGlobalScope() );
        pkg.system.export( true, "console", interpreter, scope );
        console.setTabCompleter( tabCompleter = new pkg.util.ScriptConsoleTabCompleter( console, scope ) );
        
        // initialize input/output:
        pkg.output.initialize( interpreter.getOut() );
        pkg.input.initialize( interpreter.getIn() );
        
        // and now run until the user exits
        services["shell"].run( scope, interpreter.getIn(), interpreter.getOut(), interpreter.getErr() );
        
        pkg.output.dispose();
        pkg.input.dispose();
        
        // and finally, close the view, in case it isn't already:
        close();
      }
      
    })();
    t.start();
    
    /**
     * Import the specified file within this view's context.  This actually
     * imports from the context of the read-eval-print thread (when the thread
     * returns to the read phase) because the import happens by pasting the
     * appopriate <code>import "path/to/file.os";</code> into the stream the
     * read-eval-print reader is reading from.
     * 
     * @param file         the file to import
     */
    public function doImport(file)
    {
      if( file.exists() )
      {
        toForeground();
        console.paste("import \"" + file.getPath() + "\";\n");
      }
    }
    
    services["file browser"].addDragAndDropSupport( "os", console, doImport );
  }
  
  services["file browser"].
    addAction( "os", new pkg.util.ActionAdapter( "run", null, null, '{
      
      services["console"].doImport( args[0].getSource() );
      
    } ) );
  
  /**
   * The "console" service provided by the console allows you to programmatically
   * set a file to log the output of the console to.
   */
  registerServiceFactory( function() {
    
    return new (function() extends ti.chimera.Service("console") {
      
      /**
       * Import the specified file within the context of the primary console
       * window.
       * 
       * @param file         the file to import
       */
      public function doImport(file)
      {
        file = pkg.util.getFile(file);
        
        if( viewList.size() == 0 )
          newConsole();
        
        viewList.getFirst().doImport(file);
      }
      
      /**
       * Log the output written to this console to the named file.  If no file
       * name is provided, a chooser dialog will be shown to allow the user
       * to select a file.
       * 
       * @param name         the name of which console to log
       * @param filename     the name of the file or <code>null</code>
       */
      public function logToFile( name, filename )
      {
        var file;
        
        if( filename == null )
        {
          var chooser = new ti.chimera.FileChooser( windowManager.getDialog("Select Log File") );
          
          chooser.setFileFilter( new pkg.util.ExtensionFileFilter( "html", "Html Log File" ) );
          chooser.setSelectedFile( new pkg.File("console_log.html") );
          
          if( chooser.showSaveDialog(null) == ti.chimera.FileChooser.APPROVE_OPTION )
            file = chooser.getSelectedFile();
          else
            return;
        }
        else
        {
          file = new pkg.File(filename);
        }
        
        log( name, new java.io.OutputStreamWriter( file.getOutputStream(false) ) );
      }
      
      /**
       * Log the output written to the console to a {@link java.io.Writer}.
       * 
       * @param name         the name of which console to log
       * @param out          the output stream to log to
       */
      public function log( name, out )
      {
        var view = nameToViewTable.get(name);
        if( view != null )
          view.startLogging(out);
      }
      
      /**
       * Iterrupt the thread of the named console.
       * 
       * @param name         the name of the console to interrupt
       */
      public function interrupt(name)
      {
        var view = nameToViewTable.get(name);
        if( view != null )
          view.interrupt();
      }
      
      /**
       * Create a new console.
       */
      public function newScriptConsole()
      {
        synchronized(consoleCnt)
          var name = "Console " + (++consoleCnt);
        
        addResource( new ViewResource( new ScriptConsoleView(name) ) );
      }
      
      public var newConsole = newScriptConsole;  // XXX for compatibility
      
      /**
       */
      public function createConsole(name)
      {
        var view = new ConsoleView(name);
        addResource( new ViewResource(view) );
        
        return new '{
          
          public function getIn()  { return view.getConsoleReader(); }
          public function getOut() { return view.getConsoleWriter(); }
          public function close()  { view.close(); }
          
        }();
      }
      
    })();
    
  } );
  
  /**
   * Get the welcome message
   */
  private function getWelcomeMessage()
  {
    var str = OscriptInterpreter.getVersionString();
    
    if( pkg.util.addon.isEnabled("fortune") )
      str += "\n\n" + services["fortune"].get(null);
    
    return str;
  }
  
  const var HELP_PATH = "Plugins/Console";
  services["help"].registerHelp( HELP_PATH, "ConsolePluginHelp.html", "Help for the Console Plugin" );
  
})() );



/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

