/******************************************************************************
 *     Copyright Texas Instruments 2004. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * The debugger plugin provides a way to debug objectscript code.
 */
registry.register( new (function() extends ti.chimera.Plugin( main, "Debugger" ) {
  
  const var ev = oscript.OscriptInterpreter.__eval;
  
  const var breakpointGlyph     = pkg.util.getImageIcon("breakpointglyph.gif");
  const var programCounterGlyph = pkg.util.getImageIcon("pcglyph.gif");
  
  /**
   * The Breakpoint-view makes use of some toolbar / menubar actions that
   * are context sensitive, and managed by this action manager.  These are
   * only enabled when <code>bpView</code> exists, and is stopped at a
   * breakpoint.
   */
  var am = new pkg.util.ActionManager();
  am.addGlobalAction( true, "/Debugger/Step",          "Step",          "Step",          null,
                      "StepInto.gif",       null,  '{ services["debugger"].doStep(); } );
  am.addGlobalAction( true, "/Debugger/Step Over",     "Step Over",     "Step Over",      null,
                      "StepOver.gif",       null,  '{ services["debugger"].doStepOver(); } );
  am.addGlobalAction( true, "/Debugger/Continue",      "Continue",      "Continue",       null,
                      "Continue.gif",       null,  '{ services["debugger"].doContinue(); } );
  am.addGlobalAction( true, "/Debugger/Run to Cursor", "Run to Cursor", "Run to Cursor",  null,
                      "RunToCursor.gif",    null,  '{
                        var frame = pkg.editor.EditorFrame.getCurrentFrame();
                        services["debugger"].doRunTo( frame.getFile(), frame.getLineNumber() );
                      } );
  am.getToolBar().addSeparator();
  windowManager.addMenuBarItem( "/Debugger", null );
  am.addGlobalAction( true, "/Debugger/Watch",         "Watch",         "New Watched Expression", null,
                      "NewWatch.gif",       null,  '{ services["debugger"].doWatch(null); } );
  am.addGlobalAction( true, "/Debugger/Shell",         "Shell",         "Debug Shell",    null,
                      "DebuggerWindow.gif", null,  '{ services["debugger"].doShell(); } );
  var toolBarResource = new ToolBarResource( am.getToolBar(), false );
  
  /**
   * Only a single breakpoint view is ever used
   */
  var bpView = null;
  
  /**
   * The number of outstanding breakpoints.
   */
  var nbreakpoints = 0;
  
  
  private function incrementBreakpointViewUse()
  {
    nbreakpoints++;
    if( bpView == null )
    {
      services["swing worker"].run( function() {
        
        //synchronized(breakpointGlyph)  // synchronize on something...
        {
          if( bpView == null ) // have to double check within synchronization
          {
            addResource( new ViewResource( bpView = new BreakpointView() ) );
            addResource(toolBarResource);
          }
        }
        
      }, "Launching Debugger" );
    }
  }
  
  private function decrementBreakpointViewUse()
  {
    if( (--nbreakpoints == 0) && (bpView != null) )
    {
      bpView.close();
      removeResource(toolBarResource);
    }
  }
  
  /****************************************************************************/
  /**
   * The "debugger" service.
   */
  registerServiceFactory( function() {
    
    return new (function() extends ti.chimera.Service("debugger") {
      
      /**
       * If at a breakpoint, step, otherwise undefined
       */
      public function doStep()
      {
        bpView.eventQueue.enqueue("exit");
        bpView.eventQueue.enqueue("step");
      }
      
      /**
       * If at a breakpoint, step-over the next scope change, otherwise
       * undefined.
       */
      public function doStepOver()
      {
        bpView.eventQueue.enqueue("exit");
        bpView.eventQueue.enqueue("over");
      }
      
      /**
       * If at a breakpoint, continue running, otherwise undefined.
       */
      public function doContinue()
      {
        bpView.eventQueue.enqueue("exit");
        bpView.eventQueue.enqueue("continue");
      }
      
      /**
       * If at a breakpoint, add the specified <code>expr</code> to the list
       * of watched expressions.  If <code>expr</code> is <code>null</code>,
       * the user is prompted for input.
       * <p>
       * The watched expression is evaluated at each step as the user steps
       * through the code, and the result is printed to the debugger console.
       * 
       * @param expr  the expression to watch, or <code>null</code>
       */
      public function doWatch(expr)
      {
        bpView.eventQueue.enqueue("exit");
        bpView.eventQueue.enqueue("watch");
        if( expr != null )
          bpView.eventQueue.enqueue(expr);
      }
      
      /**
       * If at a breakpoint, enter the debug shell, otherwise undefined.
       */
      public function doShell()
      {
        bpView.eventQueue.enqueue("exit");
        bpView.eventQueue.enqueue("shell");
      }
      
      /**
       * If at a breakpoint, run to the specified file and line number.  If
       * both <code>file</code> and <code>line</code> are <code>null</code>
       * then the user is prompted for input.
       * 
       * @param file   the file
       * @param line   the line number
       */
      public function doRunTo( file, line )
      {
        bpView.eventQueue.enqueue("exit");
        bpView.eventQueue.enqueue("runto");
        
        if( (file != null) && (line != null) )
        {
          bpView.eventQueue.enqueue( pkg.util.getFile(file).getPath() );
          bpView.eventQueue.enqueue(line);
        }
      }
      
      /**
       * Generate an exception string.  If the exception is a script exception,
       * then the error message will have hyperlinks for launching the editor
       * and/or debugger.
       * 
       * @param e    the exception
       * @return a string, possibly containing hyperlinks
       */
      public function makeExceptionMessage(e)
      {
        if( (e instanceof oscript.parser.ParseException) ||
            (e instanceof oscript.parser.TokenMgrError) )
        {
          return e.getMessage();
        }
        else if( e instanceof java.lang.Exception )
        {
          return e;
        }
        else if( e instanceof Exception )
        {
          // keep   the error format similar, so this basically copies what
          // goes on in Exception::castToString()
          var str = e.getType() + ": " + e.getMessage();
          
          for( var sf : e.stackFrameIterator() )
          {
            var file = sf.getFile();
            if( (file != null) && pkg.util.getFile(file).exists() )
            {
              var editFxn = (function(sf) {  // lisp in objectscript! ;-P  not really needed, but I was bored!
                
                return function() {
                  
                  // if we can determine filename, launch editor:
                  if( sf.getFile() != null )
                    services["editor"].openFileToLineNum( sf.getFile(), sf.getLineNumber() );
                  
                };
              })(sf);
              
              var debugFxn = (function(sf) {
                
                return function() {
                  
                  // launch debugger:
                  services["debugger"].openDebugger(sf);
                  
                };
                
              })(sf);
              
              str += "\n  at " + pkg.output.hyperlink( editFxn, ""+sf ) + " (" + pkg.output.hyperlink( java.awt.Color.green, debugFxn, "debug" ) + ")";
            }
            else
            {
              str += "\n  at " + sf;
            }
          }
          
          str += "\n(click on line in backtrace to launch editor, or \"debug\" to launch the debugger)\n";
          
          return str;
        }
        else
        {
          return e;
        }
      }
      
      /**
       * Open the debugger in the specified scope.
       * 
       * @param sf   the stack frame to debug
       */
      public function openDebugger(sf)
      {
        services["swing worker"].run( function() {
          addResource( new ViewResource( new DebuggerView(sf) ) );
        }, "Launching Debugger" );
      }
      
      /**
       * Set a breakpoint.
       * 
       * @param file         the AbstractFile
       * @param line         the line number
       */
      public function setBreakpoint( file, line )
      {
        file = pkg.util.getFile(file);
        if( !hasBreakpoint(file,line) )
        {
          services["editor"].addGutterIcon( file, line, breakpointGlyph );
          incrementBreakpointViewUse();
          Debugger.setBreakpoint( file, line, bpView.getBreakpointHandler() );
        }
      }
      
      /**
       * Clears a breakpoint.  The <code>null</code> value works as a wildcard,
       * so if <code>line</code> is <code>null</code>, this will remove all
       * breakpoints in the specified file, and if both <code>file</code> and
       * <code>line</code> are <code>null</code>, this will remove all break-
       * points.
       * 
       * @param file         the AbstractFile, or <code>null</code>
       * @param line         the line number, or <code>null</code>
       */
      public function clearBreakpoint( file, line )
      {
        if( file == null )
        {
          for( var file : Debugger.getBreakpointFiles().toArray() )
            callee( file, line );
        }
        else if( line == null )
        {
          file = pkg.util.getFile(file);
          for( var line : Debugger.getBreakpointLines(file).toArray() )
            callee( file, line );
        }
        else if( hasBreakpoint( file=pkg.util.getFile(file), line ) )
        {
          services["editor"].removeGutterIcon( file, line, breakpointGlyph );
          decrementBreakpointViewUse();
          Debugger.setBreakpoint( file, line, null );
        }
      }
      
      /**
       * Determine if there is currently a breakpoint set for the specified file
       * and line number.
       * 
       * @param file         the AbstractFile, or <code>null</code>
       * @param line         the line number, or <code>null</code>
       */
      public function hasBreakpoint( file, line )
      {
        file = pkg.util.getFile(file);
        return Debugger.getBreakpointLines(file).contains(line);
      }
      
    })();
    
  } );
  
  /****************************************************************************/
  /**
   * The debugger view.  This is not used for stepping through code, but rather
   * post-mortum debug, but it shares the DebuggerPanel with the BreakpointView.
   */
  function DebuggerView(sf)
    extends ti.chimera.View( main, (sf.getName() == null) ? "debugger" : ("debugger: " + sf.getName()), null )
  {
    var debuggerPanel = new pkg.debugger.DebuggerPanel(this);
    var console = debuggerPanel.console;
    
    /**
     * Get the component that makes up this view.  The view must always 
     * implement this method.
     * 
     * @return any swing component will do
     */
    public function getComponent()
    {
      return debuggerPanel;
    }
    
    debuggerPanel.enterBreakpoint( sf, sf.getScope(), sf.getFile(), sf.getLineNumber() );
    console.setTabCompleter( new pkg.util.ScriptConsoleTabCompleter( console, sf.getScope() ) );
    
    // create thread to run shell:
    (new (function() extends Thread() {
      public function run()
      {
        var in  = console.getReader();
        var out = console.getWriter();
        services["shell"].run( sf.getScope(), in, out, null );
        
        close();
      }
    })()).start();
  }
  
  
  /****************************************************************************
   * View that is used to implement the breakpoint shell.  When a breakpoint
   * is hit, the breakpoint shell is brought to the front, and gives the
   * user the chance to (s)tep, (c)ontinue, or enter the (d)ebug shell.
   */
  function BreakpointView()
    extends ti.chimera.View( main, "Debugger", null )
  {
    var debuggerPanel = new pkg.debugger.DebuggerPanel(this);
    var console = debuggerPanel.console;
    
    var menuTabCompleter = new function() extends pkg.util.ConsoleTabCompleter(console) {
      
      const var MENU_CHOICES = [
        "exit", "step", "over", "continue", "watch", "debug", "shell", "runto"
      ];
      
      protected function getCompletionSet(buf)
      {
        var completionSet = new pkg.util.CompletionSet();
        completionSet.addAll(MENU_CHOICES);
        return [completionSet,buf];
      }
      
    }();
    console.setTabCompleter(menuTabCompleter);
    
    public const var eventQueue = new ti.swing.SwingQueue();
    
    
    
    /**
     * Get the component that makes up this view.  The view must always 
     * implement this method.
     * 
     * @return any swing component will do
     */
    public function getComponent()
    {
      return debuggerPanel;
    }
    
    /**
     * Called by {@link #close} in order to determine if this view can be
     * closed.  The view is closable when there are no more breakpoints.
     * 
     * @return <code>true</code> to permit the view to be closed, and
     *   <code>false</code> to prevent it
     */
    public function isClosable()
    {
      return nbreakpoints == 0;
    }
    
    /**
     * Called when view is closed.
     */
    addCloseRunnable( function() {
      thread.interrupt();
      am.setEnabled(false);
      bpView = null;
    } );
    
    var in  = new java.io.BufferedReader( console.getReader() );
    var out = new java.io.PrintWriter( console.getWriter(), true );
    
    // create thread to read from console, and append to
    var thread = new (function() extends Thread("read input") {
      
      public function run()
      {
        try
        {
          while(true)
          {
            var input = in.readLine();
            if( (input != null) && !Thread.interrupted() )
              eventQueue.enqueue(input);
          }
        }
        catch(e)
        {}
      }
    })();
    thread.start();
    
    /**
     * Called as we enter a breakpoint... updates UI and stuff..
     */
    function enterBreakpoint( scope, file, line )
    {
      return new (function() {
        
        incrementBreakpointViewUse();
        
        debuggerPanel.enterBreakpoint( null, scope, file, line );
        
        var wasEnabled = am.isEnabled();
        
        am.setEnabled(true);
        console.setEnabled(true);
        
        toForeground();
        
        // the old icon should be the breakpoint icon, unless someone
        // else is changing it too... be nice and don't assume we are
        // the only ones setting it
        services["editor"].addGutterIcon( file, line, programCounterGlyph );
        services["editor"].openFileToLineNum( file, line );
        
        public function restore()
        {
          decrementBreakpointViewUse();
          debuggerPanel.leaveBreakpoint();
          
          services["editor"].removeGutterIcon( file, line, programCounterGlyph );
          am.setEnabled(wasEnabled);
          console.setEnabled(wasEnabled);
        }
        
      })();
    }
    
    /**
     * Called as handler leaves breakpoint, to do any needed cleanup.
     */
    function leaveBreakpoint(ctx)
    {
      ctx.restore();
    }
    
    private function isCurrentView()
    {
      return this == bpView;
    }
    
    /**
     * A single breakpoint is used for all "normal" breakpoints.  A different
     * breakpoint is used to implement "step over" or "run to cursor", which 
     * acts as a logic point to determine at each file:line whether to fwd 
     * the handle() to this bp, or to return immediately.
     */
    var bp = new (function() extends Debugger.Breakpoint() {
      
      /**
       * Called when breakpoint is hit.
       * 
       * @param scope        the current scope of execution
       * @param file         the current file
       * @param line         the current line
       * @return step-breakpoint.  If not <code>null</code>, this will
       *    be called at the next "step" in evaluating the program.  This
       *    can be used to implement single-stepping.
       */
      public function handle( scope, file, line )
      {
        var ctx;
        try 
        {
          ctx = enterBreakpoint( scope, file, line );
          
          // in case the bp-view has been closed and re-opened:
          if( !isCurrentView() )
            return bpView.getBreakpointHandler().handle( scope, file, line );
          
          out.println("\nhit breakpoint: " + file + ":" + line);
          
          function evalWatchExpr(expr)
          {
            var result;
            try {
              result = ev( expr, scope ).castToString();
            } catch(e) {
              result = "error";
            }
            return "  *  " + expr + " => " + result;
          }
          
          var watchExprList = getWatchExprList(scope);
          if( watchExprList.size() > 0 )
          {
            out.println("\nWatched Expressions:");
            out.println("------- -----------");
            for( var itr=watchExprList.iterator(); itr.hasNext(); )
              out.println( evalWatchExpr( itr.next() ) );
            out.println("");
          }
          
          while(true)
          {
            // little trick to make it look like we are reading from a BufferedReader
            // when we are actually just blocking on the eventQueue:
            var in = new (function() {
              public function readLine() {
                return eventQueue.dequeue();
              }
            })();
            
            out.print("(s)tep, step (o)ver, (c)ontinue, (w)atch, (d)ebug shell, or (r)unto: ");
            out.flush();
            
            var input = in.readLine();
            
            /* ignore "exit", which is inserted by the buttons to ensure that the debug
             * shell gets exited before processing the button's input.  If the user is
             * currently in the shell, then the "exit" will be processed by the shell
             * prior to returning to this loop.  Otherwise, we'll get the "exit" here,
             * so just ignore it:
             */
            while( input == "exit" )
              input = in.readLine();
            
            if( (input == "s") || (input == "step") )
            {
              addOverSteppingScope(scope);  // add in case user does "step-over" at end of returning fxn
              return this;
            }
            if( (input == "o") || (input == "over") )
            {
              addOverSteppingScope(scope);
              return getStepOverBp();
            }
            else if( (input == "c") || (input == "continue") )
            {
              removeOverSteppingScope(scope);
              return getStepOverBp();
            }
            else if( (input == "w") || (input == "watch") )
            {
              toForeground();
              out.print("\n  enter watch expression: ");
              out.flush();
              
              var expr = services["shell"].readExpr(in);
              addWatchExpr( scope, expr );
              out.println( evalWatchExpr(expr) );
            }
            else if( (input == "d") || (input == "debug") || (input == "shell") || (input == "debug shell") )
            {
              toForeground();
              var sf = debuggerPanel.getSelectedStackFrame();
              out.println("\nat " + sf);
              console.setTabCompleter( new pkg.util.ScriptConsoleTabCompleter( console, sf.getScope() ) );
              services["shell"].run( sf.getScope(), in, out, null );
              console.setTabCompleter(menuTabCompleter);
            }
            else if( (input == "r") || (input == "runto") )
            {
              toForeground();
              
              var file;
              var line;
              
              while(true)
              {
                out.print("\n  enter file: ");
                out.flush();
                
                var path = in.readLine();
                file = new pkg.File(path);
                
                if( file.exists() )
                  break;
              }
              
              while(true)
              {
                out.print("\n  enter line number: ");
                out.flush();
                
                try
                {
                  line = in.readLine().castToExactNumber();
                  break;
                }
                catch(NoSuchMemberException e)
                {}
              }
              
              return getRunToBp( file, line );
            }
            else
            {
              out.println("unknown command: " + input);
            }
          }
        } 
        catch(e) 
        {
          services["talkback"].uncaughtException( Thread.currentThread(), e );
          return null;
        }
        finally
        {
          leaveBreakpoint(ctx);
        }
      }
    })();
    
    /* 
     * Watch expresions are tracked per-scope, and are active if the current
     * scope is the same as, or a child of, the scope they were defined in.  
     * The active watch are evaluated in the current scope at every step.
     */
    var watchExprTable = new java.util.WeakHashMap();
    
    function addWatchExpr( scope, expr )
    {
      var exprList = watchExprTable.get(scope);
      
      if( exprList == null )
      {
        exprList = new java.util.LinkedList();
        watchExprTable.put( scope, exprList );
      }
      
      exprList.add(expr);
    }
    
    function getWatchExprList(scope)
    {
      var exprList = new java.util.LinkedList();
      
      while( scope != null )
      {
        var list = watchExprTable.get(scope);
        if( list != null )
          exprList.addAll(list);
        scope = __getPreviousScope(scope);
      }
      
      return exprList;
    }
    
    /*
     * A step over breakpoint implements step-over.  It returns right away if
     * the current scope is not one register for step-over, so it will step "over" 
     * calls to other functions, or over child  scope blocks within the same 
     * function.
     */
    
    var overSteppingScopeTable = new java.util.WeakHashMap();
    
    function addOverSteppingScope(scope)
    {
      overSteppingScopeTable.put( scope, true );
    }
    
    function removeOverSteppingScope(scope)
    {
      overSteppingScopeTable.remove(scope);
    }
    
    function isOverSteppingScope(scope)
    {
      return overSteppingScopeTable.get(scope) != null;
    }
    
    function getStepOverBp()  // use this instead of stepOverBp directly, b/c GC could reduce table size
    {
      if( overSteppingScopeTable.size() == 0 )
        return null;
      else
        return stepOverBp;
    }
    
    var stepOverBp = new (function() extends Debugger.Breakpoint() {
      
      /**
       * Called when breakpoint is hit.
       * 
       * @param scope        the current scope of execution
       * @param file         the current file
       * @param line         the current line
       * @return step-breakpoint.  If not <code>null</code>, this will
       *    be called at the next "step" in evaluating the program.  This
       *    can be used to implement single-stepping.
       */
      public function handle( scope, file, line )
      {
        if( isOverSteppingScope(scope) )
          return bp.handle( scope, file, line );
        return getStepOverBp();
      }
    })();
    
    /* 
     * A run-to-cursor breakpoint implements run-to.  It returns right away if
     * program execution is not the specified file:linenumber.
     */
    
    function getRunToBp( file, line )
    {
      return new (function() extends Debugger.Breakpoint() {
        
        /**
         * Called when breakpoint is hit.
         * 
         * @param scope        the current scope of execution
         * @param cfile        the current file
         * @param cline        the current line
         * @return step-breakpoint.  If not <code>null</code>, this will
         *    be called at the next "step" in evaluating the program.  This
         *    can be used to implement single-stepping.
         */
        public function handle( scope, cfile, cline )
        {
          if( (line == cline) && (file.equals(cfile)) )
            return bp.handle( scope, cfile, cline );
          return this;
        }
        
      })();
    }
    
    /**
     */
    public function getBreakpointHandler()
    {
      return bp;
    }
    
  }
  
})() );




/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */
