/******************************************************************************
 *        Copyright Texas Instruments 2002-2004. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * The editor view is an editor window.  The plugin can create multiple editor
 * views if needed.
 * 
 * @param file         the AbstractFile to open
 */
public function EditorView(file)
  extends ti.chimera.View( main, ((file == null) ? "New File" : file.getName()), null )
{
  static var first = true;
  if(first)
  {
    // hack to warn users about java v1.4 bug:
    if( (System.getProperty("os.name").search(/windows/i) != -1) &&
        (System.getProperty("java.version").indexOf("1.4") == 0) &&
        (registry.resolve("/Preferences/Window Manager/Mode").getValue() == "desktop mode") )
      services["prompt"].showWarningMessage("It is highly recommended that you upgrade to<br>Java v5.0, because it fixes a known bug which<br>effects the editor in Desktop mode.");
    first = false;
  }
  
  /**
   * The split-pane is the component for the view.
   */
  var splitPane = new swing.JSplitPane( swing.JSplitPane.HORIZONTAL_SPLIT, true );
  //new swing.JPanel( new java.awt.BorderLayout() );
  
  for( var itr=viewActionList.iterator(); itr.hasNext(); )
    addAction( itr.next() );
  
  /**
   * Get the component that makes up this view.  The view must always 
   * implement this method.
   * 
   * @return any swing component will do
   */
  public function getComponent()
  {
    return splitPane;
  }
  
  /**
   * Called when editor is closed.
   */
  addCloseRunnable( function() {
    
    services["file browser"].removeDragAndDropSupport( editorFrame.getBuffer() );
    
    if( file != null )
      editorTable.remove(file);
    else
      pkg.output.writeln("this may be bad... editor view not in editor table...  better tell rob!");
    
    editorFrame = null;
    
  } );
  
  /**
   * Change the editor to display a different file.
   * 
   * @param newFile      the new file to display
   */
  public function setFile(newFile)
  {
    if( newFile == file )
      return;
    
    saveCursorPosition();
    
    if( newFile.exists() )
    {
      if( editorTable.get(newFile) != null )
        editorTable.get(newFile).close();
      editorTable.remove(file);
      file = newFile;
      editorTable.put( file, this );
      services["file browser"].addToRecentItems(file);
      editorFrame.setFile(file);
      setName( file.getName() );
    }
  }
  
  /**
   * Set the contents of this text editor.
   * 
   * @param text         the new contents of this editor
   */
  public function setText(text)
  {
    // XXX should be like editorFrame.setFile() so it can figure out for
    //     itself what mode to use?
    editorFrame.getBuffer().setText(text);
    editorFrame.setMode( pkg.editor.osMode );
  }
  
  /**
   * Get the contents of this text editor.
   * 
   * @return the string contents
   */
  public function getText()
  {
    return editorFrame.getBuffer().getText();
  }
  
  
  public function getFrame()
  {
    return editorFrame;
  }
  
  private function saveCursorPosition()
  {
    var frame = getFrame();
    var cursorPositionTable = registry.resolve("/Editor/Cursor Position Table").getValue();
    cursorPositionTable.put( 
      file.getPath(), 
      [ frame.getLineNumber(), frame.getColumnNumber() ] 
    );
    registry.resolve("/Editor/Cursor Position Table").setValue(cursorPositionTable);
  }
  
  /**
   * Can this view be closed now?
   * 
   * @return <code>true</code> or <code>false</code>
   */
  public function isClosable()
  {
    var frame = getFrame();
    if( frame != null )
    {
      saveCursorPosition();
      return frame.isClosable();
    }
    return false;
  }
  
  /**
   * The editorFrameContainer provides implements some hooks used by the
   * <code>EditorFrame</code>, such as to set the window's title, or add
   * actions... this should probably be more well developed to include
   * things such as menu-bar-items, etc...
   */
  var editorFrameContainer = new (function() {
    
    /**
     * Set the title of the window/container containing the editor frame.
     */
    public function setTitle(str)
    {
      setName(str);
    }
    
    /**
     * Add an action to the popup menu...
     */
    public function addAction( name, toolTip, smallIcon, bigIcon, fxn )
    {
      pkg.output.logWarning("editorFrameContainer#addAction deprecated");
    }
    
    /**
     * Add a tab to the utility-tab-area
     * 
     * @param name    title for the tab
     * @param fxn     function() that returns a component (which is
     *    lazily created)
     */
    public function addUtilityTab( name, fxn )
    {
      tabbedPane.addTab( name, new swing.JScrollPane( fxn() ) );
      splitPane.setDividerLocation(200);
    }
    
    /**
     * Called to inform the view as to whether the file needs saving or not.
     */
    public function setNeedsSave(needsSave)
    {
      setIcon( needsSave ? needsSaveIcon : null );
    }
    
    /**
     * Called when file associated with this buffer changes
     */
    public var setFile = setFile;
    
    /**
     * Close the container of the view.
     */
    public var close = close;
    
    /**
     * a bit of a hack, but we need a way for the editor to show a file
     * chooser dialog...
     */
    public var chooseFile = chooseFile;
    
  })();
  
  /* 
   * The utility-tab area:
   */
  var tabbedPane = new swing.JTabbedPane( swing.JTabbedPane.BOTTOM );
  splitPane.add(tabbedPane);
  
  /* 
   * The text-area:
   */
  var editorFrame = new pkg.editor.EditorFrame( file, editorFrameContainer );
  splitPane.add(editorFrame);
  
  /**
   * Request focus, and call function after frame has gained focus.  This 
   * is a work-around of sorts, to allow things like scrolling to offset to
   * be performed after the frame is realized.
   */
  public function invokeWhenFrameHasFocus(fxn)
  {
    var frame = getFrame();
    frame.getBuffer().addFocusListener( new function() extends java.awt.event.FocusAdapter() {
      public function focusGained(evt)
      {
        try
          fxn();
        finally
          frame.getBuffer().removeFocusListener(this);
      }
    }() );
    toForeground();
    frame.getBuffer().requestFocus();
  }
  
  var cursorPosition = registry.resolve("/Editor/Cursor Position Table").
    getValue().get( file.getPath() );
  if( cursorPosition != null )
  {
    invokeWhenFrameHasFocus( '{
      var frame = getFrame();
      // XXX not sure about the -1... there is an off by one problem somewhere
      var off = frame.getLineOffset( cursorPosition[0] ) + cursorPosition[1] - 1;
      frame.scrollToOffset(off);
    } );
  }
  
  /* 
   * Attach DnD support to the editorFrame:
   */
  for( var i=0; i<SUPPORTED_FILE_TYPES.length(); i++ )
  {
    services["file browser"].addDragAndDropSupport( SUPPORTED_FILE_TYPES[i], 
                                                    editorFrame.getBuffer(), 
                                                    setFile );
  }
}
