/*=============================================================================
 *     Copyright Texas Instruments 2002. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * The File Browser plugin can create windows with a tree view of the file
 * system.  Actions and DnD (Drag and Drop) support can be registered on a
 * per-file-type basis, to provide integration with other plugins in the
 * system.
 */
registry.register( new (function() extends ti.chimera.Plugin( main, "File Browser" ) {
  
  mixin pkg.system.makePrivatePackage("/jar/chimera/FileBrowserPlugin/");
  
  pkg.registry.setDefaultPersistentValue( 
    "/Dialogs/File Browser/bounds",
    new java.awt.Rectangle(0,0,330,675) 
  );
  
  /**
   * A table mapping component to drop-target-listener.  If one component
   * supports multiple types of files, the same drop-target gets shared
   * by each <code>FileTypeInfo</code>
   */
  var dropTargetListenerTable = new java.util.WeakHashMap();
  
  /**
   * Get the <code>DropTargetListener</code> for the specified component.
   * If one already exists, it will be returned, otherwise a new one
   * will be created.  The will not prevent <code>component</code> from
   * being garbage collected.
   * 
   * @param component   the component that will be a drop target
   * @return the <code>DropTargetListener</code>
   */
  function getDropTargetListener(component)
  {
    // otherwise the drop-target-listener will prevent it from being GC'd:
    component = new WeakReference(component);
    
    var l = dropTargetListenerTable.get(component);
    if( l == null )
    {
      l = new (function() extends java.awt.dnd.DropTargetListener() {
        
        // maps extension to fxn:
        var fxnTable = new java.util.Hashtable();
        
        public function addSupport( extension, fxn )
        {
          fxnTable.put( extension, fxn );
        }
        
        public function dispose()
        {
          fxnTable = new java.util.Hashtable();
        }
        
        public function dragEnter(evt) {}
        public function dragExit(evt)  {}
        public function dragOver(evt)  {}
        public function dropActionChanged(evt) {}
        
        public function drop(evt)
        {
          if( (evt.getSource() != component) && evt.isDataFlavorSupported(fileFlavor) )
          {
            var transferable = evt.getTransferable();
            var file = transferable.getTransferData(fileFlavor);
            var fxn;
            
            if( file.isDirectory() )
              fxn = fxnTable.get("/dir/");
            else
              fxn = fxnTable.get( file.getExtension() );
            
            if( fxn == null )
              fxn = fxnTable.get("/any/");
            
            if( fxn != null )
            {
              evt.acceptDrop( java.awt.dnd.DnDConstants.ACTION_COPY_OR_MOVE );
              fxn(file);
              evt.dropComplete(true);
              return;
            }
          }
          
          evt.rejectDrop();
        }
        
      })();
      dropTargetListenerTable.put( component, l );
    }
    
    return l;
  }
  
  /**
   * An instance of FileTypeInfo is created to track the following per-file-
   * type information:
   * <ul>
   *   <li> Action List
   *   <li> Default (double click) Action
   *   <li> DnD supported?
   * </ul>
   * 
   * @param extension   the file extension for this file type
   */
  function FileTypeInfo( parent, extension )
  {
    var dispatchingAction = false;
    
    /**
     * List of actions, the first action in the list is the default action.
     */
    var actionFxnList = new java.util.LinkedList();
    
    /**
     * The default action, or <code>null</code> if none
     */
    var defaultAction = null;
    
    /**
     * the description of this file type
     */
    var description = null;
    
    /**
     * Get the file extension for this type.
     */
    public function getExtension()
    {
      return extension;
    }
    
    /**
     * Set the description of this file type
     */
    public function setDescription(description)
    {
      this.description = description;
    }
    
    /**
     * Get the description of this file type.
     */
    public function getDescription()
    {
      return description;
    }
    
    /**
     * Get the icon for this type of file.
     * 
     * @return the <code>swing.Icon</code>
     */
    public function getFileIcon()
    {
      var icon = ti.chimera.fs.FileSystemTreeModel.getFileIcon(extension);
      
      if( (icon == null) && (parent != null) )
        icon = parent.getFileIcon();
      
      return icon;
    }
    
    /**
     * Register DnD support for a component that can be the target of
     * a drag-and-drop.
     * 
     * @param component  the component that is the target for the drop
     * @param fxn        the function(file) that is called to handle the drop
     */
    public function addDragAndDropSupport( component, fxn )
    {
      var l = getDropTargetListener(component);
      l.addSupport( extension, fxn );
      new java.awt.dnd.DropTarget( component, l );
    }
    
    /**
     * An action function is called when the popup menu is displayed, and
     * should return an array of actions which are put into the menu being
     * built.
     * <p>
     * When the user triggers the action, the action event will have the 
     * file itself as the source of the event 
     * ({@link java.util.EventObject#getSource}).
     * 
     * @param fxn       the fxn to call when the popup menu is being built
     * @see #setDefaultAction
     */
    public function addActionFxn(fxn)
    {
      actionFxnList.addLast(fxn);
    }
    
    /**
     * Add an action, which will become the default action.  The default
     * action is the one invoked in response to double-click events.  If
     * there is already a default action, it is preserved as an action,
     * but is no longer the default action.  When the user triggers the 
     * action, the action event will have the file itself as the source
     * of the event ({@link java.util.EventObject#getSource}).
     * 
     * @param action    the action to add, should be a swing.Action
     * @see #addAction
     */
    public function setDefaultAction(action)
    {
      defaultAction = action;
    }
    
    /**
     * An swing.Action that is a wrapper for another action, but when
     * the action fires, a copy of the ActionEvent is created with
     * the <code>file</code>s path overriding any existing command-
     * string and the <code>file</code> itself overriding the source
     * of the original event; the copy ActionEvent is passed to the 
     * original action's actionPerformed().  All other methods simply 
     * delegate to the original action.
     */
    function ActionWrapper( file, action )
      extends swing.Action()
    {
      public function actionPerformed(evt)
      {
        if( !dispatchingAction )
        {
          try
          {
            if( action == defaultAction )
              services["file browser"].addToRecentItems(file);
            dispatchingAction = true;
            action.actionPerformed( 
              new java.awt.event.ActionEvent( file,
                                              evt.getID(),
                                              file.getPath(),
                                              evt.getModifiers() )
            );
          }
          finally
          {
            dispatchingAction = false;
          }
        }
      }
      
      public var getValue = action.getValue;
      public var putValue = action.putValue;
      public var isEnabled = action.isEnabled;
      public var setEnabled = action.setEnabled;
      public var addPropertyChangeListener = action.addPropertyChangeListener;
      public var removePropertyChangeListener = action.removePropertyChangeListener;
    }
    
    /**
     * Get the popup menu for the actions for this file type.
     * 
     * @param file     the selected file... this is needed because a
     *    new popup menu has to be created for the specified file in
     *    order for the ActionEvent to have the right command-string
     * @return swing.JPopupMenu
     */
    public function getPopupMenu(file)
    {
      var popup = new swing.JPopupMenu();
      populatePopupMenu( file, popup );
      return popup;
    }
    
    /**
     * Only called from this instance, or children thereof
     */
    public function populatePopupMenu( file, popup )
    {
      if( defaultAction != null )
        popup.add( new ActionWrapper( file, defaultAction ) );
      
      for( var fxn : actionFxnList )
        for( var action : fxn(file) )
          popup.add( new ActionWrapper( file, action ) );
      
      if( parent != null )
        parent.populatePopupMenu( file, popup );
    }
    
    /**
     * Get the named action for the specified file.
     * 
     * @param file   the file that the action would be performed on
     * @param name   the action name
     * @return the action, or <code>null</code>
     */
    public function getAction( file, name )
    {
      // weird, huh?
      
      function check(action)
      {
        if( action.getValue( swing.Action.NAME ) == name )
          throw new ActionWrapper( file, action );
      }
      
      try
      {
        if( defaultAction != null )
          check(defaultAction);
        
        for( var itr=actionFxnList.iterator(); itr.hasNext(); )
          check( itr.next() );
        
        if( parent != null )
          return parent.getAction( file, name );
      }
      catch(ActionWrapper action)
      {
        return action;
      }
      
      return null;
    }
    
    /**
     * Get the default action for the specified file.
     * 
     * @param file   the file that the action would be performed on
     * @return the action, or <code>null</code>
     */
    public function getDefaultAction(file)
    {
      if( defaultAction != null )
        return new ActionWrapper( file, defaultAction );
      else if( parent != null )
        return parent.getDefaultAction(file);
      return null;
    }
  }
  
  /**
   * Table that maps file type (extension) to a FileTypeInfo object.  This
   * is initialized here, rather than initialized and cleaned by a Resource
   * because we don't want to loose registered file type info if we go
   * inactive.
   * <p>
   * Do not access directly, but rather use <code>getFileTypeInfo(file)</code>
   * or for synchronized write access use <code>accessFileTypeInfo</code>.
   */
  var fileTypeInfoTable = new java.util.Hashtable();
  
  /**
   * Helper function to give synchronized access to a FileTypeInfo object,
   * creating a new FileTypeInfo object iff needed.
   */
  function accessFileTypeInfo( extension, fxn )
  {
    synchronized(fileTypeInfoTable)
      fxn( getFileTypeInfoImpl(extension) );
  }
  
  /**
   * Get the <code>FileTypeInfo</code> object, containing per-file-type info,
   * for the specified <code>file</code>.
   * 
   * @param file   the file to get type-info for
   * @return the <code>FileTypeInfo</code> object
   */
  function getFileTypeInfo(file)
  {
    var extension = file.getExtension();
    if( file.isDirectory() )
      extension = "/dir/";
    return getFileTypeInfoImpl(extension);
  }
  
  function getFileTypeInfoImpl(extension)
  {
    synchronized(fileTypeInfoTable)
    {
      var fti = fileTypeInfoTable.get(extension);
      
      if( fti == null )
      {
        // for now, all fti have the parent "/any/":
        fti = new FileTypeInfo( fileTypeInfoTable.get("/any/"), extension );
        fileTypeInfoTable.put( extension, fti );
      }
      
      return fti;
    }
  }
  
  // create "/any/" which is the parent of all types... this needs to be
  // done before creating any other fti
  getFileTypeInfoImpl("/any/");
  
  /**
   * The data flavor used to access the file object itself in a 
   * <code>FileTransferable</code>
   */
  const var fileFlavor = new java.awt.datatransfer.DataFlavor( oscript.fs.AbstractFile, "file" );
  
  /**
   * The data flavor used to access the file path from a
   * <code>FileTransferable</code>
   */
  const var stringFlavor = java.awt.datatransfer.DataFlavor.stringFlavor;
  
  /**
   * The type of transferable used for file drag-and-drops.  It supports
   * <code>fileFlavor</code> (returning the file object itself) and the
   * <code>stringFlavor</code> (return the file path).
   */
  function FileTransferable(file)
    extends java.awt.datatransfer.Transferable()
  {
    public function getTransferData(flavor)
    {
      if( flavor == fileFlavor )
        return file;
      else if( flavor == stringFlavor )
        return "file:" + file.getPath();
      throw new java.awt.datatransfer.UnsupportedFlavorException(flavor);
    }
    
    public function getTransferDataFlavors()
    {
      return [ fileFlavor, stringFlavor ];
    }
    
    public function isDataFlavorSupported(flavor)
    {
      var flavors = getTransferDataFlavors();
      for( var i=0; i<flavors.length(); i++ )
        if( flavors[i] == flavor )
          return true;
      return false;
    }
  }
  
  var browserViewTable = null;
  
  /**
   * Helper, to access a view.  If there is already a view for the specified root
   * path (<code>file</code>), that is returned, else a new view is created.
   */
  function getBrowserView(file)
  {
    synchronized(browserViewTable)
    {
      var view = browserViewTable.get(file);
      
      if( view == null )
      {
        view = new FileBrowserView(file);
        
        var node = registry.resolve("/FileBrowser/openViewSet");
        
        view.addCloseRunnable( function() {
          
          synchronized(browserViewTable)
          {
            browserViewTable.remove(file);
            
            // remove the path from the openViewSet:
            var openViewSet = node.getValue();
            openViewSet.remove( file.getPath() );
            node.setValue(openViewSet);
          }
          
        } );
        
        browserViewTable.put( file, view );
        
        // add the new path to the openViewSet:
        var openViewSet = node.getValue();
        openViewSet.add( file.getPath() );
        node.setValue(openViewSet);
        
        addResource( new ViewResource(view) );
      }
      
      return view;
    }
  }
  
  /**
   * Initialize/cleanup
   */
  addResource( new (function() extends ti.chimera.Resource(true) {
    
    public function install()
    {
      browserViewTable = new java.util.Hashtable();
    }
    
    public function uninstall()
    {
      browserViewTable = null;
    }
    
  })() );
  
  if( !registry.exists("/FileBrowser/openViewSet") )
  {
    var defaultSet = new java.util.HashSet();
    defaultSet.add( (new pkg.File("/")).getPath() );
    
    registry.link( 
      new ti.chimera.registry.PersistentNode( defaultSet, 
                                              new ti.chimera.registry.TypeNodeContract( java.util.Set ),
                                              "set of open views" ),
      "/FileBrowser/openViewSet" );
  }
  if( !registry.exists("/FileBrowser/recentItemList") )
  {
    registry.link( 
      new ti.chimera.registry.PersistentNode( new java.util.LinkedList(), 
                                              new ti.chimera.registry.TypeNodeContract( java.util.List ),
                                              "list of recently opened files" ),
      "/FileBrowser/recentItemList" );
    
    registry.subscribeToValue( "/FileBrowser/recentItemList", null, function( node, recentItemList ) {
      
      for( var itr=recentItemList.iterator(); itr.hasNext(); )
        if( ! (new pkg.fs.File( itr.next() )).exists() )
          itr.remove();
      
      // first remove current menubar entries:
      if( registry.exists("/MenuBar/File/Recent Items") )
        for( var item : registry.resolve("/MenuBar/File/Recent Items").getValue().getChildNames() )
          registry.unlink("/MenuBar/File/Recent Items/" + item);
      
      // decide on menubar entry names... they must be unique:
      var pathToNameTable = new java.util.Hashtable();
      var nameToPathTable = new java.util.Hashtable();
      
      for( var path : recentItemList )
      {
        var name = pkg.fs.basename(path);
        if( nameToPathTable.containsKey(name) )
        {
          var otherName = name;
          var otherPath = nameToPathTable.get(otherName);
          
          // leave old mapping, so we don't re-use that name
          
          var tok = new java.util.StringTokenizer( pkg.fs.normalize(path), "/" );
          var otherTok = new java.util.StringTokenizer( pkg.fs.normalize(otherPath), "/" );
          
          var lastPathElement;
          var lastOtherPathElement;
          while( tok.hasMoreElements() && otherTok.hasMoreElements() )
          {
            lastPathElement = tok.nextElement();
            lastOtherPathElement = otherTok.nextElement();
            
            if( lastPathElement != lastOtherPathElement )
              break;
          }
          
          name = pkg.fs.basename(path) + " (" + lastPathElement + ")";
          otherName = pkg.fs.basename(otherPath) + " (" + lastOtherPathElement + ")";
          
          nameToPathTable[otherName] = otherPath;
          pathToNameTable[otherPath] = otherName;
        }
        
        nameToPathTable[name] = path;
        pathToNameTable[path] = name;
      }
      
      // now create new menubar entries:
      for( var path : recentItemList )
        pkg.util.addMenuBarItem( "File/Recent Items", pathToNameTable[path], 
                                 function(path) { return '{ services["file browser"].open(path); }; }(path) );
      
      windowManager.addMenuBarItem( "/File/Recent Items", null );
      pkg.util.addMenuBarItem( "File/Recent Items", "Clear", 
                               '{ registry.resolve("/FileBrowser/recentItemList").setValue( new java.util.LinkedList() ); } );
    } );
  }
  
  /**
   * Get list of recent items which still exist
   */
  private function getRecentItemList()
  {
    var recentItemList = registry.resolve("/FileBrowser/recentItemList").getValue();
    for( var recentItem : recentItemList.toArray() )
      if( !(new pkg.fs.File(recentItem)).exists() )
        recentItemList.remove(recentItem);
    return recentItemList;
  }
  
  /**
   * The "file browser" service.
   */
  registerServiceFactory( function() {
    
    return new (function() extends ti.chimera.service.FileBrowser() {
      
      /**
       * Open a browser window with the specified path as the root.
       * 
       * @param path    the path to the root node in browser tree
       */
      public function showBrowser(path)
      {
        var view = getBrowserView( pkg.util.getFile(path) );
        view.toForeground();
      }
      
      /**
       * Expand the visible trees to make the specified path visible.
       * 
       * @param path    the file or path
       */
      public function makeVisible(path)
      {
        var file = pkg.util.getFile(path);
        
        for( var e=browserViewTable.elements(); e.hasMoreElements(); )
          e.nextElement().makeVisible(file);
      }
      
      /**
       * Register DnD support for a component that can be the target of
       * a drag-and-drop.
       * <p>
       * An <code>extension</code> value of <code>"/dir/"</code> indicates
       * all files that are directories, regardless of whether they have
       * a specific extension in their name.  A value of <code>"/any/"</code>
       * will apply to all files.
       * 
       * @param extension  the extension for the file type that can be dropped
       * @param component  the component that is the target for the drop
       * @param fxn        the function(file) that is called to handle the drop
       */
      public function addDragAndDropSupport( extension, component, fxn )
      {
        accessFileTypeInfo( extension, function(fti) { fti.addDragAndDropSupport( component, fxn ); } );
      }
      
      /**
       * Remove DnD support for a component, so that the component can be GC'd
       * 
       * @param component  the component that is the target for the drop
       */
      public function removeDragAndDropSupport(component)
      {
        getDropTargetListener(component).dispose();
      }
      
      /**
       * Register the specified component as a drag-source.  The <code>fxn</code>
       * takes the drag-event as a parameter, and returns the file object that
       * is being dragged.
       * 
       * @param component  the component that is the drag source
       * @param fxn        the function(evt) which returns the file
       */
      public function addDragSourceSupport( component, fxn )
      {
        var dragSourceListener = new (function() extends java.awt.dnd.DragSourceListener() {
          // don't really do anything interesting here... I don't think we care about 
          // any of these events:
          public function dragEnter(evt) {}
          public function dragExit(evt)  {}
          public function dragOver(evt)  {}
          public function dropActionChanged(evt) {}
          public function dragDropEnd(evt) {}
        })();
        
        var dragGestureRecognizer = new (function() extends java.awt.dnd.DragGestureListener() {
          
          public function dragGestureRecognized(evt)
          {
            var file = fxn(evt);
            
            if( file != null )
            {
              var dragIcon  = getFileTypeInfo(file).getFileIcon();
              var dragImage = dragIcon.getImage();
              
              evt.startDrag( java.awt.dnd.DragSource.DefaultCopyDrop, 
                             dragImage, new java.awt.Point(),
                             new FileTransferable(file), dragSourceListener );
            }
          }
          
        })();
        
        java.awt.dnd.DragSource.getDefaultDragSource().
          createDefaultDragGestureRecognizer( 
            component,
            java.awt.dnd.DnDConstants.ACTION_COPY_OR_MOVE,
            dragGestureRecognizer 
          );
      }
      
      /**
       * An action function is called when the popup menu is displayed, and
       * should return an array of actions which are put into the menu being
       * built.
       * <p>
       * When the user triggers the action, the action event will have the 
       * file itself as the source of the event 
       * ({@link java.util.EventObject#getSource}).
       * <p>
       * An <code>extension</code> value of <code>"/dir/"</code> indicates
       * all files that are directories, regardless of whether they have
       * a specific extension in their name.  A value of <code>"/any/"</code>
       * will apply to all files.
       * 
       * @param extension  the extension for the file type this action 
       *   applies to
       * @param fxn       the fxn to call when the popup menu is being built
       * @see #setDefaultAction
       */
      public function addActionFxn( extension, fxn )
      {
        accessFileTypeInfo( extension, function(fti) { 
          fti.addActionFxn(fxn); 
        } );
      }
      
      /**
       * Add an action, accessible through a popup menu.  If there no other
       * actions, this will become the default action.  When the user triggers
       * the action, the action event will have the file path as the action-
       * command string ({@link java.awt.event.ActionEvent#getActionCommand}).
       * <p>
       * An <code>extension</code> value of <code>"/dir/"</code> indicates
       * all files that are directories, regardless of whether they have
       * a specific extension in their name.  A value of <code>"/any/"</code>
       * will apply to all files.
       * 
       * @param extension  the extension for the file type this action 
       *   applies to
       * @param action     the action to add, should be anything that can
       *    be put in a {@link #JPopupMenu}
       * @see #setDefaultAction
       */
      public function addAction( extension, action )
      {
        accessFileTypeInfo( extension, function(fti) { 
          fti.addActionFxn( '{ return [action]; } ); 
        } );
      }
      
      /**
       * Add an action, which will become the default action.  The default
       * action is the one invoked in response to double-click events.  If
       * there is already a default action, it is preserved as an action,
       * but is no longer the default action.  When the user triggers the 
       * action, the action event will have the file path as the action-
       * command string ({@link java.awt.event.ActionEvent#getActionCommand}).
       * <p>
       * An <code>extension</code> value of <code>"/dir/"</code> indicates
       * all files that are directories, regardless of whether they have
       * a specific extension in their name.  A value of <code>"/any/"</code>
       * will apply to all files.
       * 
       * @param extension  the extension for the file type this action 
       *   applies to
       * @param action     the action to add, should be a swing.Action
       * @see #addAction
       */
      public function setDefaultAction( extension, action )
      {
        accessFileTypeInfo( extension, function(fti) { fti.setDefaultAction(action); } );
      }
      
      /**
       * Set the description for the specified file type.
       * 
       * @param extension  the extension for the file type this action 
       *   applies to
       * @param description the file description
       */
      public function setDescription( extension, description )
      {
        accessFileTypeInfo( extension, function(fti) { fti.setDescription(description); } );
      }
      
      /**
       * Get the <code>FileTypeInfo</code> for the specified file.  This has
       * information such as the file's icon (<code>getFileIcon()</code>) and
       * popup menu (<code>getPopupMenu(file)</code>).
       * 
       * @param file   the file to get type-info for
       * @return the <code>FileTypeInfo</code> object
       */
      public var getFileTypeInfo = getFileTypeInfo;
      
      /**
       * Get the popup menu to show for the specified file.
       * 
       * @param file   the file associated with the popup
       * @return a JPopupMenu
       */
      public function getPopupMenu(file)
      {
        var fti = getFileTypeInfo(file);
        if( fti != null )
          return fti.getPopupMenu(file);
        return null;
      }
      
      /**
       * Trigger a specified <code>action</code> for the specified  
       * <code>file</code>.  If the <code>action</code> is <code>null</code>
       * then trigger the default action, if any.
       * 
       * @param file   the file associated with the action
       * @param name   the string name of the action to perform (same
       *    as the string in the popup menu)
       */
      public function fireAction( file, name )
      {
        var fti = getFileTypeInfo(file);
        if( fti != null )
        {
          var action = (name == null) ? 
            fti.getDefaultAction(file) : 
            fti.getAction( file, name );
          if( action != null )
          {
            action.actionPerformed(
              new java.awt.event.ActionEvent( file, 0, file.getPath(), 0 )
            );
          }
        }
      }
      
      /**
       * Open the specified <code>file</code>, or if <code>null</code> show
       * file browser to let user pick a file.  The file is opened using
       * the default action for the file type.
       * 
       * @param file   the file to open, or <code>null</code>
       */
      public function open(file)
      {
        if( file == null )
        {
          var chooser = new ti.chimera.FileChooser( windowManager.getDialog("Select File") );
          
          if( file != null )
            chooser.setSelectedFile(file);
          
          var extensions = [];
          for( var extension : fileTypeInfoTable.keySet() )
          {
            if( (extension.length() > 0) && (extension[0] != "/") )  // ignore /any/, /dir/, etc
            {
              var fti = fileTypeInfoTable.get(extension);
              if( fti.getDefaultAction(null) != null )
                extensions.push(extension);
            }
          }
          
          // in the unlikely event that we don't know how to 
          // open any sorts of file, bail out
          if( extensions.length() == 0 )
            return;
          
          chooser.addChoosableFileFilter( new function()  extends ti.chimera.FileChooser.AbstractFileFilter() {
            
            public function accept(f)
            {
              var ext = f.getExtension();
              var name = f.getName();
              if( (name.length() == 0) || (name[0] == ".") )
                return false;
              for( var extension : extensions )
                if( extension == ext )
                  return true;
              return false;
            }
            
            public function getDescription()
            {
              return "All Files (*." + extensions.join(", *.") + ")";
            }
            
          }()  );
          
          for( var extension : extensions )
            chooser.addChoosableFileFilter( new pkg.util.ExtensionFileFilter( extension, fileTypeInfoTable.get(extension).getDescription() ) );
          
          if( chooser.showOpenDialog(null) == ti.chimera.FileChooser.APPROVE_OPTION )
            file = chooser.getSelectedFile();
          else
            return;
        }
        
        file = pkg.util.getFile(file);
        
        var fti = getFileTypeInfo(file);
        
        if( fti != null )
        {
          var action = fti.getDefaultAction(file);
          if( action != null )
            action.actionPerformed( new java.awt.event.ActionEvent( this, 0, "" ) );
        }
      }
      
      /**
       * Get an array of files that were recently opened, in order of most
       * recent first.
       * 
       * @return array of File
       */
      public function getRecentItems()
      {
        return getRecentItemList().toArray();
      }
      
      /**
       * Add a file to the recent items list, potentially removing an old 
       * entry if the list is already full.
       * <p>
       * The is used internally by this plugin, but it can also be used
       * by other plugins in the case that the file was opened by some
       * other mechanism.
       * 
       * @param file   the file that was opened
       */
      public function addToRecentItems(file)
      {
        var recentItemList = getRecentItemList();
        var path = file.getPath();
        
        if( recentItemList.contains(path) )
          recentItemList.remove(path);
        
        while( recentItemList.size() >= 15 )
          recentItemList.remove( recentItemList.size() - 1 );
        
        recentItemList.add( 0, path );
        registry.resolve("/FileBrowser/recentItemList").setValue(recentItemList);
      }
      
    })();
  } );
  
  var filter = new (function() extends ti.chimera.fs.FileSystemTreeModel.FileSystemTreeFilter() {
    
    public function accept(file)
    {
      return ! pkg.cl.isHidden(file);
    }
    
  })();
  
})() );

/* Open paths in openViewSet... recursion used to avoid concurrent modification
 * since showBrowser can cause openViewSet to be modified.
 */
(function(itr) {
  if( itr.hasNext() )
  {
    var file = new pkg.File( itr.next() );
    callee(itr);
    if( file.exists() )
      services["file browser"].showBrowser(file);
  }
})( registry.resolve("/FileBrowser/openViewSet").getValue().iterator() );


/* Add some generic file browser type actions:
 */


services["file browser"].
  setDefaultAction( "jar", new pkg.util.ActionAdapter( "mount", null, null, function(evt) {
    var file = evt.getSource();
    pkg.system.mountJarFile(file);
  } ) );

services["file browser"].setDescription( "jar", "Jar Files" );

services["file browser"].
  addAction( "/dir/", new pkg.util.ActionAdapter( "open new browser", null, null, function(evt) {
    services["file browser"].showBrowser( evt.getSource().getPath() );
  } ) );

services["file browser"].
  addAction( "/dir/", new pkg.util.ActionAdapter( "new directory", null, null, function(evt) {
    var name = services["prompt"].promptForString("enter name for new directory",null,null);
    if( name != null )
    {
      var file = new pkg.File( evt.getSource().getPath() + "/" + name );
      if( file.exists() )
      {
        services["prompt"].showErrorMessage(file.getName() + " already exists");
        return;
      }
      try
        file.mkdir();
      catch(java.io.IOException e)
        services["prompt"].showErrorMessage("could not delete: " + file.getName());
    }
  } ) );

services["file browser"].
  addAction( "/dir/", new pkg.util.ActionAdapter( "new file", null, null, function(evt) {
    var name = services["prompt"].promptForString("enter name for new file",null,null);
    if( name != null )
    {
      try
      {
        var file = new pkg.File( evt.getSource().getPath() + "/" + name );
        if( file.exists() )
        {
          services["prompt"].showErrorMessage(file.getName() + " already exists");
          return;
        }
        file.createNewFile();
        services["file browser"].open(file);
      }
      catch(java.io.IOException e)
      {
        services["prompt"].showErrorMessage("could not create new file: " + file.getName());
      }
    }
  } ) );

if( ! registry.exists("/Preferences/File Browser") )
{
  registry.link( 
    new ti.chimera.registry.PersistentNode( true, 
                                            ti.chimera.registry.NodeContract.BOOLEAN_CONTRACT,
                                            "comfirm delete operation by displaying a dialog" ),
    "/Preferences/File Browser/Confirm Delete?" );
}

services["file browser"].
  addAction( "/any/", new pkg.util.ActionAdapter( "delete", null, null, function(evt) {
    
    var file = evt.getSource();
    
    // if necessary, prompt the user to verify the delete:
    if( registry.resolve("/Preferences/File Browser/Confirm Delete?").getValue() )
      if( "No" == services["prompt"].askQuestion("Are you sure you want to delete: " + file.getName() + "?", ["Yes","No"] ) )
        return;
    
    // and then delete:
    try
      file.delete();
    catch(java.io.IOException e)
      services["prompt"].showErrorMessage("could not delete: " + file.getName());
    
  } ) );



/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */
