/*=============================================================================
 *     Copyright Texas Instruments 2002. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * A view that shows a hierarchial tree view of the file system
 * rooted at the specified root file.
 * 
 * @param root   the AbstractFile that should be shown as the 
 *    root of the tree
 */
public function FileBrowserView(root)
  extends ti.chimera.View( main, function(name){ return (name=="")?"File Browser":name; }( root.getName() ), null )
{
  var fsModel = new ti.chimera.fs.FileSystemTreeModel( root, filter );
  var tree = new swing.JTree(fsModel);
  var scrollPane = new swing.JScrollPane(tree);
  
  /**
   * Get the component that makes up this view.  The view must always 
   * implement this method.
   * 
   * @return any swing component will do
   */
  public function getComponent()
  {
    return scrollPane;
  }
  
  private var popup = null;
  
  public function cancelPopup()
  {
    if( popup != null )
    {
      popup.setVisible(false);
      popup = null;
    }
  }
  
  /* Set-up popup menu:
   */
  var popupTrigger = new ti.chimera.PopupTrigger( new (function()
    extends ti.chimera.PopupTrigger.PopupListener() {
      
      public function showPopup(evt)
      {
        var path = tree.getPathForLocation( evt.getX(), evt.getY() );
        if( path != null )
        {
          var file = path.getLastPathComponent().getFile();
          var fti  = getFileTypeInfo(file);
          
          if( fti != null )
          {
            popup = fti.getPopupMenu(file);
            popup.show( tree, evt.getX(), evt.getY() );
            popup = new WeakReference(popup);
          }
        }
      }
      
    } )() );
  
  tree.addMouseListener(popupTrigger);
  tree.addMouseMotionListener(popupTrigger);
  
  /* Set-up mouse handler for double-click events:
   */
  tree.addMouseListener( new (function() extends java.awt.event.MouseAdapter() {
    
    public function mouseClicked(evt)
    {
      if( evt.getClickCount() > 1 )
      {
        var path = tree.getSelectionPath();
        if( path != null )
        {
          var file = path.getLastPathComponent().getFile();
          if( !file.isDirectory() )
          {
            services["file browser"].open(file);
            evt.consume();
          }
        }
      }
    }
    
  })() );
  
  /* Install tree-cell-renderer so the correct icons are drawn:
   */
  tree.setCellRenderer( new ti.chimera.fs.FileSystemTreeModel.FileSystemTreeCellRenderer() );
  
  /* Install tree-expansion-listener so tree can be updated as there are
   * changes to the underlying filesystem:
   */
  tree.addTreeExpansionListener( new ti.chimera.fs.FileSystemTreeModel.FileSystemTreeExpansionListener() );
  
  /**
   * Make the specified file visible:
   */
  public function makeVisible(file)
  {
    var path = fsModel.getTreePath(file);
System.err.println(file + ": going to expandPath()");
    tree.expandPath(path);
    tree.scrollPathToVisible(path);
  }
  
  /* Set the current directory to be visible:
   */
  makeVisible(root);
  makeVisible( pkg.fs.getCwd() );
  
  /* Setup Drag & Drop (DnD) support:
   */
  services["file browser"].addDragSourceSupport( tree, function(evt) {
    cancelPopup();
    var p = evt.getDragOrigin();
    var path = tree.getPathForLocation( p.x, p.y );
    if( path != null )
      return path.getLastPathComponent().getFile();
    return null;
  } );
  
  addCloseRunnable( '{ 
    // XXX is this needed to get the tree, timer, etc. all cleaned up???
    tree = null;
  } );
}



/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */
