/******************************************************************************
 *     Copyright Texas Instruments, Inc., 2002.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * A "fortune" plugin that can display a random fortune on demand.
 */
registry.register( new (function() extends ti.chimera.Plugin( main, "Fortune" ) {
  
  /**
   * An array of fortune category names.
   */
  var fortuneCategories = null;
  
  /**
   * A utility function to load all the fortunes from the specified
   * file, and return them in an array of strings
   */
  function getFortunes(category)
  {
    var db = new Database("/cache/fortunes/" + category + ".db");
    
    if( db.fortunes == null )
    {
      var file = new pkg.File("/jar/chimera/fortunes/" + category);
      var fortunes = [];
      
      var br = new java.io.BufferedReader( new java.io.InputStreamReader( file.getInputStream() ) );
      
      var fortune = "";
      var line;
      while( (line=br.readLine()) != null )
      {
        if( line == "%" )
        {
          if( fortune != "" )
            fortunes[ fortunes.length() ] = fortune;
          fortune = "";
        }
        else
        {
          fortune += line + "\n";
        }
      }
      
      db.fortunes = fortunes;
      db.flush();
    }
    
    return db.fortunes;
  }
  
  /**
   * Initialize/cleanup
   */
  addResource( new (function() extends ti.chimera.Resource(true) {
    
    /**
     * Install this resource.
     */
    public function install()
    {
      if( fortuneCategories != null )
        return;
      
      fortuneCategories = [];
      
      for( var file : pkg.fs.children("/jar/chimera/fortunes") )
        fortuneCategories[ fortuneCategories.length() ] = file.getName();
    }
    
    /**
     * Uninstall this resource.  This should be the inverse of
     * {@link #install} and so should cleanup anything that was
     * created as a result of the install.
     */
    public function uninstall()
    {
      // don't unload:
      //fortuneCategories = null;
    }
    
  })() );
  
  /**
   * Register a service to allow other plugins to make use of the object
   * browser.
   */
  registerServiceFactory( function() {
    
    return new (function() extends ti.chimera.Service("fortune") {
      
      /**
       * Get a random fortune.
       * 
       * @param category    the category name, or <code>null</code>
       *    to randomly choose a category
       * @return a string
       */
      public function get(category)
      {
        if( category == null )
          category = fortuneCategories[ (java.lang.Math.random() * fortuneCategories.length()).castToExactNumber() ];
        
        var fortunes = getFortunes(category);
        
        if( fortunes == null )
          throw new ProgrammingErrorException("bad category: " + category);
        
        return fortunes[ (java.lang.Math.random() * fortunes.length()).castToExactNumber() ];
      }
      
      /**
       * Show a dialog containing a random fortune.
       * 
       * @param category    the category name, or <code>null</code>
       *    to randomly choose a category
       */
      public function show(category)
      {
        services["prompt"].showInfoMessage( get(category) );
      }
      
      /**
       * Get the names of all the categories
       * 
       * @return an array of strings
       */
      public function getCategories()
      {
        return fortuneCategories;
      }
      
      public function setEnabled(enabled)
      {
        if( enabled && !alreadyEnabled )
        {
          pkg.util.addMenuBarItem( "Plugins", "Fortune",  function(evt) { show(null); } );
          alreadyEnabled = true;
        }
      }
    
    })();
    
  } );
  
  var alreadyEnabled = false;
  
})() );


/**
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

