/******************************************************************************
 *        Copyright Texas Instruments 2002-2004. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * This is the view that is created for browsing help.
 * 
 * @param url          the string URL to the help
 */
public function HelpView(url)
  extends ti.chimera.View( main, "Help", null )
{
  /**
   * The view-pane is the component for the view.
   */
  var viewPane = new swing.JPanel( new java.awt.BorderLayout() );
  
  /**
   * Get the component that makes up this view.  The view must always 
   * implement this method.
   * 
   * @return any swing component will do
   */
  public function getComponent()
  {
    return viewPane;
  }
  
  
  /* 
   * The tool-bar panel:
   */
  var toolBarPanel = new swing.JPanel();
  viewPane.add( toolBarPanel, java.awt.BorderLayout.NORTH );
  
  
  /* 
   * The content window, a split pane.  The RHS is the HTML component, the LHS 
   * the "utility tabs", ie. index tree, history, and perhaps eventually things 
   * like search...
   */
  
  // the html pane, where the actual help content is displayed:
  var htmlPane = new (function() extends pkg.HTMLEditorPane() {
    
    setEditable(false);
    addHyperlinkListener( new (function() extends swing.event.HyperlinkListener() {
      public function hyperlinkUpdate(evt)
      {
        if( evt.getEventType().toString() == "ACTIVATED" )
          navigation.displayPage( evt.getURL() );
      }
    })() );
    setSize( 450, 450 );     // hard code the size to try and fix goofy sizing issues... it seems to ignore the height anyways!
    
  })();
  
  // the utility pane, contains tabs for different "utilities":
  var utilityPane = new swing.JTabbedPane();
  
  viewPane.add( new swing.JSplitPane( swing.JSplitPane.HORIZONTAL_SPLIT,
                                      false,
                                      utilityPane,
                                      new swing.JScrollPane(htmlPane) ),
                java.awt.BorderLayout.CENTER );
  
  
  /**************************************************************************/
  /**
   * Index Implementation.
   */
  var index = new (function () {
    
    var indexTree = new swing.JTree( new pkg.SortedTreeModel(helpTree) );
    
    var selectionModel = new swing.tree.DefaultTreeSelectionModel();
    selectionModel.setSelectionMode( swing.tree.TreeSelectionModel.SINGLE_TREE_SELECTION );
    indexTree.setSelectionModel(selectionModel);
    
    indexTree.addTreeSelectionListener( new (function() extends swing.event.TreeSelectionListener() {
      public function valueChanged(evt)
      {
        var treePathComponents = evt.getPath().getPath();
        var path = null;
        
        // start at idx=1 to skip first component of path "Help"
        for( var i=1; i<treePathComponents.length(); i++ )
        {
          if( path == null )
            path = "" + treePathComponents[i];
          else
            path += "/" + treePathComponents[i];
        }
        
        var helpNode = helpTree.get(path);
        if( helpNode != null )
          navigation.displayPage( pkg.util.getSystemResource( helpNode.getURL() ) );
      }
    })() );
    
    utilityPane.addTab( "Index", new swing.JScrollPane(indexTree) );
    
    /**
     * Actually display a page.  This should really only be called by
     * navigation, otherwise navigation will not be in-sync.
     */
    public function displayPage(url)
    {
      try
      {
        htmlPane.setURL(url);
        
        var title = htmlPane.getTitle();
        if( title != null )
          setName(title);
        else
          setName("Help");
      }
      catch(e)
      {
        services["talkback"].uncaughtException( java.lang.Thread.currentThread(), e );
      }
    }
    
  })();
  
  
  /**************************************************************************/
  /**
   * Navigation Implementation.  Handles back/forward/home.  Makes use of
   * index to actually display a page.
   */
  var navigation = new (function() {
    
    // the history navigation toolbar:
    var tb = new swing.JToolBar();
    
    var backButton    = new pkg.util.ActionAdapter( "Back",    "Go to previous page", pkg.util.getImageIcon("Back24.gif"),
                                           function(evt) { back(); } );
    var indexButton   = new pkg.util.ActionAdapter( "Index",   "Show Index",          pkg.util.getImageIcon("Home24.gif"),
                                           function(evt) { displayPage(null); } );
    var forwardButton = new pkg.util.ActionAdapter( "Forward", "Go to next page",     pkg.util.getImageIcon("Forward24.gif"),
                                           function(evt) { forward(); } );
    
    tb.add(backButton);
    tb.add(indexButton);
    tb.add(forwardButton);
    
    toolBarPanel.add(tb);
    
    
    function NavNode( url, prev )
    {
      public var next = null;           // this is public only for other instances of NavNode
      
      public function getPrev() { return prev; }
      public function getNext() { return next; }
      public function getURL()  { return url; }
      
      if( prev != null )
        prev.next = this;
    }
    
    var navListCurrent = null;
    
    /**
     * Display a URL.  This also manages the history... (well, it will eventually)
     * 
     * @param url          the URL (ie java.net.URL) to load
     */
    public function displayPage(url)
    {
      navListCurrent = new NavNode( url, navListCurrent );
      update();
    }
    
    /**
     * Navigate back to the previous URL.
     */
    public function back()
    {
      navListCurrent = navListCurrent.getPrev();
      update();
    }
    
    /**
     * Navigate forward to the next URL, if there is one.
     */
    public function forward()
    {
      navListCurrent = navListCurrent.getNext();
      update();
    }
    
    /**
     * Handles updating navigation button bar state, and actually loading
     * the current page.
     */
    function update()
    {
      backButton.setEnabled( navListCurrent.getPrev() != null );
      forwardButton.setEnabled( navListCurrent.getNext() != null );
      
      index.displayPage( navListCurrent.getURL() );
    }
    
  })();
  
  navigation.displayPage( pkg.util.getSystemResource(url) );
  
  addAction( new pkg.util.ActionAdapter( "Help", null, pkg.util.getImageIcon("Help16.gif"),
                                function(evt) { services["help"].displayHelp("Plugins/Help"); } ) );
}
