/*=============================================================================
 *     Copyright Texas Instruments, Inc., 2001.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * The object-browser plug-in is a plugin that provides a way to browse an
 * object, and it's javadoc comments. 
 */
registry.register( new (function() extends ti.chimera.Plugin( main, "Object Browser" ) {
  
  /**
   * Used to provide access to an object's members.
   */
  var debugger = null;
  
  
  /**
   * Initialize/cleanup
   */
  addResource( new (function() extends ti.chimera.Resource(true) {
    
    public function install()
    {
      if( debugger == null )
        debugger = new Debugger();
    }
    
    public function uninstall()
    {
    }
    
  })() );
  
  /**
   * Register a service to allow other plugins to make use of the object
   * browser.
   */
  registerServiceFactory( function() {
    
    return new (function() extends ti.chimera.Service("Object Browser") {
      
      function launch( obj, str )
      {
        services["swing worker"].run( function() {
          addResource( new ViewResource( new ObjectBrowserView( obj, str ) ) );
        }, "Launching Object Browser" );
      }
      
      /**
       * Launch an object-browser.
       * 
       * @param obj          the object to browse
       */
      public function browse(obj)
      {
        launch( obj, "Object: " + toStr(obj) );
      }
      
      /**
       * Launch object-browser to browse the named service.
       * 
       * @param name         the name of the service to browse
       */
      public function browseService(name)
      {
        launch( services[name], "Service: " + name );
      }
      
      /**
       * Launch object-browser to browse all services.  Currently
       * this just opens a browser view per service, but eventually
       * this may do something cooler like open a single view, and
       * have each service as a child of the root... or something.
       */
      public function browseServices()
      {
        services["swing worker"].run( function() {
          for( var service : registry.resolve("/Services").getValue().getChildNames() )
            browseService(service);
        }, "Launching Object Browser" );
      }
      
    })();
    
  } );
  
  
  /**
   * The view that implements the object browsing.
   * 
   * @param obj          the object to browse
   * @param name         the name of the view
   */
  function ObjectBrowserView( obj, name )
    extends ti.chimera.View( main, name, null )
  {
    /**
     * keep a table of objects in tree, to prevent circular references
     * from blowing things up.
     */
    var objTable = new java.util.Hashtable();
    
    /**
     * The tree of the object, and it's properties.
     */
    function ObjectTreeNode( obj, name )
      extends pkg.SortedTreeNode( /*XXX name*/ name.castToString() )
    {
      // for now, only show public members... in future which should
      // show private to, with a "private" icon or something... 
      for( var e=debugger.enumerateMembers(obj); e.hasMoreElements(); )
      {
        var memberName = e.nextElement();
        var member     = debugger.getMemberAccessor( obj, memberName );
        var val        = member.getValue();
        var attr;
        
        // TODO, choose spiffy icons for public vs. private..
        if( (member.getAttr() & oscript.data.Reference.ATTR_PUBLIC) != 0 )
          attr = "public";
        else
          attr = "private";
        
        if( (val != null) && 
            !isUndefined(val) &&
            !(obj instanceof Function) &&
            (objTable.get(val) == null) )
        {
          objTable.put( val, val );
          insert( new ObjectTreeNode( val, memberName + " (" + attr + ")" ) );
        }
      }
      
      /**
       * Get the object this node represents.
       */
      public function getObject()
      {
        return obj;
      }
    }
    
    
    /* 
     * The UI:
     */
    var tree = new ObjectTreeNode( obj, toStr(obj) );
    
    var objTree     = new swing.JTree( new pkg.SortedTreeModel(tree) );
    var docTextPane = new pkg.HTMLEditorPane();
    var splitPane   = new swing.JSplitPane( swing.JSplitPane.HORIZONTAL_SPLIT,
                                            new swing.JScrollPane(objTree),
                                            new swing.JScrollPane(docTextPane) );
    
    // we only want to be able to select a single node at a time:
    var selectionModel = new swing.tree.DefaultTreeSelectionModel();
    selectionModel.setSelectionMode( swing.tree.TreeSelectionModel.SINGLE_TREE_SELECTION );
    objTree.setSelectionModel(selectionModel);
    
    // setup listener to handle change in selected node in tree:
    objTree.addTreeSelectionListener( new (function() extends swing.event.TreeSelectionListener() {
      public function valueChanged(evt)
      {
        var path = objTree.getSelectionPath();
        var node = path.getLastPathComponent();
        setNode(node);
      }
    })() );
    
    
    /**
     * Set the currently selected node...
     */
    function setNode(node)
    {
      var obj = node.getObject();
      var cmt = null;
      
      if( node == tree )
        obj = obj.getType();
      
      if( obj instanceof Function )
        cmt = obj.getComment();
      
      if( cmt == null )
        cmt = "no comment";
      
      docTextPane.setText(cmt);
    }
    
    // initialize help pane:
    setNode(tree);
    
    /**
     * Get the component that makes up this view.  It is always a Box
     * component, but the contents of the box depend on the state of
     * the plugin...
     */
    public function getComponent()
    {
      return splitPane;
    }
  }
  
  
  /**
   * utility to convert to string... XXX
   */
  function toStr(obj)
  {
    // XXX hack, because JavaClass still needs a getName() method...
    var type = obj.getType();
    var val;
    if( type instanceof Function )
      val = type.getName();
    else
      val = "java type";
    
    if( (val == null) || isUndefined(val) )
      val = "anon";
    
    return val;
  }
  
  // XXX hack... need a way to test for undefined
  function isUndefined(obj)
  {
    return obj == oscript.data.Value.UNDEFINED;
  }
})() );






/**
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

