/*=============================================================================
 *     Copyright Texas Instruments, Inc., 2001.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * The output-console is a plugin that provides services to display stderr and
 * stdout output.  Then the stderr and stdout output is remapped, using an
 * {@link java.io.OutputStream} that makes use of these services to display
 * output.  This way the plugin is started on demand.  The provided services
 * also create the needed view on demand.
 */
registry.register( new (function() extends ti.chimera.Plugin( main, "Output" ) {
  
  /**
   * Keep track of the instances of the views... the OutputConsoleView class
   * handles keeping these updated when the view is created/destroyed.
   */
  var viewTable;
  
  /**
   * Character converter...
   */
  var btcc;
  
  /**
   * Initialize/cleanup
   */
  addResource( new (function() extends ti.chimera.Resource(true) {
    
    public function install()
    {
      btcc = new ((new JavaPackage("sun.io")).ByteToCharUTF8)();
      viewTable = new java.util.Hashtable();
    }
    
    public function uninstall()
    {
      btcc = null;
      viewTable = null;
    }
    
  })() );
  
  /**
   * A resource used to install/uninstall the output console plugin
   * streams.
   */
  var activeResource = new (function() extends ti.chimera.Resource(false) {
    
    private var oldStdout = null;
    private var oldStderr = null;
    
    /**
     * Construct a new output stream which directs output both to
     * the original stream, and to the service specified by
     * <code>name</code>.
     * 
     * @param name    the service name, ie "stdout" or "stderr"
     * @param origOut the original output stream
     * @return an output stream
     */
    private function makeOutputStream( name, origOut )
    {
      return new (function() extends java.io.ByteArrayOutputStream() {
        
        public function flush()
        {
          synchronized(this)
          {
            var str = toString();
            origOut.print(str);
            origOut.flush();
            if( registry.resolve("/Window Manager/visible").getValue() )
              services[name].write(str);
            reset();
          }
        }
        
      })();
    }
    
    /**
     * Called when this resource is installed... remaps stderr and
     * stdout to our own output streams
     */
    public function install()
    {
      oldStdout = System.out;
      oldStderr = System.err;
      System.setErr( new java.io.PrintStream( makeOutputStream( "stderr", oldStderr ), true ) );
      System.setOut( new java.io.PrintStream( makeOutputStream( "stdout", oldStdout ), true ) );
    }
    
    /**
     * Called when this resource is uninstalled... returns stderr
     * and stdout to their original mappings.
     */
    public function uninstall()
    {
      System.setErr(oldStdout);
      System.setOut(oldStderr);
      oldStdout = oldStderr = null;
    }
    
  })();
  
  /**
   * Output console view... this is where we end up writing output to, after a
   * bit of redirection.
   * 
   * @param name         the name of the output view
   */
  function OutputConsoleView(name)
    extends ti.chimera.View( main, name, null )
  {
    private var console = pkg.util.makeConsole( this, 150, 80, true );
    
    /**
     * This view has a single component, the console.
     */
    public function getComponent()
    {
      return console;
    }
    
    /**
     * Used by the service.
     */
    public var writer = new ti.io.TeeWriter( console.getWriter(),
                                             new java.io.FileWriter(name + ".txt") );
  }
  
  /**
   * Base class for the output services.
   */
  function OutputService(name)
    extends ti.chimera.Service(name)
  {
    /**
     * Write a string to the output console window.
     * 
     * @param str   the string to write
     */
    public function write(str)
    {
      var w = getWriter();
      w.write(str);
      w.flush();
    }
    
    /**
     * Get the proper writer, depending on the name of the service.
     */
    function getWriter()
    {
      var view = viewTable.get(name);
      
      if( view == null )
      {
        services["swing worker"].run( function() {
          view = new OutputConsoleView(name);
          addResource( new ViewResource(view) );
          viewTable.put( name, view );
        }, "opening " + name + " console" );
      }
      
      return view.writer;
    }
  }
  
  /*
   * The "stderr" and "stdout" services provided by this plugin.  By having the 
   * output stream make use of these services, the {@link ti.chimera.Registry}
   * will automagically handling starting the plugin on demand.
   */
  registerServiceFactory( function() { return new OutputService("stderr"); } );
  registerServiceFactory( function() { return new OutputService("stdout"); } );
  
  /* 
   * The "output console" service is used for (de)enabling the output
   * console plugin.  When activated, stdout/stderr are remapped, and
   * when deactivated they are returned to their original mapping
   */
  registerServiceFactory( function() {
    
    return new (function() extends ti.chimera.Service("output console") {
      
      /**
       * Enable/disable the output console feature
       * 
       * @param enabled   <code>true</code> to enable, 
       *    <code>false</code> to disable
       */
      public function setEnabled(enabled)
      {
        (enabled ? addResource : removeResource)(activeResource);
      }
      
    })();
    
  } );
  
})() );


/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

