/******************************************************************************
 *     Copyright Texas Instruments, Inc., 2002.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * The pager plugin creates multiple virtual workspaces, and manages them
 * by showing/hiding dialogs when the user switches between workspaces.  It
 * also supports a "sticky layer", so any dialogs that are made sticky are
 * visible in any workspace.  The names of the sticky dialogs are stored in
 * a persistent node in the registry.
 */
registry.register( new (function() extends ti.chimera.Plugin( main, "Pager" ) {
  
  var toolBar;
  var toolBarButtonPanel;
  
  const var WORKSPACE_NAMES = [ "One", "Two", "Three", "Four" ];
  
  if( !registry.exists("/Pager/sticky set") )
  {
    var node = 
      new ti.chimera.registry.PersistentNode( new java.util.HashSet(),
                                               new ti.chimera.registry.TypeNodeContract(java.util.Set),
                                               "the list of names of dialogs that are sticky" );
    registry.link( node, "/Pager/sticky set" );
  }
  
  /**
   * The registry-resource handles cleaning up subscribers/nodes for us
   * when the plugin becomes inactive.  It is also the unmanaged resource
   * that makes us active/inactive by adding/removeing it as a resource.
   * <p>
   * Because we have special requirements on the install/uninstall order
   * we don't actually directly install this resource, but rather it is
   * installed/uninstalled via pagerResource
   */
  var rr = new pkg.registry.RegistryResource(true);
  
  /**
   * Initialize/cleanup
   */
  var pagerResource = new (function() extends ti.chimera.Resource(false) {
    
    var stickAction;
    var unstickAction;
    
    function switchToWorkspace(name)
    {
      rr.unlink("/Pager/current");
      rr.link( registry.resolve("/Pager/workspaces/" + name), "/Pager/current" );
    }
    
    /**
     * Install this resource.
     */
    public function install()
    {
      rr.install();
      
      var focusDialog = null;    // the dialog that is currently in focus
      var stickySet = null;
      
      rr.subscribeToValue(
        "/Pager/sticky set", null,
        function( node, value ) {
          stickySet = value;
      } );
      
      var toolBarPanel = new swing.JPanel( new java.awt.GridLayout(0,1) );
//      toolBarPanel.add( new swing.JLabel("go to workspace:") );
      toolBarButtonPanel = new swing.JPanel();
      toolBarPanel.add(toolBarButtonPanel);
      
      toolBar.add(toolBarPanel);
      
      rr.mkdir("/Pager/workspaces");
      rr.mkdir("/Pager/sticky");
      
      for( var i=0; i<WORKSPACE_NAMES.length(); i++ )
      {
        rr.mkdir("/Pager/workspaces/" + WORKSPACE_NAMES[i]);
        rr.mkdir("/MenuBar/Window/" + WORKSPACE_NAMES[i]);
      }
      
      windowManager.addMenuBarItem( "/Window", null );
      
      pkg.registry.applyFunction( 
        "/Pager/workspaces", "/MenuBar/Window", rr, false,
        function( srcNode, dstNode, val ) {
          var l = new java.util.LinkedList();
          l.add( new pkg.util.ActionAdapter( 
            "foo", null, null,
            function(evt) {
              var workspaceName = registry.basename( registry.dirname( dstNode.getPrimaryPath() ) );
              switchToWorkspace(workspaceName);
              val.toFront();
            } ) );
          return l;
        } );
      
      stickAction = new pkg.util.ActionAdapter( "Stick", null, null, function(evt) {
        
        // move from current -> sticky:
        if( focusDialog != null )
        {
          var name = focusDialog.getTitle();
          var node = registry.resolve("/Pager/current/" + name);
          registry.unlink("/Pager/current/" + name);
          registry.link( node, "/Pager/sticky/" + name );
          refresh();
        }
        
      } );
      
      unstickAction = new pkg.util.ActionAdapter( "Unstick", null, null, function(evt) {
        
        // move from sticky -> current:
        if( focusDialog != null )
        {
          var name = focusDialog.getTitle();
          var node = registry.resolve("/Pager/sticky/" + name);
          registry.unlink("/Pager/sticky/" + name);
          registry.link( node, "/Pager/current/" + name );
          refresh();
        }
        
      } );
      
      windowManager.addMenuBarItem( "Window", stickAction );
      windowManager.addMenuBarItem( "Window", unstickAction );
      
      rr.subscribeToValue( 
        "/Docks/selected dock/dialog", null,
        function( node, dialog ) {
          focusDialog = dialog;
          refresh();
      } );
      
      function refresh()
      {
        var isSticky = registry.exists("/Pager/sticky/" + focusDialog.getTitle());
        
        stickAction.setEnabled( !isSticky );
        unstickAction.setEnabled( isSticky );
      }
      
      rr.link( registry.resolve("/Pager/workspaces/" + WORKSPACE_NAMES[0]), "/Pager/current" );
      
      /**
       * subscriber to workspaces directory updates buttons on
       * tool-bar
       */
      rr.subscribeToValue( "/Pager/workspaces", null, (function() {
        
        var lastDt = null;
        var buttonTable = new java.util.Hashtable();
        var buttonGroup = new swing.ButtonGroup();
        
        return function( node, newDt ) {
          
          if( lastDt != null )
            for( var removed : lastDt.notIn(newDt) )
              toolBarButtonPanel.remove( buttonTable.remove(removed) );
          
          var selected = true;
          
          for( var added : newDt.notIn(lastDt) )
          {
            var button = new swing.JToggleButton( added, selected );
            buttonTable.put( added, button );
            button.addActionListener( function(evt) { switchToWorkspace(added); } );
            buttonGroup.add(button);
            toolBarButtonPanel.add(button);
            selected = false;
          }
          
          lastDt = newDt;
          
        };
        
      })() );
      
      /**
       * subscriber to workspace directory makes dialogs hidden/visible
       * as the are added/removed from the workspace.  This is used for
       * both the current ("/Pager/current") and sticky ("/Pager/sticky")
       * workspaces, and also works when the current workspace is changed.
       */
      function makeWorkspaceSubscriber()
      {
        var lastDt = null;
        
        return function( node, newDt ) {

          if( lastDt != null )
            for( var removed : lastDt.notIn(newDt) )
              lastDt.get(removed).getValue().setVisible(false);
          
          for( var added : newDt.notIn(lastDt) )
            newDt.get(added).getValue().setVisible(true);
          
          lastDt = newDt;
          
        };
      }
      
      rr.subscribeToValue( "/Pager/current", null, makeWorkspaceSubscriber() );
      rr.subscribeToValue( "/Pager/sticky",  null, makeWorkspaceSubscriber() );
      
      rr.subscribeToValue( "/Pager/sticky", null, (function() {
        
        var lastDt = null;
        
        return function( node, newDt ) {
          
          if( stickySet == null )
            return;
          
          if( lastDt != null )
            for( var removed : lastDt.notIn(newDt) )
              stickySet.remove(removed);
          
          for( var added : newDt.notIn(lastDt) )
            stickySet.add(added);
          
          lastDt = newDt;
          
          // ensure value gets preserved:
          registry.resolve("/Pager/sticky set").setValue(stickySet);
          
        };
        
      })() );
      
      /**
       * subscriber to "/Dialogs" listen for dialogs being created/removed
       * and add new dialogs to the current workspace, and when dialogs
       * are removed, removes them from whatever workspace they are in.
       */
      rr.subscribeToValue("/Dialogs", null, (function() {
        
        var lastDt = null;
        var srcPathSubscriberTable = new java.util.Hashtable();
        
        return function( node, newDt ) {
          
          if( stickySet == null )
            return;
          
          for( var added : newDt.notIn(lastDt) )
          {
            var basePath = "/Dialogs/" + added;
            var srcPath  = basePath + "/instance";
            
            /* we can't assume that srcPath has been created yet, and blocking
             * the registry thread would be bad, so we must subscribe to srcPath
             */
            var srcPathSubscriber;
            registry.subscribeToCreation( srcPath, srcPathSubscriber = function(node) {
              
              synchronized(registry)
              {
                if( registry.exists(srcPath) )
                {
                  var dstPath;
                  
                  if( (stickySet != null) && stickySet.contains(added) )
                    dstPath = "/Pager/sticky/" + added;
                  else
                    dstPath = registry.resolve("/Pager/current/").getPrimaryPath() + "/" + added;
                  
                  rr.link( registry.resolve(srcPath), dstPath );
                }
              }
              
            } );
            
            srcPathSubscriberTable.put( added, srcPathSubscriber );
            
          }
          
          if( lastDt != null )
          {
            for( var removed : lastDt.notIn(newDt) )
            {
              // first look in workspaces...
              for( var workspace : registry.resolve("/Pager/workspaces").getValue().getChildNames() )
              {
                var path = "/Pager/workspaces/" + workspace + "/" + removed;
                if( registry.exists(path) )
                {
                  rr.unlink(path);
                  break;
                }
              }
              
              // ... then look in sticky layer
              var path = "/Pager/sticky/" + removed;
              if( registry.exists(path) )
                rr.unlink(path);
              
              registry.unsubscribeFromCreation( srcPathSubscriberTable.get(removed) );
            }
          }
          
          lastDt = newDt;
          
        };
        
      })() );
      
    }
    
    /**
     * Uninstall this resource.  This should be the inverse of
     * {@link #install} and so should cleanup anything that was
     * created as a result of the install.
     */
    public function uninstall()
    {
      rr.unlink("/Pager/current");
      rr.uninstall();
      
      windowManager.removeMenuBarItem( "Window", stickAction );
      windowManager.removeMenuBarItem( "Window", unstickAction );
      
      stickAction = unstickAction = null;
      
      /* make all dialogs visible (XXX should track which ones *should* be visible
       * because a dialog may be created by not yet made visible by its creator)
       */
      for( var itr=registry.resolve("/Dialogs").getValue().getChildNames(); itr.hasNext(); )
      {
        try {
          registry.resolve("/Dialogs/" + itr.next() + "/instance").getValue().setVisible(true);
        } catch(e) {
          // ignore b/c dialog might be in the process of being created
        }
      }
      
      toolBar = null;
      toolBarButtonPanel = null;
    }
    
  })();
  
  addResource( new ToolBarResource( function() { return toolBar = new swing.JToolBar(); }, true ) );
  
  /**
   * Register a service to allow other plugins to make use of the object
   * browser.
   */
  registerServiceFactory( function() {
    
    return new (function() extends ti.chimera.Service("pager") {
      
      /**
       * Start the pager.
       */
      public function start()
      {
        addResource( pagerResource );
      }
      
      /**
       * Exit the pager.  All dialogs are made visible.
       */
      public function stop()
      {
        removeResource( pagerResource );
      }
      
      public function setEnabled(enabled)
      {
        if(enabled)
        {
          wasEnabled = true;
          start();
        }
        else
        {
          //stop();
          if(wasEnabled)
            services["prompt"].showErrorMessage("Stopping the pager is broken; this change will take effect when you restart.");
        }
      }
      
    })();
    
  } );
  
  var wasEnabled = false;
  
})() );


/**
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

