/*=============================================================================
 *     Copyright Texas Instruments, Inc., 2001.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * Implements the "prompt" service by displaying popup dialogs.
 */
registry.register( new (function() extends ti.chimera.Plugin( main, "Prompt" ) {
  
  /**
   * Table maps dialog type, to the box that forms the contents of the dialog.  If
   * another dialog of the same type is requested, we just add to the box, rather
   * than creating a new dialog.
   */
  var boxTable = new java.util.Hashtable();
  
  /**
   * Register an implementation of the "prompt" service.
   */
  registerServiceFactory( function() {
    
    return new (function() extends ti.chimera.service.Prompt() {
      
      /**
       * All the dialogs created use this special subclass of Box, which
       * automatically pack()s and center()s the dialog once you add a
       * new component.
       */
      private function DialogBox( type, dialog )
        extends swing.Box( swing.BoxLayout.Y_AXIS )
      {
        private var _add = add;
        public function add(args...)
        {
          _add.callAsFunction(args);
          dialog.pack();
          dialog.center();
          dialog.setVisible(true);
        }
        
        private var _remove = remove;
        public function remove(args...)
        {
          _remove.callAsFunction(args);
          if( getComponentCount() == 0 )
            dispose();
        }
        
        public function dispose()
        {
          removeAll();
//          synchronized(boxTable)
          {
            setVisible(false);
            dialog.setVisible(false);
            dialog.dispose();
            boxTable.remove(type);
          }
        }
        
        boxTable.put( type, this );
      }
      
      /**
       * A utility that converts the message to html
       */
      private function convertMsg(msg)
      {
        // convert "\n" to "<br>"
        var idx;
        while( (idx=msg.indexOf("\n")) != -1 )
          msg = msg[0..idx-1] + "<br>" + msg[idx+1..msg.length()-1];
        return "<html>" + msg + "</html>";
      }
      
      /**
       * A utility that displays a modal dialog of the specified type
       * with the specified message.
       */
      private function showMsg( type, title, msg )
      {
        if( windowManager.isVisible() )
        {
          var label = new swing.JLabel( convertMsg(msg) );
          
          var box;
          var isNew = false;
          var pane;
          
          // creating the DialogBox must be sync'd on the boxTable, 
          // because the DialogBox constructor is what adds the new
          // entry to the table:
          synchronized(boxTable)
          {
            box = boxTable.get(type);
            if( box == null )
            {
              var dialog = windowManager.getDialog(title);
              
              box = new DialogBox( type, dialog );
              pane = new ti.chimera.OptionPane( dialog, box, type );
              
              dialog.addCloseRunnable( '{
                synchronized(box)
                {
                  box.setVisible(false);
                  box.dispose();
                  box.notifyAll();
                }
              } );
              
              isNew = true;
            }
          }
          
          swing.SwingUtilities.invokeLater( '{
            box.add(label);
          } );
          
          if(isNew)
          {
            pane.showModal();
          }
          else
          {
            services["swing worker"].run( '{
              synchronized(box)
                while( box.isVisible() )
                  box.wait(100);
            }, null );
          }
        }
        else
        {
          writeln(msg);
        }
      }
      
      /**
       * Display an error message to the user.  This method blocks until the user
       * dismisses the message.
       * 
       * @param msg          the error message to display
       */
      public function showErrorMessage(msg)
      {
        main.error(msg);
        showMsg( swing.JOptionPane.ERROR_MESSAGE, "Error", msg );
      }
      
      /**
       * Display a warning message to the user.  This method blocks until the user
       * dismisses the message.
       * 
       * @param msg          the warning message to display
       */
      public function showWarningMessage(msg)
      {
        main.warning(msg);
        showMsg( swing.JOptionPane.WARNING_MESSAGE, "Warning", msg );
      }
      
      /**
       * Display an info message to the user.  This method blocks until the user
       * dismisses the message.
       * 
       * @param msg          the info message to display
       */
      public function showInfoMessage(msg)
      {
        showMsg( swing.JOptionPane.INFORMATION_MESSAGE, "Information", msg );
      }
      
      /**
       * Display a progress message to the user.  A progress message has a progress
       * bar to display progress.  This returns a {@link Progress} to allow the
       * client code to programatically update the progress, and dispose of the
       * progress message once complete.
       * <p>
       * The progress displayed to the user is initially in "indeterminate" mode
       * to indicate that the length of time to take is unknown.  Once the length
       * of time is known, {@link Progress#setRange} and {@link Progress#setValue}
       * can be used to update the progress display.
       * <p>
       * For example:
       * <pre>
       *   var progress = services["prompt"].showProgress("This will take a while");
       *   
       *   // progress is in "indeterminate" mode until you compute length of time:
       *   progress.setRange( 0, computeTime() );
       *   
       *   var t = currentTimeMillis();
       *   while( !done() )
       *   {
       *     doSomeWork();
       *     p.setValue( currentTimeMillis() - t );
       *   }
       *   
       *   // done, so dismiss progress dialog:
       *   progress.dispose();
       * </pre>
       * 
       * @param msg          the message to display
       */
      public function showProgress(msg)
      {
        synchronized(boxTable)
          var box = boxTable.get("prompt");
        
        if( box == null )
        {
          var dialog = windowManager.getDialog("Progress");
          
          box = new DialogBox( "prompt", dialog );
          
          var pane = new ti.chimera.OptionPane(
            dialog, box, 
            swing.JOptionPane.INFORMATION_MESSAGE,
            swing.JOptionPane.DEFAULT_OPTION,
            null, []
          );
          
          dialog.pack();
          dialog.setVisible(true);
        }
        
        var progressBox = new swing.Box( swing.BoxLayout.Y_AXIS );
        var progress = new swing.JProgressBar();
        progress.setIndeterminate(true);
        
        function setMessage(msg)
        {
          swing.SwingUtilities.invokeLater( '{
            progressBox.removeAll();
            progressBox.add( new swing.JLabel( convertMsg(msg) ) );
            progressBox.add(progress);
          } );
        }
        
        setMessage(msg);
        
        swing.SwingUtilities.invokeLater( function() {
          box.add(progressBox);
        } );
        
        return new (function() extends ti.chimera.service.Prompt.Progress() {
          
          /**
           * Change the progress message that is shown to the user
           * 
           * @param msg    the new message to display
           */
          public var setMessage = setMessage;
          
          /**
           * Update the range of this display.  The value (see {@link #setValue})
           * must be between <code>min</code> and <code>max</code> inclusive.
           * 
           * @param min          the min
           * @param max          the max
           */
          public function setRange( min, max )
          {
            swing.SwingUtilities.invokeLater( function() {
              
              progress.setMinimum(min);
              progress.setMaximum(max);
              
              var val = getValue();
              if( val > max )
                setValue(max);
              else if( val < min )
                setValue(val);
              
            } );
          }
          
          /**
           * Update the progress.  The <code>val</code> should be between the
           * min and max (inclusive) specified in {@link #setRange}.  It is
           * an error to call this if {@link #setRange} has not been called.
           * 
           * @param val          the new value
           */
          public function setValue(val)
          {
            swing.SwingUtilities.invokeLater( function() {
              progress.setValue(val);
            } );
          }
          
          /**
           * Get the current progress value.
           * 
           * @return the current value
           */
          public function getValue()
          {
            return progress.getValue();
          }
          
          /**
           * Indicate that the progress has completed, and the progress dialog or
           * message should no longer be displayed.
           */
          public function dispose()
          {
            swing.SwingUtilities.invokeLater( function() {
              box.remove(progressBox);
            } );
          }
          
        })();
      }
      
      /**
       * Ask a question of the user.  The <code>msg</code> is displayed to the
       * user, who must then choose one of the choices.  The choice selected
       * by the user is returned.  This method blocks until a result is chosen.
       * For example:
       * <pre>
       *   var result = services["prompt"].askQuestion( "Do you want to continue?", [ "Yes", "No" ] );
       *   if( result == "No" )
       *     return;
       *   ....
       * </pre>
       * 
       * @param msg          the message to display
       * @param choices      array of choices presented to the user
       * @return one of the elements from <code>choices</code>
       */
      public function askQuestion( msg, choices )
      {
        var dialog = windowManager.getDialog("Question");
        var pane   = new ti.chimera.OptionPane(
          dialog, convertMsg(msg), 
          swing.JOptionPane.QUESTION_MESSAGE,
          swing.JOptionPane.DEFAULT_OPTION,
          null, choices
        );
        
        pane.setWantsInput(false);
        pane.showModal();
        
        return pane.getValue();
      }
      
      /**
       * Prompt for string input.  The <code>msg</code> is displayed to the
       * user and the user is prompted for text input.  If <code>regexp</code>
       * is not null, it is a regular expression string that the input must
       * match in order for the user to proceed.  For example, the regular 
       * expression <code>".*\.os$"</code> matches any filename ending in 
       * <code>".os"</code>.  
       * <p>
       * This method blocks until valid input is entered at which point the 
       * input string is returned, or the user cancels in which case 
       * <code>null</code> is returned.
       * 
       * @param msg      the message to display to the user
       * @param default  the default value for the input
       * @param regexp   the regular expression that the input must match
       *    (XXX regular expression support not yet implemented)
       * @return the input, or <code>null</code> if the user canceled.
       */
      public function promptForString( msg, default, regexp )
      {
        if( default == null )
          default = "";
        
        var dialog = windowManager.getDialog("Input");
        var pane   = new ti.chimera.OptionPane(
          dialog, convertMsg(msg), 
          swing.JOptionPane.PLAIN_MESSAGE,
          swing.JOptionPane.OK_CANCEL_OPTION,
          null, null
        );
        pane.setWantsInput(true);
        pane.setInitialSelectionValue(default);
        pane.showModal();
        
        var val = pane.getInputValue();
        if( val == swing.JOptionPane.UNINITIALIZED_VALUE )
          return null;
        return val;
      }
      
      
    })();
    
  } );

})() );





/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

