/*=============================================================================
 *     Copyright Texas Instruments, Inc., 2001.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * Implements the "prompt" service by displaying popup dialogs.
 */
registry.register( new (function() extends ti.chimera.Plugin( main, "Prompt" ) {
  
  /**
   * Register an implementation of the "prompt" service.
   */
  registerServiceFactory( function() {
    
    return new (function() extends ti.chimera.service.Prompt() {
      
      /**
       * Display an error message to the user.
       * 
       * @param msg          the error message to display
       */
      public function showErrorMessage(msg)
      {
        main.error(msg);
        pkg.output.writeln(msg);
      }
      
      /**
       * Display a warning message to the user.
       * 
       * @param msg          the warning message to display
       */
      public function showWarningMessage(msg)
      {
        main.warning(msg);
        pkg.output.writeln(msg);
      }
      
      /**
       * Display an info message to the user.
       * 
       * @param msg          the info message to display
       */
      public function showInfoMessage(msg)
      {
        pkg.output.writeln(msg);
      }
      
      /**
       * Display a progress message to the user.
       * 
       * @param msg          the progress message to display
       */
      public function showProgress(msg)
      {
        pkg.output.writeln(msg);
        
        return new (function() extends ti.chimera.service.Prompt.Progress() {

          var _min;
          var _max;
          var _val = 0;
          
          /**
           * Change the progress message that is shown to the user
           * 
           * @param msg    the new message to display
           */
          public function setMessage(msg)
          {}
          
          /**
           * Update the range of this display.  The value (see {@link #setValue})
           * must be between <code>min</code> and <code>max</code> inclusive.
           * 
           * @param min          the min
           * @param max          the max
           */
          public function setRange( min, max )
          {
            _min = min;
            _max = max;
          }
          
          /**
           * Update the progress.  The <code>val</code> should be between the
           * min and max (inclusive) specified in {@link #setRange}.  It is
           * an error to call this if {@link #setRange} has not been called.
           * 
           * @param val          the new value
           */
          public function setValue(val)
          {
            _val = val;
            
            var p = 1.0*val/_max*100;
            
            pkg.output.writeln( 1*p + "%" );
          }
          
          /**
           * Get the current progress value.
           * 
           * @return the current value
           */
          public function getValue()
          {
            return _val;
          }
          
          /**
           * Indicate that the progress has completed, and the progress dialog or
           * message should no longer be displayed.
           */
          public function dispose()
          {
            pkg.output.writeln( "Completed!" );
          }
          
        })();
      }
      
      /**
       * Ask a question of the user. 
       * @param msg          the message to display
       * @param choices      array of choices presented to the user
       * @return one of the elements from <code>choices</code>
       */
      public function askQuestion( msg, choices )
      {
        pkg.output.writeln( "askQuestion() not supported..." );  
        return choices[0];
      }
      
      /**
       * Prompt for string input.
       * 
       * @param msg      the message to display to the user
       * @param default  the default value for the input
       * @param regexp   the regular expression that the input must match
       *    (XXX regular expression support not yet implemented)
       * @return the input, or <code>null</code> if the user canceled.
       */
      public function promptForString( msg, default, regexp )
      {
        pkg.output.writeln( "promptForString() not supported..." );  
        return default;
      }
      
      
    })();
    
  } );

})() );





/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */
