/*=============================================================================
 *     Copyright Texas Instruments, Inc., 2001.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



/******************************************************************************
 * The registry brower plugin, displays registry browser views that let the
 * user see and potentially modify values in the registry.
 * <p>
 * It uses a factory pattern to create the components for viewing (and 
 * modifying) the value of a registry node.  A component factory can be 
 * registered on a per-node-contract-type basis.  In the case that more than
 * one factory is registered for a particular type, the one that was most 
 * recently registered that can return a component is used.
 */
registry.register( new (function() extends ti.chimera.Plugin( main, "Registry Browser" ) {
  
  pkg.registry.setDefaultPersistentValue( 
    "/Dialogs/Registry Browser/bounds",
    new java.awt.Rectangle(225,135,525,525) 
  );
  
  /**
   * The table mapping node-contract type to a list of node-view factories.
   */
  var viewFactoryTable = new java.util.Hashtable();
  
  /**
   * Register a service to allow other plugins to make use of the registry
   * browser.
   */
  registerServiceFactory( function() {
    
    return new (function() extends ti.chimera.Service("registry browser") {
      
      /**
       * Show a registry browser view, with the specified root path.
       */
      public function showBrowser(path)
      {
        addResource( new ViewResource( new RegistryView(path) ) );
      }
      
      /**
       * Register a node-view factory.  The factory is just a function
       * that takes the node itself as an argument, and returns a swing
       * component.
       * 
       * @param nodeContractType   the type of node-contract
       * @param factory            the factory
       */
      public function registerNodeViewFactory( nodeContractType, factory )
      {
        var list = viewFactoryTable.get(nodeContractType);
        
        if( list == null )
        {
          list = new java.util.LinkedList();
          viewFactoryTable.put( nodeContractType, list );
        }
        
        list.addFirst(factory);
      }
      
      // XXX deregisterNodeViewFactory not implemented yet
      
    })();
    
  } );
  
  /****************************************************************************
   * The registry view.
   */
  function RegistryView(path)
    extends ti.chimera.View( main, "Registry: " + path, null )
  {
    /*************************************************************************/
    var tree = new swing.JTree( new ti.chimera.registry.RegistryTreeModel( main, path ) );
    
    tree.setCellRenderer( new ti.chimera.registry.RegistryTreeModel.RegistryTreeCellRenderer() );
    
    var selectionModel = new swing.tree.DefaultTreeSelectionModel();
    selectionModel.setSelectionMode( swing.tree.TreeSelectionModel.SINGLE_TREE_SELECTION );
    tree.setSelectionModel(selectionModel);
    
    tree.addTreeSelectionListener( new (function() extends swing.event.TreeSelectionListener() {
      
      public function valueChanged(evt)
      {
        var treePathComponents = evt.getPath().getPath();
        var node = treePathComponents[ treePathComponents.length() - 1 ].getNode();
        
        rightPanel.setVisible(false);
        rightPanel.removeAll();
        rightPanel.add( getNodeView(node), java.awt.BorderLayout.CENTER );
        
        var commentPanel = new swing.JPanel();
        var commentLabel = new swing.JLabel("<html>" + node.getComment() + "</html>");
        commentLabel.setBorder( swing.BorderFactory.createTitledBorder("Description") ); 
        rightPanel.add( commentLabel, java.awt.BorderLayout.SOUTH );
        rightPanel.setVisible(true);
        
        splitPane.resetToPreferredSizes();
        
        if( splitPane.getDividerLocation() < 150 )
          splitPane.setDividerLocation(150);
      }
      
    })() );
    
    addCloseRunnable( function() {
      tree.getModel().dispose();
    } );
    
    /*************************************************************************/
    var rightPanel = new swing.JPanel( new java.awt.BorderLayout() );
    
    /*************************************************************************/
    var splitPane = new swing.JSplitPane( swing.JSplitPane.HORIZONTAL_SPLIT,
                                          false,
                                          new swing.JScrollPane(tree),
                                          rightPanel );
    
    /**
     * Get the component that makes up this view.  The view must always 
     * implement this method.
     * 
     * @return any swing component will do
     */
    public function getComponent()
    {
      return splitPane;
    }
  }
  /***************************************************************************/
  
  /**
   * Get the node-view for the specified node.  This checks for node-
   * view factories registered for the type of the node's node-contract,
   * and returns the component created by the first factory that can
   * create a component to view (and possibly edit) this node.
   * 
   * @param node    the node to get a component for
   * @return a swing.JComponent
   */
  function getNodeView(node)
  {
    var list = viewFactoryTable.get( node.getNodeContract().getClass() );
    var component = null;
    
    if( list != null )
      for( var itr=list.iterator(); itr.hasNext() && (component == null); )
        component = itr.next()(node);
    
    if( component == null )
      component = defaultNodeViewFactory(node);
    
    return component;
  }
  
  /**
   * The default node-view factory just displays the value (read-only)
   * as text.
   * 
   * @param node    the node to get a component for
   * @return a swing.JComponent
   */
  function defaultNodeViewFactory(node)
  {
    var subscriber;
    var textArea = new swing.JTextArea();
    textArea.setEditable(false);
    
    node.subscribe( subscriber = function( node, value ) {
      if( value != null )
        textArea.setText( "" + value );
    } );
    
    return new (function() extends swing.JScrollPane(textArea) {
      var super_removeNotify = removeNotify;
      public function removeNotify() { node.unsubscribe(subscriber); super_removeNotify(); }
    })();
  }
  
})() );


pkg.registry.installStandardNodeViewFactories();



/**
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

