/*=============================================================================
 *     Copyright Texas Instruments, Inc., 2003.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * The console-plugin is a bit of a special case... it is started from the
 * bootstrap code, and cannot be closed.  Also, it makes no sense to have
 * more than one console, so it is never added to the plugin menu.
 */
registry.register( new (function() extends ti.chimera.Plugin( main, "Script Path" ) {
  
  /**
   * this plugin is always active.
   */
  addResource( new (function() extends ti.chimera.Resource(false) {
    
    public function install() {}
    public function uninstall() {}
    
  })() );
  
  if( ! registry.exists("/ScriptPath/pathList") )
  {
    registry.link( 
      new ti.chimera.registry.PersistentNode( new java.util.LinkedList(), 
                                              new ti.chimera.registry.TypeNodeContract( java.util.LinkedList ),
                                              "list of script-path entries" ),
      "/ScriptPath/pathList" );
  }
  
  /**
   * The list of entries in the script-path created by this plugin.  This is
   * preserved in the registry.  This is only the entries added by this plugin,
   * rather than all entries, to help keep the user from screwing up by
   * permanently removing an entry that is important to make the system work.
   * When this plugin is loaded, all the entries that are in this are added
   * to the script-path.
   */
  var pathList = registry.resolve("/ScriptPath/pathList").getValue();
  
  function addScriptPath(path)
  {
    if( ! pathList.contains(path) )
      pathList.add(path);
    pkg.system.addScriptPath(path);
    registry.resolve("/ScriptPath/pathList").setValue(pathList);
    services["file browser"].makeVisible(path);
  }
  
  function removeScriptPath(path)
  {
    if( ! pathList.contains(path) )
      return;
    pathList.remove(path);
    pkg.system.removeScriptPath(path);
    registry.resolve("/ScriptPath/pathList").setValue(pathList);
  }
  
  // restore existing script-path entries:
  for( var itr=pathList.iterator(); itr.hasNext(); )
    addScriptPath( itr.next() );
  
  /**
   * The view that allows users to add/remove entries from the script-path
   */
  function ScriptPathView()
    extends ti.chimera.View( main, "Script Path", null )
  {
    var panel = new swing.JPanel( new java.awt.BorderLayout() );
    
    // a description of the dialog:
    panel.add( new swing.JLabel("<html>The script-path is the set of places that the scripting " +
                                "engine checks to locate a script file.  Some entries in the " +
                                "script-path are added by the scripting engine itself, and " +
                                "cannot be removed.  You can use this dialog to add and " +
                                "remove other paths.</html>"),
               java.awt.BorderLayout.NORTH );
    panel.setPreferredSize( new java.awt.Dimension(300,300) );
    
    // the list of entries in the script-path:
    var list = new swing.JList();
    list.setModel( new swing.DefaultListModel() );
    list.setSelectionMode( swing.ListSelectionModel.SINGLE_SELECTION );
    
    {
      var scrollPane = new swing.JScrollPane(list);
      scrollPane.setBorder( new swing.border.TitledBorder("script-path: ") );
      panel.add( scrollPane, java.awt.BorderLayout.CENTER );
    }
    
    // the button panel:
    var buttonPanel = new swing.JPanel();
    panel.add( buttonPanel, java.awt.BorderLayout.SOUTH );
    
    // the 'Add' button, to add a new entry to the script-path:
    var addButton = new swing.JButton( "Add", pkg.util.getImageIcon("Plus24.gif") );
    addButton.addActionListener( function(evt) {
      
      var chooser = new ti.chimera.FileChooser( windowManager.getDialog("Add Script Path") );
      chooser.setFileFilter( new (function() extends ti.chimera.FileChooser.AbstractFileFilter() {
        
        public function accept(f)
        {
          return f.isDirectory();
        }
        
        public function getDescription()
        {
          return "All Directories";
        }
        
      })() );
      
      if( chooser.showOpenDialog(null) == ti.chimera.FileChooser.APPROVE_OPTION )
        addScriptPath( chooser.getSelectedFile().getPath() );
      
      refresh();
      
    } );
    buttonPanel.add(addButton);
    
    // the 'Remove' button, to remove an entry from the script-path:
    var removeButton = new swing.JButton( "Remove", pkg.util.getImageIcon("Minus24.gif") );
    removeButton.addActionListener( function(evt) {
      removeScriptPath( list.getSelectedValue() );
      refresh();
    } );
    buttonPanel.add(removeButton);
    
    /**
     * Get the component that contains this view.
     * 
     * @return a swing component
     */
    public function getComponent()
    {
      return panel;
    }
    
    function refresh()
    {
      var model = list.getModel();
      model.removeAllElements();
      for( var path : pkg.system.getScriptPath() )
        model.addElement(path);
      refreshButtons();
    }
    
    function refreshButtons()
    {
      var val = list.getSelectedValue();
      removeButton.setEnabled( (val != null) && pathList.contains(val) );
    }
    
    list.addListSelectionListener( function(evt) {
      refreshButtons();
    } );
    
    refresh();
  }
  
  /**
   * View resource for the script-path view.
   */
  var viewResource = new ViewResource( function() { return new ScriptPathView(); }, false );
  
  /**
   * The service that is the public interface to this plugin.
   */
  registerServiceFactory( function() {
    
    return new (function() extends ti.chimera.Service("script path") {
      
      /**
       * Show the script-path editor view.
       */
      public function show()
      {
        addResource(viewResource);
      }
      
      /**
       * Hide the script-path editor view.
       */
      public function hide()
      {
        removeResource(viewResource);
      }
      
    })();
    
  } );
  
})() );

