/*=============================================================================
 *     Copyright Texas Instruments, Inc., 2001.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * A tree node that can (optionally) have children. The children are
 * maintained in sorted order.
 *
 * @param value        the value of the tree node
 */
public function SortedTreeNode( value )
{
  var children = new java.util.Vector();
  var childrenSorted = false;

  /*
   * The comparator used by the sorting routine to sort the children.
   */
  var nodeComparator = new ( function() extends java.util.Comparator() {
    // Could not get " return node1.getValue().castToString().compareTo(
    // node2.getValue().castToString() ); " to work...
    public function compare( node1, node2 )
    {
      if( node1.getValue() < node2.getValue() )
      {
        return -1;
      }
      else if( node1.getValue() == node2.getValue() )
      {
        return 0;
      }
      else
      {
        return 1;
      }
    }
  } )();

  function sortChildren()
  {
    if( !childrenSorted )
    {
      synchronized(children)
      {
        java.util.Collections.sort(children, nodeComparator);
        childrenSorted = true;
      }

    }
  }

  /**
   * Callable by derived classes to insert a new child node.
   */
  public function insert( newChild )
  {
    synchronized(children)
    {
      children.add(newChild);
      childrenSorted = false;
    }

  }

  /**
   * Return the value of the node.
   *
   * @return value of the node
   */
  public function getValue()
  {
    return value;
  }

  /**
   * Return the number of child nodes.
   *
   * @return number of child nodes
   */
  public function getChildCount()
  {
    return children.size();
  }

  /**
   * Get the specified child. Makes sure that the children
   * are maintained in sorted order before returning the
   * specified child.
   *
   * @param idx          the index of the requested child
   * @return the child with the specified index
   */
  public function getChild( idx )
  {
    synchronized(children)
    {
      sortChildren();

      return children.get(idx);
    }

  }

  /**
   * Given a child, determine the child's index.  This is the inverse of the
   * <code>getChild</code> operation. Makes sure that the children are
   * maintained in sorted order before returning the index.
   *
   * @param child        the child
   * @return the child's index, or <code>-1</code> if the node isn't a child
   */
  public function getIndexOfChild( child )
  {
    synchronized(children)
    {
      sortChildren();

      return children.indexOf(child);
    }

  }

  /**
   * Is this node a leaf?
   */
  public function isLeaf()
  {
    return children.size() == 0;
  }

  /**
   * Return the value of the node as a string.
   */
  public function castToString()
  {
    return value.castToString();
  }

}



/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

