/*=============================================================================
 *     Copyright Texas Instruments 2002. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * The debugger plugin provides a way to debug objectscript code.
 */
registry.register( new (function() extends ti.chimera.Plugin( main, "Talkback" ) {
  
  mixin pkg.system.makePrivatePackage("/jar/chimera/TalkbackPlugin/");
  
  var viewCount = 0;
  
  var hitOutOfMemoryErrorCnt = 0;
  
  var ignoredExceptions = [ java.lang.ThreadDeath ];
  var commonExceptions = [ [ java.lang.OutOfMemoryError, 
                             function( t, e ) {
                               hitOutOfMemoryErrorCnt++;
                               return ("You have run out of memory, to allow more memory\n" +
                                       "to be used use the <code>-Xmx</code> command line argument,\n" + 
                                       "for example <code>java -Xmx128m -jar ...</code> to use a max\n" + 
                                       "heap size of 128MB.");
                             } ],
                           [ java.lang.InterruptedException,
                             function( t, e ) {
                               return ("An unhandled interruption (perhaps due to a timeout?) has\n" + 
                                       "occurred in the thread \"" + t.getName() + "\".");
                             } ] ];
  
  var reportDataFxnList = new java.util.LinkedList();
  var reportAttachmentFxnList = new java.util.LinkedList();
  
  /**
   * Add a function that will be called to add extra data to the talkback
   * report.  Care should be taken that this function does not throw an
   * exception.
   * 
   * @param fxn     the function that should return a string
   */
  function addReportDataFxn(fxn)
  {
    reportDataFxnList.add(fxn);
  }
  
  /**
   * Add a function that will be called to (potentially) add an attachment to
   * the talkback report.  It should return a <code>pkg.email.Attachment</code>
   * or <code>null</code> to add no attachment.  Care should be taken that
   * this function does not throw an exception.
   * 
   * @param fxn     the function that should return an attachment
   *    or <code>null</code>
   */
  function addReportAttachmentFxn(fxn)
  {
    reportAttachmentFxnList.add(fxn);
  }
  
  addReportDataFxn( function() {
    
    var str = ("Environment: \n" + 
               "------------ \n");
    
    str += "   version:   " + oscript.OscriptInterpreter.getVersionString() + "\n";
    
    for( var i=0; i<ENVIRONMENT_PROPERITIES.length(); i++ )
      str += "   " + ENVIRONMENT_PROPERITIES[i] + ":   " + getProperty(ENVIRONMENT_PROPERITIES[i]) + "\n";
    
    str += "   address:   " + java.net.InetAddress.getLocalHost() + "\n";
    
    return str;
    
  } );
  addReportDataFxn( function() {
    
    var r = Runtime.getRuntime();
    var str = ("Memory: \n" + 
               "------- \n");
    
    str += "   hitOutOfMemoryErrorCnt:   " + hitOutOfMemoryErrorCnt + "\n";
    str += "   freeMemory:    " + r.freeMemory() + "\n";
    str += "   totalMemory:   " + r.totalMemory() + "\n";
    
    return str;
    
  } );
  
  /**
   * The "talkback" service.
   */
  registerServiceFactory( function() {
    
    return new (function() extends ti.chimera.service.Talkback() {
      
      /**
       * Add a function that will be called to add extra data to the talkback
       * report.  Care should be taken that this function does not throw an
       * exception.
       * 
       * @param fxn     the function that should return a string
       */
      public var addReportDataFxn = addReportDataFxn;
      
      /**
       * Add a function that will be called to (potentially) add an attachment to
       * the talkback report.  It should return a <code>pkg.email.Attachment</code>
       * or <code>null</code> to add no attachment.  Care should be taken that
       * this function does not throw an exception.
       * 
       * @param fxn     the function that should return an attachment
       *    or <code>null</code>
       */
      public var addReportAttachmentFxn = addReportAttachmentFxn;
      
      /**
       * Deal with an unhandled exception.  The thread (<code>t</code>) may
       * be <code>null</code>, for example if the thread the exception is
       * thrown in is AWT event dispatch thread.  
       * 
       * @param t          the thread the exception was thrown in
       * @param e          the exception
       */
      public function uncaughtException( t, e )
      {
        main.debug( 1, ""+e );
        
        e = unwrapException(e);
        
        var msg = "Uncaught exception in thread: " + ((t == null) ? "AWT" : t) + "\n";
        
        if( ! uiVisible() )
        {
          System.err.println(msg + e);
          return;
        }
        
        for( var ignoredException : ignoredExceptions )
          if( e instanceof ignoredException )
            return;
        
        /* if it is a script exception, give the user a chance to debug:
         */
        if( e instanceof Exception )
          if( services["prompt"].askQuestion( "An unhandled script error has occurred.<br>" +
                                              "Would you like a chance to debug?",
                                              [ "Yes", "No" ] ) == "Yes" )
            pkg.output.writeln( services["debugger"].makeExceptionMessage(e) );
        
        /* detect common user correctable errors, and display appropriate
         * error dialog instead of bringing up talkback...
         */
        for( var commonException : commonExceptions )
          if( e instanceof commonException[0] )
            return (services["prompt"].showErrorMessage( commonException[1]( t, e ) ),null);
        
        if( e instanceof java.lang.Throwable )
        {
          var sw  = new java.io.StringWriter();
          e.printStackTrace( new java.io.PrintWriter(sw) );
          msg += sw;
        }
        else
        {
          msg += e;
        }
        
        if( viewCount > 15 )
          System.err.println("too many errors: error: " + msg);
        else
try {
        addResource( new ViewResource( new TalkbackView(msg) ) );
} catch(e) {
  pkg.output.writeln("error sending talkback report: " + e);
}
      }
      
      /**
       * Is the user interface visible.
       */
      function uiVisible()
      {
        var windowManager = services["window manager"];
        if( windowManager != null )
          return windowManager.isVisible();
        return false;
      }
      
      /**
       * Bring up the talkback report window to let the user send the
       * developers an error report.  This can be wired up to the
       * "Help/Report Bug" menubar entry to let the user report
       * weirdness which does not necessarily result in an unhandled
       * exception.
       */
      public function reportError()
      {
        addResource( new ViewResource( new TalkbackView(null) ) );
      }
      
    })();
    
  } );
  
  // properties that are included in message sent back to developers:
  public static var ENVIRONMENT_PROPERITIES = [ "java.version",
                                                "os.name",
                                                "os.version",
                                                "user.name",
                                                "user.dir",
                                                "user.dir.orig" ];
  
  /**
   * Get a system property.
   */
  var getProperty = System.getProperty;
  
  /* Create the registry nodes for talkback related preferences:
   */
  if( !registry.exists("/Preferences/Talkback") )
  {
    registry.link( 
      new ti.chimera.registry.PersistentNode( "chimera-devel@list.sand.design.ti.com", 
                                              ti.chimera.registry.NodeContract.STRING_CONTRACT,
                                              "The email address that talkback reports are sent to." ),
      "/Preferences/Talkback/Recipient" );
    
    registry.link( 
      new ti.chimera.registry.PersistentNode( "smtp.sand.design.ti.com", 
                                              ti.chimera.registry.NodeContract.STRING_CONTRACT,
                                              "The SMTP server used to send the talkback report." ),
      "/Preferences/Talkback/SMTP Server" );
    
    registry.link( 
      new ti.chimera.registry.PersistentNode( "", 
                                              ti.chimera.registry.NodeContract.STRING_CONTRACT,
                                              "Your email address, used as the \"From\" field " +
                                              "in the talkback report" ),
      "/Preferences/Talkback/Email Address" );
  }
  
  /**
   * Sometimes an exception will be wrapped inside another exception.  
   */
  function unwrapException(e)
  {
    /* Unwrap exceptions that might be wrapped in other exceptions...
     */
    while(true)
    {
      if( e instanceof java.lang.reflect.InvocationTargetException )
        e = e.getTargetException();
      else if( e instanceof oscript.exceptions.PackagedScriptObjectException )
        e = e.val;
      else
        return e;
    }
  }
  
})() );




/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */
