/*=============================================================================
 *     Copyright Texas Instruments 2002. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

mixin java.io;

/**
 * A collection of "command-line" type utility functions.
 */
public var cl = new (function() {
  
  /**
   * Run a function from a background thread.
   * 
   * @param fxn          the function to invoke from another thread context
   * @return the thread object
   */
  public function bg(fxn)
  {
    var thread = new java.lang.Thread( fxn, "bg" );
    
    thread.start();
    
    return thread;
  }
  
  /**
   * Cat the contents of a file to the screen.  Like the unix "cat" command.
   * 
   * @param path         the path to the file
   * @return the file as a string
   */
  public function cat(path)
  {
    var br = new BufferedReader( new InputStreamReader( resolveFile(path).getInputStream() ) );
    
    var str = "";
    var line;
    while( (line=br.readLine()) != null )
      str += line + "\n";
    
    return str;
  }
  
  /**
   * Create the file, if it doesn't exist.  Like the unix "touch" command.
   * 
   * @param path         the path to the file
   */
  public function touch(path)
  {
    var file = resolveFile(path);
    
    if( !file.exists() )
      file.createNewFile();
  }
  
  /**
   * Open the file using the editor.
   * 
   * @param path        the path to the file to edit
   */
  public function edit(path)
  {
    var file = resolveFile(path);
    
    if( !file.exists() )
      return "No such file: " + file;
    else if( !file.canRead() )
      return "Cannot read file: " + file;
    
    services["editor"].openFile(file);
  }
  
  /**
   * Like the unix command line utility, "grep", find pattern matches
   * in the specified files.  The match objects have the following
   * properties:
   * <ul>
   *   <li> file  -  the matching file
   *   <li> line  -  the line number
   *   <li> str   -  the string text of the matching line
   * </ul>
   * 
   * @param regexp    the regular expression to match
   * @param paths     one or more files to find matches in
   * @return an array of match objects, which converts itself to a
   *    nice string representation when castToString()
   */
  public function grep( regexp, paths... )
  {
    regexp = compileRegExp(regexp);
    var results = new GrepResultsArray();
    for( var path : paths )
    {
      var file = resolveFile(path);
      if( file.isFile() )
        grepFile( results, regexp, file );
    }
    return results;
  }
  
  /**
   * Recursive grep.  For any arguments that are directories, the
   * contents of the directory are recursively processed.  See
   * {@link #grep}
   * 
   * @param regexp    the regular expression to match
   * @param paths     one or more files to find matches in
   * @return an array of match objects, which converts itself to a
   *    nice string representation when castToString()
   * @see grep
   */
  public function grepr( regexp, paths... )
  {
    regexp = compileRegExp(regexp);
    var results = new GrepResultsArray();
    
    function greprImpl(file)
    {
      if( file.isDirectory() && !isHidden(file) )
        for( var child : pkg.fs.children(file) )
          greprImpl(child);
      else if( file.isFile() )
        grepFile( results, regexp, file );
    }
    
    for( var path : paths )
      greprImpl( resolveFile(path) );
    
    return results;
  }
  
  /**
   * A handy utility that lets grep and grepr take either a search string or
   * a regular expression
   */
  private function compileRegExp(regexp)
  {
    if( regexp instanceof RegExp )
      return regexp;
    return new RegExp( regexp.castToString().replace(/\ /,"\\ ") );
  }
  
  /**
   * Helper function to grep an individual file
   * @param results   the results array, any matches are appended
   *    to this array
   * @param regexp    the regular expression to match
   * @param file      the file to search
   */
  private function grepFile( results, regexp, file )
  {
    if( file.canRead() && !isHidden(file) )
    {
      var br = new BufferedReader( new InputStreamReader( file.getInputStream() ) );
      var line = 1;
      var str;
      while( (str=br.readLine()) != null )
      {
        var matches = str.match(regexp);
        if( matches.length() > 0 )
          results[ results.length() ] = new GrepResult( file, line, str, matches );
        line++;
      }
    }
  }
  
  /**
   * Basically a regular array, but overrides castToString to display the
   * grep results nicely.
   */
  private function GrepResultsArray()
    extends Array()
  {
    public function castToString()
    {
      var str = "";
      for( var result : this )
        str += result.castToString() + "\n";
      return str;
    }
  }
  
  /**
   * An object that represents a grep match on a particular line and line.
   * Converts itself nicely to a hyperlink string that shows the match, and
   * when clicked will open the editor to the appropriate file and line
   * number.
   * 
   * @param file     the file containing the match
   * @param line     the line number of the matching line
   * @param str      the text of the line
   * @param matches  the <code>RegExpResult</code> object itself
   */
  private function GrepResult( public file, public line, public str, matches )
  {
    public function castToString()
    {
      var colorizedMatchStr = "";
      if( matches.index > 0 )
        colorizedMatchStr += str.substring( 0, matches.index );
      colorizedMatchStr += pkg.output.fgColor( java.awt.Color.red, str.substring( matches.index, matches.index + matches[0].length() ) );
      if( (matches.index + matches[0].length()) < str.length() )
        colorizedMatchStr += str.substring( matches.index + matches[0].length() );
      return pkg.output.hyperlink( '{
        services["editor"].openFileToLineNum( file, line );
      }, file.getName() + ":" + line ) + " " + colorizedMatchStr;
    }
  }
  
  /**
   * works sort of like the UNIX command-line "ls" program.  Returns
   * an array of files.
   * 
   * @param path        the path to the file/directory to list
   * @return an array of files, which converts itself to a nice
   *    string representation when castToString()
   */
  public function ls(args...)
  {
    var path;
    
    if( (args.length() == 0) || (args[0] == null) )
      path = "";
    else
      path = args[0];
    
    // XXX temporary hack:
    path = (path.length() == 0) ? getCwd() : path;
    
    var file = resolveFile(path);
    
    if( !file.exists() )
      throw new Exception("no such file: " + file);
    
    var files = new FileArray(file);
    for( var child : pkg.fs.children(file) )
      if( ! isHidden(child) )
        files[ files.length() ] = child;
    
    return files;
  }
  
  /**
   * Table of function(file) => boolean for determining if the file should
   * be hidden.  Eventually the interface should be to specify regexp, but
   * not until JDK v1.4.x is the minimum required JDK version.
   */
  var hiddenCheckers = [
    
    /* Check for ending in "~"
     */
    function(file) {
      var name = file.getName();
      return (name.length() > 0) && (name[ name.length() - 1 ] == "~");
    },
    
    /* Check for beginning with "."
     */
    function(file) {
      var name = file.getName();
      return (name.length() > 0) && (name[0] == ".");
    }
    
  ];
  
  /**
   * Is this a hidden file?
   * 
   * @param path      the path to the file/directory
   * @return <code>true</code> if hidden, else <code>false</code>
   */
  public function isHidden(path)
  {
    var file = resolveFile(path);
    for( var i=0; i<hiddenCheckers.length(); i++ )
      if( hiddenCheckers[i](file) )
        return true;
    return false;
  }
  
  /**
   * Basically a regular array, but overrides castToString to display the
   * files nicely.
   * 
   * @param currentFile   the file representing this directory (ie the parent
   *    of the children which are the members of this array)
   */
  private function FileArray(currentFile) extends Array()
  {
    // these are only public because of "bug" which requires statics to be public
    public static const var SIZE_UNITS = [ "b", "k", "m", "g" ];
    public static const var COLUMN_WIDTH = 30;
    public static const var NUM_COLUMNS = 2;
    public static const var COLUMN_PADDING = "   ";
    public static const var PADDING = (function() {
        
        var arr = [];
        var str = "";
        
        for( var i=0; i<COLUMN_WIDTH; i++ )
        {
          arr[i] = str.castToString();  // by doing castToString, we "flatten" the internal representation which should be better for memory usage
          str += " ";
        }
        
        return arr;
        
      })();
    
    /**
     * Table of function(file) => function(filename) => String:
     * <p>
     * We increment through until we find a match (ie. function that returns 
     * non-null).  The returned value is the fxn to apply to the filename to 
     * apply attributes.  The order of entries in the table, is the order of
     * precedence; after the first match, no more checkers are called.
     */
    var attributeCheckers = [
      
      /* Check for Directory:
       */
      function(file) {
        if( file.isDirectory() )
          return function(filename) {
            return pkg.output.hyperlink( java.awt.Color.green, '{ 
              pkg.output.writeln( ls( file.getPath() ).castToString() );
            }, filename );
          };
        return null;
      },
      
      /* Check for .os file:
       */
      function(file) {
        if( file.getExtension() == "os" )
          return function(filename) {
            return pkg.output.hyperlink( '{ edit( file.getPath() ); }, filename );
          };
        return null;
      },
      
      /* Others?? check for .html file?
       */
      
      /* This one must be last.
       */
      function(file) {
        return function(filename) {
          return filename;
        };
      }
    ];
    
    public function castToString()
    {
      var str = "";
      
      // build the header:
      for( var i=0; i<NUM_COLUMNS; i++ )
        str += " Filename " + PADDING[COLUMN_WIDTH-13] + "size" + COLUMN_PADDING;
      str += "\n";
      for( var i=0; i<NUM_COLUMNS; i++ )
        str += " -------- " + PADDING[COLUMN_WIDTH-13] + "----" + COLUMN_PADDING;
      str += "\n";
      
      // build the body:
      for( var i=-1; i<length(); i++ )
      {
        var file;
        var filename;
        
        if( i < 0 )
        {
          file = resolveFile(currentFile.getPath() + "/../");
          filename = "..";
        }
        else
        {
          file = this[i];
          filename = file.getName();
        }
        
        var len = filename.length();
        
        /* the function that is applied to the filename after truncation
         * (to make things fit in columns)... this can be a no-op, or
         * can apply a hyperlink, color, etc.
         */
        var applyAttributesFxn = null;
        
        /* as long as the table isn't fucked, we are guaranteed that at
         * least the very last checker will match, so we don't have to
         * worry about going off the end of the table.  So don't mess
         * up the table, 'eh 
         */
        for( var i=0; applyAttributesFxn == null; i++ )
          applyAttributesFxn = attributeCheckers[i](file);
        
        var size = getSizeString(file);
        var pad  = COLUMN_WIDTH - filename.length() - size.length();
        
        // if there isn't enough room, truncate filename so pad is at least 1
        if( pad < 1 )
        {
          filename = filename[0..filename.length()-1+(pad-1)];
          pad = 1;
        }
        
        // build the actual hyperlink after computing padding so that
        // the extra characters don't mess up the calculation:
        filename = applyAttributesFxn(filename);
        
        str += " " + filename + PADDING[pad] + size + COLUMN_PADDING;
        
        if( ((i+1)%NUM_COLUMNS) == (NUM_COLUMNS-1) )
          str += "\n";
      }
      
      return str + "\n";
    }
    
    function getSizeString(file)
    {
      var sz = file.length();
      var unitIdx = 0;
      
      while( (sz > 1024) && (unitIdx < SIZE_UNITS.length()) )
      {
        sz /= 1024;
        unitIdx++;
      }
      
      return sz + SIZE_UNITS[unitIdx];
    }
  }
  
  /**
   * Change the current working dir, return the new current working dir.
   * 
   * @param path        the path to the directory to change to
   */
  public function cd(path)
  {
    setCwd( resolvePath(path) );
    return getCwd();
  }
  
  /* other commands like rm/mkdir/rmdir/mv/cp/etc would be pretty trivial,
   * although some of them would need a bit more work on AbstractFile first.
   */
  
  /**
   * In order to not change the global current working dir, we maintain our
   * own cwd... this package uses resolvePath() and resolveFile() to convert
   * a relative path to absolute path.
   */
  var cwd = pkg.fs.getCwd().getPath();
  
  public function setCwd(path)
  {
    cwd = path;
  }
  
  public function getCwd()
  {
    return cwd;
  }
  
  function resolvePath(path)
  {
    return pkg.fs.normalize( cwd, path );
  }
  
  /**
   * Take an argument that can either be an absolute or relative path, and
   * return a file.
   */
  public function resolveFile(path)
  {
    if( path instanceof oscript.fs.AbstractFile )
      return path;
    return new pkg.File( resolvePath(path) );
  }
  
})();



/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

