/******************************************************************************
 *     Copyright Texas Instruments 2002-2004. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


var max = java.lang.Math.max;
var min = java.lang.Math.min;
var abs = java.lang.Math.abs;

mixin pkg.editor;


/**
 * An editor frame, the text component that is the editor.
 * 
 * @param file         the file
 * @param container    ...
 */
public function EditorFrame( file, container )
  extends swing.JPanel( new java.awt.BorderLayout() )
{
  /**
   * Table that maps line # to gutter icon.
   */
  var gutterIconTable = new java.util.Hashtable();
  
  /**
   * Keep track of the widest icon, to simplify drawing and sizing the
   * glyph gutter.
   */
  var maxGutterIconWidth = 0;
  
  /**
   * The editor buffer is the actual text component, that is the main part
   * of the editor frame.
   */
  var editorBuffer = new (function(frame) extends swing.JEditorPane() {
    
    // needed to make a focus event of the editorBuffer to a buffer instance:
    public function getFrame() { return frame; }
    
    // when we resize, we have to notify the glyph gutter, because it may
    // need to resize in response
    var _super_setBounds = setBounds;
    var lastHeight = -1;
    public function setBounds( x, y, w, h )
    {
      _super_setBounds( x, y, w, h );
      if( h != lastHeight )
      {
        lastHeight = h;
        glyphGutter.updateSize();
      }
    }
    
  })(this);
  
  /**
   * The glyph-gutter is on the left hand side of the editor-buffer.  It
   * displays the line numbers, and can have other icons attached to any
   * line (for example, used to indicate current line of execution or
   * breakpoints, etc.)
   */
  var glyphGutter = new (function() extends swing.JComponent() {
    
    private var gutterColor = java.awt.Color.lightGray;
    
    /**
     * It is possible that there are more than one icon for a give
     * line... if that is the case, we choose the (repaintCount %
     * iconList.size()) icon to display.  A period timer causes
     * redraws, when there are lines with multiple icons, causing
     * the display to cycle through all the different icons on that
     * line.
     */
    private var repaintCount = 0;
    
    private var repaintTimer = new swing.Timer( 500, function(evt) {
      repaint();
    } );
    
    // keep track of current font metrics, font width, and font height,
    // assume a monospace font, because things work badly otherwise...
    private var fm = null;  // font metrics
    private var fw = -1;    // font width
    private var fh = -1;    // font height
    private var _super_setFont = setFont;
    
    public function setFont(f)
    {
      _super_setFont(f);
      
      fm = editorBuffer.getFontMetrics(f);
      if( fm != null )
      {
        fh = fm.getHeight();
        fw = fm.charWidth("m");
      }
      
      if( offScreen != null )
      {
        offScreen.flush();
        offScreen = null;
      }
    }
    
    // called by editor-buffer when it's height has changed, to
    // give us a change to adjust our width as needed
    public function updateSize()
    {
      if( fm != null )
      {
        var nlines = (editorBuffer.getHeight() / fh) + 1;
        
        var cols = 1;
        
        while( (nlines /= 10) > 0 )
          cols++;
        
        var newWidth  = (cols * fw) + maxGutterIconWidth;
        var newHeight = editorBuffer.getHeight();
        
        if( (newWidth != getWidth()) || (newHeight != getHeight()) )
        {
          setPreferredSize( new java.awt.Dimension( newWidth, newHeight ) );
          javax.swing.SwingUtilities.invokeLater( function() {
            revalidate();
          } );
        }
      }
    }
    
    // for some reason paint(g) is getting called frequently, so and swing's 
    // double buffering doesn't seem to stop that, so do our own dbl buffering.
    // The line numbers are drawn to the buffer, which is at least as tall as
    // the on-screen.  The glyph icons are drawn in the paint() method.
    var offScreen = null;
    var offScreenDimension = null;
    
    public function paint(g)
    {
      var scheduleRepaint = false;
      
      repaintCount++;
      
      if( (fm == null) || (fh <= 0) )
        return;
      
      for( var e=gutterIconTable.keys(); e.hasMoreElements(); )
      {
        var line = e.nextElement();
        var list = gutterIconTable.get(line);
        var icon = list[ abs( repaintCount % list.size() ) ];
        
        if( list.size() > 1 )
          scheduleRepaint = true;
        
        icon.paintIcon( this, g, 0, (line*fh) - (fm.getAscent() / 2) - (icon.getIconHeight() / 2) );
      }
      
      g.translate( maxGutterIconWidth, 0 );
      renderLineNumbers( g, 0, getHeight() / fh );
      
      if(scheduleRepaint)
        repaintTimer.start();
      else
        repaintTimer.stop();
    }
    

    function renderLineNumbers( g, startLine, endLine )
    {
      var c = g.getColor();
      
      g.setColor(gutterColor);
      g.setFont( getFont() );
      g.fillRect( 0, (startLine * fh), getWidth() - maxGutterIconWidth, (endLine * fh) );
      g.setColor(c);
      
      oscript.swing.text.ODEUtils.renderLineNumbers( g, startLine, endLine, fh );
    }
    
    addMouseListener( new function() extends java.awt.event.MouseAdapter() {
      
      public function mouseClicked(evt)
      {
        if( fh != -1 )
          notifyGutterClick( (evt.getY() / fh) + 1 );  // + 1 because line # start at 1, not 0
      }
      
    }() );
    
  })();
  
  /**
   * The status line is used to display status messages, etc.  It can be set to
   * display arbitrary text and/or an icon.
   */
  var statusLine = new swing.JLabel();
  
  /**
   * The list of callback functions to call when the editor frame is removed
   * from it's parent.
   */
  var closeListenerList = new java.util.LinkedList();
  
  /**
   * Add a callback function to call when the editor frame is closed.
   * 
   * @param fxn   the callback function()
   */
  public function addCloseListener(fxn)
  {
    closeListenerList.add(fxn);
  }
  
  /**
   * Set the font.
   * 
   * @param font         the java.awt.Font
   */
  public function setFont(font)
  {
    var off = getBuffer().getCaretPosition();
    var node = registry.resolve("/Preferences/Editor/Font");
    if( font != node.getValue() )
      node.setValue(font);
    editorBuffer.setFont(font);
    glyphGutter.setFont(font);
    swing.SwingUtilities.invokeLater( '{
      scrollToOffset(off);
    } );
  }
  
  function fontSubscriber( node, font )
  {
    setFont(font);
  }
  
  function bgColorSubscriber( node, color )
  {
    editorBuffer.setBackground(color);
  }
  
  function fgColorSubscriber( node, color )
  {
    editorBuffer.setForeground(color);
    editorBuffer.setCaretColor(color);
  }
  
  var _super_removeNotify = removeNotify;
  public function removeNotify()
  {
    registry.unsubscribeFromValue(fontSubscriber);
    registry.unsubscribeFromValue(bgColorSubscriber);
    registry.unsubscribeFromValue(fgColorSubscriber);
    
    if( mode != null )
      mode.cleanupFrame(this);
    
    for( var itr=closeListenerList.iterator(); itr.hasNext(); )
      itr.next()();
    
    _super_removeNotify();
  }
  
  var _super_addNotify = addNotify;
  public function addNotify()
  {
    registry.subscribeToValue( "/Preferences/Editor/Font", null, fontSubscriber );
    registry.subscribeToValue( "/Preferences/Editor/Background Color", null, bgColorSubscriber );
    registry.subscribeToValue( "/Preferences/Editor/Foreground Color", null, fgColorSubscriber );
    
    _super_addNotify();
    
    getBuffer().requestFocus();
  }
  
  
  {
    var sp = new swing.JScrollPane(editorBuffer);
    sp.setRowHeaderView(glyphGutter);
    add( sp, java.awt.BorderLayout.CENTER );
    add( statusLine, java.awt.BorderLayout.SOUTH );
  }
  
  /**
   * Get this frames container object.  This object is used as a way for
   * editor and editor-extensions to make use of features provided by the
   * embedder of the editor frame, such as to display a utilty tab pane,
   * or set the title of the dialog window, etc... this needs to be better
   * documented!
   */
  public function getContainer()
  {
    return container;
  }
  
  /**
   * Add the gutter icon for the specified line in this frame.
   * 
   * @param line         the line number
   * @param icon         a {@link javax.swing.Icon}
   */
  public function addGutterIcon( line, icon )
  {
    var iconList = gutterIconTable.get(line);
    if( iconList == null )
    {
      iconList = new java.util.LinkedList();
      gutterIconTable.put( line, iconList );
    }
    
    iconList.add(icon);
    
    refreshGlyphGutter();
  }
  
  /**
   * Remove the gutter icon for the specified line in this frame.
   * 
   * @param line         the line number
   * @param icon         a {@link javax.swing.Icon}
   */
  public function removeGutterIcon( line, icon )
  {
    var iconList = gutterIconTable.get(line);
    if( iconList == null )
      return;
    
    iconList.remove(icon);
    
    if( iconList.size() == 0 )
      gutterIconTable.remove(line);
    
    refreshGlyphGutter();
  }
  
  private function refreshGlyphGutter()
  {
    maxGutterIconWidth = 0;
    for( var list : gutterIconTable.values() )
      for( var icon : list )
        maxGutterIconWidth = max( maxGutterIconWidth, icon.getIconWidth() );
    
    glyphGutter.updateSize();
    glyphGutter.repaint();
  }
  
  
  private var gutterClickListenerList = null;
  
  private function notifyGutterClick(line)
  {
    if( gutterClickListenerList != null )
      for( var fxn : gutterClickListenerList.toArray() )
        fxn(line);
  }
  
  /**
   * Add a gutter click listener.  A gutter click listener is a function
   * that takes a line number (that was clicked on) as an argument.
   * 
   * @param fxn   the gutter click listener
   */
  public function addGutterClickListener(fxn)
  {
    if( gutterClickListenerList == null )
      gutterClickListenerList = new java.util.LinkedList();
    gutterClickListenerList.add(fxn);
  }
  
  /**
   * Remove the gutter click listener.
   * 
   * @param fxn   the gutter click listener
   */
  public function removeGutterClickListener(fxn)
  {
    if( gutterClickListenerList != null )
      gutterClickListenerList.remove(fxn);
  }
  
  /**
   * Get the {@link DocumentWriter} for this frame.
   * 
   * @param off   offset to start writing at
   * @param len   num of chars, starting at <code>off</code>, to overwrite
   */
  public function getDocumentWriter( off, len )
  {
    return new (getMode().getDocumentWriterType())( this, off, len );
  }
  
  /**
   * The buffer is the actual text component.
   */
  public function getBuffer()
  {
    return editorBuffer;
  }
  
  getBuffer().setEditorKit( new oscript.swing.text.ODEEditorKit() );
  
  var mode = new Mode();
  editorBuffer.setInputMap( WHEN_FOCUSED, mode.getInputMap() );
  editorBuffer.setActionMap( mode.getActionMap() );
  
  /**
   * Set the mode for this editor.
   * 
   * @param mode         the mode to set for this buffer
   */
  public function setMode(mode)
  {
    if( this.mode != null )
      this.mode.cleanupFrame(this);
// XXX    this.mode.setParent(mode);
    this.mode = new Mode( mode, "editor-frame-mode" );
    this.mode.initializeFrame(this);
    editorBuffer.setInputMap( WHEN_FOCUSED, this.mode.getInputMap() );
    editorBuffer.setActionMap( this.mode.getActionMap() );
    mode.runHooks( "set-mode", this );
  }
  
  setMode(globalMode);  // so we have a reasonable default
  
  /**
   * Get the current Mode.
   * 
   * @return an .Mode
   */
  public function getMode()
  {
    return mode.getParent();
  }
  
  /* Need better model for accessing status line... either multiple
   * clients should be able to be visible at the same time, or there
   * needs to be a way to restore to the previous state.
   */
  
  /**
   * Set the icon displayed on the status line.
   * 
   * @param icon         an icon
   */
  public function setStatusLineIcon(icon)
  {
    statusLine.setIcon(icon);
  }
  
  /**
   * Set the status line text.
   * 
   * @param text         the text to display in the status line
   */
  public function setStatusLineText(text)
  {
    statusLine.setText(text);
  }
  
  /**
   * Clear the status line.
   */
  public function clearStatusLine()
  {
    setStatusLineIcon(null);
    setStatusLineText("");
  }
  
  /**
   * Display a status line message.  The status line will be automatically
   * cleared.
   * 
   * @param str           the text to display
   */
  public function displayStatusMessage(str)
  {
    if( displayStatusTimer != null )
      displayStatusTimer.stop();
    
    setStatusLineIcon(INFO_SMALL_ICON);
    setStatusLineText(str);
    
    displayStatusTimer = new swing.Timer( 
      3000,
      new pkg.util.ActionAdapter( "", null, null, function(evt) {
        displayStatusTimer.stop();
        displayStatusTimer = null;
        clearStatusLine();
      } ) );
    displayStatusTimer.start();
  }
  
  var INFO_SMALL_ICON = pkg.util.getImageIcon("Information16.gif");
  var displayStatusTimer = null;
  
  /**
   * Set the file.
   * 
   * @param file         an pkg.File
   */
  public function setFile(file)
  {
    if( this.file == file )
      return;
    
    this.file = file;
    try
      container.setFile(file);
    catch(NoSuchMemberException e)
    {}
    refresh();
    mode.runHooks( "open-file", this );
  }
  
  /**
   * Get the file.
   * 
   * @return a pkg.File
   */
  public function getFile()
  {
    return file;
  }
  
  /**
   * Close this frame... actually close this frame's container.
   */
  public function close()
  {
    container.close();
  }
  
  /**
   * Is this view closable?
   * 
   * @return <code>true</code> or <code>false</code>
   */
  public function isClosable()
  {
    if(needsSave)
    {
      var file = getFile();
      var name = (file != null) ? file.getName() : "buffer";
      var answer = services["prompt"].askQuestion(
        name + " has unsaved changes.  Do you want to save?",
        ["Cancel","No","Yes"]
      );
      
      if( answer == "Yes" )
        save();
      else if( answer == "No" )
        return true;
    }
    
    return !needsSave;
  }
  
  var needsSave = false;
  
  /**
   * don't set needsSave directly, but use this method, as it updates container
   * on the state of the frame.
   */
  private function setNeedsSave(needsSave)
  {
    if( needsSave != this.needsSave )
    {
      try
        container.setNeedsSave(needsSave);
      catch(NoSuchMemberException e)
      { /* ignore, container may not implement this */ }

      this.needsSave = needsSave;
    }
  }
  
  getBuffer().getDocument().addDocumentListener( new (function() extends swing.event.DocumentListener() {
      public function insertUpdate(evt)
      {
        setNeedsSave(true);
      }
      public var removeUpdate  = insertUpdate;
      public var changedUpdate = insertUpdate;
    })() );
  
  /**
   * Save the contents of this buffer to the current file.
   */
  public function save()
  {
    if( (file == null) || !file.canWrite() )
    {
      try
      {
        var chosenFile = container.chooseFile( file, true );
        if( chosenFile != null )
        {
          saveAs(chosenFile);
          return;
        }
      }
      catch(NoSuchMemberException e)
      {
        // ignore, container may not implement chooseFile
      }
    }
    
    try
    {
      saveToFile(file);
      setNeedsSave(false);
      resetUndos();
      displayStatusMessage("saved to " + file);
    }
    catch(e)
    {
      services["prompt"].showErrorMessage( e.getMessage() );
    }
  }
  
  /**
   */
  public function saveAs(file)
  {
    try
    {
      saveToFile(file);
      setFile(file);
    }
    catch(e)
    {
      services["prompt"].showErrorMessage( e.getMessage() );
    }
    
  }
  
  /**
   * Save the contents of this buffer to the specified file.
   * 
   * @param file       the file to save to
   */
  public function saveToFile(file)
  {
    if( file == null )
      throw new Exception("file is null");
    
    if( !file.exists() )
      file.createNewFile();
    
    if( !file.canWrite() )
      throw new Exception("file is read-only: " + file);
    
    var buf = getBuffer().getText();
    var fos = file.getOutputStream(false);
    var out = new java.io.BufferedOutputStream(fos);
    (new java.io.PrintStream(out)).print(buf);
    out.flush();
    out.close();
    fos.close();
  }
  
  /**
   * Reload the contents of this buffer from the current file.
   */
  public function refresh()
  {
    if( file != null )
    {
      if( !file.canRead() )
      {
        displayStatusMessage("cannot read " + file);
        return;
      }
      
      displayStatusMessage("refreshing from " + file);
      var in  = new java.io.BufferedReader( new java.io.InputStreamReader( file.getInputStream() ) );
      var str = "";
      var line;
      
      while( (line=in.readLine()) != null )
        str += line + "\n";
      
      getBuffer().setText(str);
      resetUndos();
      
      // use invokeLater() so this happens *after* the doc event
      // that sets this flag to true
      swing.SwingUtilities.invokeLater( function() {
        setNeedsSave(false);
      } );
    }
    else
    {
      displayStatusMessage("no file, can't refresh");
    }
  }
  
  /**
   * The list of undos are kept in the table, as well as the current idx.
   * The undosIdx is incremented when new undoable edits occur or an undo
   * has been redone, and decremented when an edit is undone.
   */
  var undos;
  var undosIdx;
  var undosLastIdx;
  
  /**
   * Reset the undos.  This is done when the file is saved or reloaded.
   */
  function resetUndos()
  {
    undos = [];
    undosLastIdx = undosIdx = 0;
  }
  resetUndos(); 
  
  /**
   * Undo the last document edit.
   */
  public function undo()
  {
    if( undosIdx > 0 )
    {
      var edit = undos[ --undosIdx ];
      
      if( edit.canUndo() )
        edit.undo();
      else
        displayStatusMessage("cannot undo: " + edit.getPresentationName());
    }
    else
    {
      displayStatusMessage("no more undos");
    }
    
    setNeedsSave( undosIdx > 0 );
  }
  
  /**
   * Redo the last undo, if there is one.
   */
  public function redo()
  {
    if( undosIdx < undosLastIdx )
    {
      var edit = undos[ undosIdx++ ];
      
      if( edit.canRedo() )
        edit.redo();
      else
        displayStatusMessage("cannot redo: " + edit.getPresentationName());
    }
    else
    {
      displayStatusMessage("no more redos");
    }
    
    setNeedsSave( undosIdx > 0 );
  }
  
  /**
   * Copy the selected text, if any, into the clipboard, and then
   * delete the text from the document.
   */
  public function cut()
  {
    getBuffer().cut();
  }
  
  /**
   * Copy the selected text, if any, into the clipboard.
   */
  public function copy()
  {
    getBuffer().copy();
  }
  
  /**
   * Insert the current text contents of the clipboard, if any, into
   * the document at the current cursor position.
   */
  public function paste()
  {
    getBuffer().paste();
  }
  
  /**
   * Get the offset of the beginning of the specified line.
   * 
   * @param line         line number starting at 1
   */
  public function getLineOffset(line)
  {
    var buf = getBuffer();
    var doc = buf.getDocument();
    var map = doc.getDefaultRootElement();
    var elm = map.getElement( line - 1 );
    
    if( elm == null )
      return -1;    // XXX ???
    
    return elm.getStartOffset(); // depends on element structure!
  }
  
  /**
   * Get the line number corresponding to the specified offset.
   * 
   * @param off          the offset
   */
  public function getOffsetLine(off)
  {
    var map = getBuffer().getDocument().getDefaultRootElement();
    return map.getElementIndex(off) + 1;  // depends on element structure!
  }
  
  /**
   * Scroll to make the specified line number visible.
   * 
   * @param line         line number starting at 1
   */
  public function scrollToLine(line)
  {
    scrollToOffset( getLineOffset(line) );
  }
  
  /**
   * Scroll to make the specified offset visible.
   * 
   * @param off          offset into document
   */
  public function scrollToOffset(off)
  {
    var buf  = getBuffer();
    var rect = buf.modelToView( swing.text.Utilities.getRowStart( buf, off ) );
    
    buf.getCaret().setDot(off);
    
    if( rect != null )
    {
      // make it so the line we scroll to is in the middle of the screen:
      var vis  = buf.getVisibleRect();
      rect.setSize( vis.width, vis.height );
      rect.translate( 0, -vis.height/2 );
      
      buf.scrollRectToVisible(rect);
    }
    
    buf.requestFocus();
  }
  
  /**
   * Get the line number of the current cursor position.
   * 
   * @return the line number (starting at 1)
   */
  public function getLineNumber()
  {
    return getOffsetLine( getBuffer().getCaretPosition() );
  }
  
  /**
   * Get the column number of the current cursor position.
   * 
   * @return the column number (starting at 1)
   */
  public function getColumnNumber()
  {
    var buf = getBuffer();
    var off = buf.getCaretPosition();
    var start = swing.text.Utilities.getRowStart( buf, off );
    
    return off - start + 1;
  }
  
  var popup = new swing.JPopupMenu();
  
  /**
   * Add to popup menu.
   * 
   * @param action      a {@link javax.swing.Action}
   */
  public function addPopupMenuAction(action)
  {
    mode.addPopupMenuAction(action);
  }
  
  /**
   * Remove from popup menu.
   * 
   * @param action      a {@link javax.swing.Action}
   */
  public function removePopupMenuAction(action)
  {
    popup.remove(action);
  }
  
  /**
   * Install trigger for displaying popup menu.
   */
  public static var MOUSE_LISTENER =
    new ti.chimera.PopupTrigger( 
      new (function() extends ti.chimera.PopupTrigger.PopupListener() {
        
        public function showPopup(evt)
        {
          getCurrentFrame().showPopup( evt.getX(), evt.getY() );
        }
        
      })() );
  editorBuffer.addMouseListener(MOUSE_LISTENER);
  editorBuffer.addMouseMotionListener(MOUSE_LISTENER);
  
  /**
   * Show this frame's popup menu at the specified coordinates, relative to 
   * this component.
   * 
   * @param x   the x-coord
   * @param y   the y-coord
   */
  public function showPopup( x, y )
  {
    mode.getPopupMenu().show( getBuffer(), x, y );
  }
  
  /**
   * We keep track of the frame that currently has focus.  Many of the key-
   * bindings need access to the current frame.
   */
//  static var currentFrame;
  if( currentFrame == null )
    currentFrame = this;
  
  public static var FOCUS_LISTENER = 
    new (function() extends java.awt.event.FocusAdapter() {
      
      public function focusGained(evt)
      {
        var frame = evt.getSource().getFrame();
        if( frame != null )
          currentFrame = frame;
      }
      
    })();
  
  editorBuffer.addFocusListener(FOCUS_LISTENER);
  
  /**
   * Get the frame that currently has focus.  This can be used by extension
   * script code that needs access to the current frame.
   * 
   * @return a frame, or <code>null</code>
   */
  public static function getCurrentFrame()
  {
    return currentFrame;
  }
  
  /**
   * Get the frame with the specified name.
   * 
   * @param name         the name of the frame
   * @return a frame, or <code>null</code>
   */
  public static function getFrame(name)
  {
    throw new ProgrammingErrorException("unimplemented");
  }
  
  /**
   * Get the name of this buffer.
   * 
   * @return the string name of the frame
   */
//  public function getName()
//  {
//    throw new ProgrammingErrorException("unimplemented");
//  }
  
  /**
   * @deprecated use Mode#.addPopupMenuAction
   */
  public function addAction( name, toolTip, smallIcon, bigIcon, fxn )
  {
    mode.addPopupMenuAction( name, toolTip, smallIcon, bigIcon, fxn );
  }
  
  /**
   * Add UndoableEditListener for managing undo/redo...
   */
  getBuffer().getDocument().addUndoableEditListener(
    new (function() extends swing.event.UndoableEditListener() {
      public function undoableEditHappened(evt)
      {
        var edit = evt.getEdit();
        if( edit.isSignificant() )
        {
          undos[ undosIdx++ ] = edit;
          undosLastIdx = undosIdx;
        }
      }
    })() );
  
  {
    var tmp = file;
    file = null;
    setFile(tmp);
  }
}


/**
 * Debug code for doing some benchmarking and profiling...
 */
function foo()
{
  var currentTimeMillis = java.lang.System.currentTimeMillis;
  
  var frame = EditorFrame.getCurrentFrame();
  var buf   = frame.getBuffer();
  var data  = new DocumentArray( buf.getDocument() );
  var len   = data.length();
  
  var t = currentTimeMillis();
  
  for( var i=0; i<len; i++ )
    data[i];
  
  var str = "full doc:  time: " + (currentTimeMillis() - t) + "ms.. ";
  
  t = currentTimeMillis();
  
  for( var i=0; (i<len) && (i<10); i++ )
    data[i];
  
  str += "short doc:  time: " + (currentTimeMillis() - t) + "ms";
  
  pkg.output.writeln(str);
}




/**
 * This shouldn't be accessed directly by anything but the evalXXX() functions.
 * It is the scope that the evalXXX() functions evaluate in.
 */
var scope;
/* this throwing an exception hack is a way to get access to the current
 * scope.
 */
try
  throw new Exception("bogus");
catch(e)
  scope = e.stackFrameIterator().next().getScope();


/**
 * This is the primary means for loading extension script code, as well as a
 * way to just evaluate arbitrary script code.
 * 
 * @param file         the file to evaluate
 * @return the result of evaluating the file
 */
function evalFile(file)
{
  if( file.exists() && file.canRead() )
    return oscript.OscriptInterpreter.__eval( file, scope );
}

/**
 * Evaluate the named buffer.  If <code>name</code> is <code>null</code>,
 * prompt for a buffer name, defaulting to the current buffer.
 * 
 * @param name         the name of the buffer to evaluate
 * @return the result of evaluating the buffer
 */
function evalBuffer(name)
{
  if( name == null )
  {
    var frame = EditorFrame.getCurrentFrame();
    name = util.prompt("buffer name");
  }
  
  // XXX
  throw new ProgrammingErrorException("unimplemented");
}

/**
 * And once everything is kicked off, load the extension scripts...
 */
globalMode.runHooks("init");


// for now, this gives us a way to futz with the editor interactively:
function launchDebugger()
{
  var itr = pkg.system.currentStackFrame().iterator();
  itr.next();  // pop off currentStackFrame stack frame
  registry.getService("debugger").openDebugger( itr.next() );
}


globalMode.addKeyFxn( "ctrl shift E", "eval-mode", "enter eval mode", (function() {
  var history = new History();
  return function(evt) {
    var str = util.prompt( "evaluate", history );
    if( (str != null) && (str.length() > 0) )
    {
      if( (str[str.length()-1] != ";") &&
          (str[str.length()-1] != "}") )
        str += ";";
      
      var result;
      eval "result = " + str;
      
      str = "";
      
      EditorFrame.getCurrentFrame().setStatusLineText("result: " + result);
    }
  };
})() );

globalMode.addKeyFxn( "ESCAPE", "action-mode", "enter action mode", (function() {
  var history = new History();
  return function(evt) {
    var mode = EditorFrame.getCurrentFrame().getMode();
    var cs   = mode.getCompletionSet();
    var str  = util.prompt( "action", history, cs );
    if( (str != null) && (str.length() > 0) )
      mode.performAction(str);
  };
})() );


/**
 * Keyboard shortcuts are good, so install a keyboard-shortcut for displaying the popup.
 */
globalMode.addKeyFxn( "shift ESCAPE", "show-popup", "show the popup menu", function(evt) {
  
  var frame = EditorFrame.getCurrentFrame();
  var buf   = frame.getBuffer();
  var rect  = buf.modelToView( buf.getCaretPosition() );
  
  if( rect != null )
    frame.showPopup( rect.x, rect.y );
  
} );



/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

