/*=============================================================================
 *     Copyright Texas Instruments 2002. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * An escape-mode is used for times when you want to block normal input to the
 * editor buffer.  This class is an abstract class, and the subclasser must
 * implement a method <code>handleChar</code> which gets passed the character
 * that the user types.  By default this mode is setup to handle all key
 * presses by calling the <code>handleChar</code> method.  
 */
public function EscapeMode()
  extends pkg.editor.Mode(null)
{
  var BACK_SPACE = swing.KeyStroke.getKeyStroke("BACK_SPACE");
  var BACK_SPACE_ACTION = new pkg.util.ActionAdapter( "", null, null, function(evt) {
    try {
      handleBackSpace();
    } catch(e) {
      pkg.output.writeln("error: " + e);
    }
  } );
  
  inputMap.setParent( new (function() extends swing.InputMap() {
    
    public function get(key)
    {
      if( !key.isOnKeyRelease() )
      {
        // XXX probably could improve performance by caching ActionAdapters and
        // avoiding the extra GC load:
        if( pkg.editor.util.isValidChar( key.getKeyChar() ) )
          return new pkg.util.ActionAdapter( "", null, null, '{ try { handleChar( key.getKeyChar() ); } catch(e) { pkg.output.writeln("error: " + e); } } );
        else if( key.equals(BACK_SPACE) )
          return BACK_SPACE_ACTION;
        else
          return new pkg.util.ActionAdapter( "", null, null, '{ try { handleInvalidKey(key); } catch(e) { pkg.output.writeln("error: " + e); } } );
      }
    }
    
  })() );
  
  actionMap.setParent( new (function() extends swing.ActionMap() {
    
    public function get(key)
    {
      return key;
    }
    
  })() );

  var frame    = pkg.editor.EditorFrame.getCurrentFrame();
  var lastMode = frame.getMode();
  
  /**
   * The derived class should overload this, which is called for valid 
   * keyboard input.  This is only called for alphanumeric type characters, 
   * {@link #handleBackSpace} is called to handle backspace key, and 
   * {@link #handleInvalidKey} is called to handle other characters, such 
   * as arrow keys.
   * 
   * @param c    the character typed
   * @see #handleInvalidKey
   * @see #handleBackSpace
   */
  function handleChar(c)
  {}
  
  var escapeKeySet = new java.util.HashSet();
  escapeKeySet.add( pkg.util.getKeyStroke("ctrl C") );
  escapeKeySet.add( pkg.util.getKeyStroke("UP") );
  escapeKeySet.add( pkg.util.getKeyStroke("DOWN") );
  escapeKeySet.add( pkg.util.getKeyStroke("LEFT") );
  escapeKeySet.add( pkg.util.getKeyStroke("RIGHT") );
  
  /**
   * Handle "invalid" key presses, ie. keys that do not correspond to an
   * alphanumeric or punctuation character.
   * 
   * @param key  The key stroke
   * @see #handleChar
   * @see #handleBackSpace
   * @see pkg.editor.util.isValidChar
   */
  function handleInvalidKey(key)
  {
    if( escapeKeySet.contains(key) )
      leaveMode();
  }
  
  
  /**
   * The derived class should overload this, which is called when backspace is typed.
   * 
   * @see #handleChar
   * @see #handleInvalidKey
   */
  function handleBackSpace()
  {}
  
  /**
   * Action function called to do clean-up when we exit this escape-mode.
   */
  public function leaveMode()
  {
    frame.clearStatusLine();
    frame.setMode(lastMode);
  }
  
  frame.setMode(this);
}



/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */
