/******************************************************************************
 *     Copyright Texas Instruments 2002. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Base class for all modes.  A "mode" is intended to capture the customizations
 * to the editor specific to a certain content type.  It provides:
 * <ul>
 *   <li> key-handling
 *   <li> popup-menu options, in case the user of the editor-frame is
 *        using popup-menues
 *   <li> others?  perhaps menubar menues?  maybe popup-menus is just
 *        a specialization of the menubar?
 * </ul>
 * 
 * @if args.length() >= 1
 * @  param parent   the mode's parent
 * @fi
 * @if args.length() == 2
 * @  param name     the mode's name
 * @fi
 */
public function Mode( args... )
{
  var parent = null;
  var name = "noname";
  
  /**
   * completion-set for all action strings defined by this mode and it's
   * parent modes.
   */
  var completionSet = new (function() extends pkg.util.CompletionSet() {
    
    getCompletions = (function(super) {
      return function(str) {
        // quick-n-dirty implementation(TM):
        var list = new java.util.LinkedList();
        for( var itr=super(str); itr.hasNext(); )
          list.add( itr.next() );
        if( parent != null )
          for( var itr=parent.getCompletionSet().getCompletions(str); itr.hasNext(); )
            list.add( itr.next() );
        return list.iterator();
      };
    })(getCompletions);
    
  })();
  
  /**
   * Table mapping defined keys to comments.
   */
  var descriptionTable = new java.util.Hashtable();
  
  /**
   * The table mapping key to string representation of key sequence
   */
  var strTable = new java.util.Hashtable();
  
  /* 
   * Keystroke handling:
   */
  
  var actionMap = new swing.ActionMap();
  var inputMap  = new (function() extends swing.InputMap() {
    
    var cnt = 0;
    
    public var get = (function(super) {
      return function(key) {
        runAnyKeyFxns(key);
        cnt++;
        return super(key);
      };
    })(get);
    
  })();
  
  // override as needed:
  public function initializeFrame(frame)
  {
    if( parent != null )
      parent.initializeFrame(frame);
  }
  
  public function cleanupFrame(frame)
  {
    if( parent != null )
      parent.cleanupFrame(frame);
  }
  
  /**
   * Add a hook function, that will be invoked when the named hook is run, 
   * such as "new-frame", and stuff... the parameters passed to the hook
   * function are determined by the hook (which is documented where??)
   * 
   * @param name   the hook name
   * @param fxn    the function to call
   */
  public function addHook( name, fxn )
  {
    pkg.editor.hooks.addHook( this.name, name, fxn );
  }
  
  /**
   * Remove the specified hook function.
   * 
   * @param fxn    the function to remove
   */
  public function removeHook(fxn)
  {
    pkg.editor.hooks.removeHook( this.name, fxn );
  }
  
  /**
   * Run the specified hook.  This calls all the functions registered
   * with {@link #addHook} for the same hook <code>name</code>
   * 
   * @param name   the hook name
   * @param ...    remaining args are passed to hook function
   */
  public function runHooks( name, args... )
  {
    pkg.editor.hooks.runHooks.callAsFunction( [ this.name, name ].concat(args) );
    if( parent != null )
      parent.runHooks.callAsFunction( [name].concat(args) );
  }
  
  /**
   * Key Handlers that are run for any key.  This array is only
   * accessed from the AWT Event Thread, so no synchronization is
   * used.
   */
  var anyKeyFxns = [];
  
  /**
   * Call all registered any-key handlers.  This should only be called
   * from AWT Event Thread
   */
  function runAnyKeyFxns(key)
  {
    for( var i=0; i<anyKeyFxns.length(); i++ )
      anyKeyFxns[i](key);
  }
  
  /**
   * Add a key handler that gets called for any key.  The handler
   * is a function that takes as argument the key that it is handling.
   */
  public function addAnyKeyFxn(fxn)
  {
    swing.SwingUtilities.invokeLater( function() {
      anyKeyFxns[ anyKeyFxns.length() ] = fxn;
    } );
  }
  
  /**
   * Remove a key handler that gets called for any key.
   */
  public function removeAnyKeyFxn(fxn)
  {
    swing.SwingUtilities.invokeLater( function() {
      var tmp = [];
      for( var i=0; i<anyKeyFxns.length(); i++ )
        if( anyKeyFxns[i] != fxn )
          tmp[ tmp.length() ] = anyKeyFxns[i];
      anyKeyFxns = tmp;
    } );
  }
  
  /**
   * Get the name of this mode.
   */
  public function getName()
  {
    return name;
  }
  
  function setName(name)
  {
    this.name = name;
  }
  
  /**
   * Get the parent of this mode.
   */
  public function getParent()
  {
    return parent;
  }
  
  /**
   * Sometimes a mode needs to be re-parented... for example the
   * EditorFrame itself has a mode, for frame specific bindings;
   * when the EditorFrame's mode is changed, it simply reparents
   * it's own mode, in order to inherit the behavior of the mode
   * that is set; this way frame specific bindings are preserved.
   * 
   * @param parent  the parent to set
   */
  public function setParent(parent)
  {
    this.parent = parent;
    if( parent != null )
    {
      inputMap.setParent( parent.getInputMap() );
      actionMap.setParent( parent.getActionMap() );
    }
  }
  
  if( args.length() >= 2 )
    setName(args[1]);
  if( args.length() >= 1 )
    setParent(args[0]);
  
  /**
   * Get the {@link DocumentWriter} for this mode.  A mode may subclass
   * <code>DocumentWriter</code> to add additional functionality, but the 
   * constructor must still take same args as <code>DocumentWriter</code>
   * 
   * @return the <code>DocumentWriter</code>, or subclass, type
   */
  public function getDocumentWriterType()
  {
    if( parent != null )
      return parent.getDocumentWriterType();
    return DocumentWriter;
  }
  
  /**
   * Get the completion set, the set of completions for actions
   * defined for this mode.
   * 
   * @return the <code>CompletionSet</code>
   */
  public function getCompletionSet()
  {
    return completionSet;
  }
  
  /**
   * Get the input map associated with this mode.  See 
   * {@link javax.swing.InputMap}.
   */
  public function getInputMap()
  {
    return inputMap;
  }
  
  /**
   * Get the action map associated with this mode.  See
   * {@link javax.swing.ActionMap}.
   */
  public function getActionMap()
  {
    return actionMap;
  }
  
  /**
   * Add a handler for the specified key.  The string describing the
   * key sequence is as described in ${@link javax.swing.KeyStroke#getKeyStroke(String)}
   * 
   * @param str          the string describing the key sequence
   * @param key          the action key
   * @param description  the description describing the action, or <code>null</code>
   * @param action       the action
   */
  public function addKeyAction( str, key, description, action )
  {
    if( description != null )
      descriptionTable.put( key, description );
    
    completionSet.add(key);
    
    if( str != null )
    {
      strTable.put( key, str );
      getInputMap().put( pkg.util.getKeyStroke(str), key );
    }
    
    getActionMap().put( key, action );
  }
  
  /**
   * Add a handler function for the specified key.  The string describing the
   * key sequence is as described in ${@link javax.swing.KeyStroke#getKeyStroke(String)}
   * 
   * @param str          the string describing the key sequence
   * @param key          the action key
   * @param description  the description describing the action, or <code>null</code>
   * @param fxn          the function to call
   */
  public function addKeyFxn( str, key, description, fxn )
  {
    addKeyAction( str, key, description, new pkg.util.ActionAdapter(str,null,null,fxn) );
  }
  
  /**
   * Run the specified action.
   * 
   * @param key          the action key
   */
  public function performAction(key)
  {
    var a = getActionMap().get(key);
    
    if( a == null )
    {
      pkg.output.writeln("no action defined for \"" + key + "\"");
      return;
    }
    
    a.actionPerformed(null);  // XXX shouldn't pass null
  }
  
  /**
   * Return an iterator of the keys defined in this mode.
   */
  public function getKeys()
  {
    return descriptionTable.keySet().iterator();
  }
  
  /**
   * Get the string representation of the keystroke for the 
   * specified action defined in this mode.
   * 
   * @param key          the action key
   * @return the string representation of the keystroke
   */
  public function getKeyString(key)
  {
    return strTable.get(key);
  }
  
  /**
   * Get the help for the specified action defined in this mode.
   * 
   * @param key          the action key
   * @return the description of the key, or <code>null</code>
   */
  public function getKeyDescription(key)
  {
    return descriptionTable.get(key);
  }
  
  /* 
   * Popup Menu handling:
   */
  
  var popupMenuActionFxnList = new java.util.LinkedList();
  
  /**
   * Add a function, which is called when popup menu is displayed, and returns 
   * an array of zero or more actions to include in the popup menu (in the 
   * order returned).  This lets decisions about building the popup menu take
   * place when the popup menu is built.
   * <p>
   * (note, to be consistent with {@link #addPopupMenuAction}, you can also
   * return an array of [name,toolTip,smallIcon,bigIcon,fxn] as an alternative
   * to an action.)
   * 
   * @param fxn   the function to call when popup menu is built
   */
  public function addPopupMenuActionFxn(fxn)
  {
    popupMenuActionFxnList.add(fxn);
  }
  
  /**
   * Add an action to the pop-up menu.
   * 
   * @if args.length() == 1
   * @  param action    the action
   * @else
   * @  param name      the string name
   * @  param toolTip   a string float-over tool-tip, or <code>null</code>
   * @  param smallIcon a small icon, or <code>null</code>
   * @  param bigIcon   large icon, or <code>null</code>
   * @  param fxn       the function to handle the ActionEvent
   * @fi
   */
  public function addPopupMenuAction(args...)
  {
    if( args.length() == 1 )
    {
      var action = args[0];
      addPopupMenuActionFxn( '{ return [action]; } );
    }
    else
    {
      callee( arr2action(args) );
    }
  }
  
  private function arr2action(args)
  {
    var name = args[0];
    var toolTip = args[1];
    var smallIcon = args[2];
    var bigIcon = args[3];
    var fxn  = args[4];
    
    if( (smallIcon != null) && (smallIcon instanceof String) )
      smallIcon = pkg.util.getImageIcon(smallIcon);
    
    if( (bigIcon != null) && (bigIcon instanceof String) )
      bigIcon = pkg.util.getImageIcon(bigIcon);
    
    return new pkg.util.ActionAdapter( name, toolTip, smallIcon, fxn );
  }
  
  /**
   * close-action is handled specially... it (drumroll, please...) closes the
   * editor frame!
   */
  private var closeAction = 
    new pkg.util.ActionAdapter( "Close", null, null, function(evt) {
      pkg.editor.EditorFrame.getCurrentFrame().close();
    } );
  
  /**
   * Get the popup-menu.
   * 
   * @return a swing.JPopupMenu
   */
  public function getPopupMenu()
  {
    var popupMenu = new swing.JPopupMenu();
    
    // use a popup menu listener to replace the caret with a bogus
    // caret, to prevent the caret from handling the mouse events
    // and changing the text selection while the popup is visible
    popupMenu.addPopupMenuListener( new (function() extends swing.event.PopupMenuListener() {
      
      var savedCaret = null;
      var savedMark;
      var savedDot;
      
      public function popupMenuCanceled(evt) { /*no-op*/ }
      
      public function popupMenuWillBecomeVisible(evt)
      {
        var tc = getJTextComponent( evt.getSource().getInvoker() );
        if( tc != null )
        {
          savedCaret = tc.getCaret();
          savedMark  = savedCaret.getMark();
          savedDot   = savedCaret.getDot();
          tc.setCaret(bogusCaret);
        }
      }
      
      public function popupMenuWillBecomeInvisible(evt)
      {
        var tc = getJTextComponent( evt.getSource().getInvoker() );
        if( tc != null )
        {
          tc.setCaret(savedCaret);
          savedCaret.setDot(savedMark);
          savedCaret.moveDot(savedDot);
          savedCaret = null;
        }
      }
      
    })() );
    
    populatePopupMenu(popupMenu);
    
    // handle "Close" as special case, to make it be at the bottom
    // of the popup menu:
    popupMenu.add(closeAction);
    
    return popupMenu;
  }
  
  // only public so we can call parent.populatePopupMenu()
  public function populatePopupMenu(popupMenu)
  {
    if( parent != null )
      parent.populatePopupMenu(popupMenu);
    
    for( var fxn : popupMenuActionFxnList )
    {
      var actions = fxn();
      if( actions != null )
      {
        for( var action : actions )
        {
          if( action instanceof Array )
            action = arr2action(action);
          popupMenu.add(action);
        }
      }
    }
    
    if( popupMenuActionFxnList.size() > 0 )
      popupMenu.addSeparator();
  }
  
  static function getJTextComponent(c)
  {
    if( c == null )
      return null;
    else if( c instanceof swing.text.JTextComponent )
      return c;
    else
      return callee( c.getParent() );
  }
  static var bogusCaret = new (function() extends swing.text.Caret() {
    
    public function install(c) {}
    public function deinstall(c) {}
    public function paint(g) {}
    public function addChangeListener(l) {}
    public function removeChangeListener(l) {}
    public function isVisible() { return true; }
    public function setVisible(v) {}
    public function isSelectionVisible() { return true; }
    public function setSelectionVisible(v) {}
    public function setMagicCaretPosition(p) {}
    public function getMagicCaretPosition() { return null; }
    public function setBlinkRate(rate) {}
    public function getBlinkRate() { return 0; }
    public function getDot() { return 0; }
    public function getMark() { return 0; }
    public function setDot(dot) {}
    public function moveDot(dot) {}
    
  })();
}



/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */
