/******************************************************************************
 *     Copyright Texas Instruments 2002. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//pkg.output.writeln("loading pkg.editor.cMode");

mixin pkg.editor;

// move these somewhere else:
function min(args...)
{
  var v = args[0];
  for( var i=1; i<args.length(); i++ )
    if( args[i] < v )
      v = args[i];
  return v;
}

function max(args...)
{
  var v = args[0];
  for( var i=1; i<args.length(); i++ )
    if( args[i] > v )
      v = args[i];
  return v;
}


public const var cMode = new (function() extends Mode( globalMode, "c" ) {
  
  /**
   * Is the current cursor position in the current buffer in a multi-
   * line comment?
   */
  public function inMultiLineComment()
  {
    return util.searchBackward("/*") > util.searchBackward("*/");
  }
  
  /**
   * Is the current cursor position in the current buffer in a
   * single line comment?
   */
  public function inSingleLineComment()
  {
    var frame = EditorFrame.getCurrentFrame();
    var buf   = frame.getBuffer();
    var off   = buf.getCaretPosition();
    var start = swing.text.Utilities.getRowStart( buf, buf.getCaretPosition() );
    
    if( start < 0 )
      start = 0;
    
    // the extra space is to give the parser a chance to recognize
    // the comment token:
    return util.searchForwardFromTo( "// ", start, off+1 ) != -1;
  }
  
  /**
   * Increase the indent level of the selected text (or current line) 
   * by one
   */
  public function increaseIndentLevel()
  {
    var frame = EditorFrame.getCurrentFrame();
    var buf   = frame.getBuffer();
    var doc   = buf.getDocument();
    var caret = buf.getCaret();
    var start = min( caret.getDot(), caret.getMark() );
    var end   = max( caret.getDot(), caret.getMark() );
    
    start = swing.text.Utilities.getRowStart( buf, start );
    
    var inclusive = start == end;
    
    while( (inclusive && (start <= end)) || (start < end) )
    {
      doc.insertString( start, INDENT, null );
      
      // adjust end position to componsate for inserted indent:
      end += INDENT_SIZE;
      
      start = swing.text.Utilities.getRowEnd( buf, start ) + 1;
    }
  }
  
  /**
   * Decrease the indent level of the selected text (or current line)
   * by one.
   */
  public function decreaseIndentLevel()
  {
    var frame = EditorFrame.getCurrentFrame();
    var buf   = frame.getBuffer();
    var doc   = buf.getDocument();
    var caret = buf.getCaret();
    var start = min( caret.getDot(), caret.getMark() );
    var end   = max( caret.getDot(), caret.getMark() );
    
    start = swing.text.Utilities.getRowStart( buf, start );
    
    var inclusive = start == end;
    
    while( (inclusive && (start <= end)) || (start < end) )
    {
      // we want to remove *up to* INDENT_SIZE characters, but no
      // non-whitespace chars:
      var removed;
      for( removed=0; removed<INDENT_SIZE; removed++ )
      {
        if( doc.getText( start, 1 ) != " " )
          break;
        doc.remove( start, 1 );
      }
      
      // adjust end position to componsate for removed indent:
      end -= removed;    
      
      start = swing.text.Utilities.getRowEnd( buf, start ) + 1;
    }
  }
  
  /**
   * Comment out selected text (or current line)
   */
  public function commentRegion()
  {
    var frame = EditorFrame.getCurrentFrame();
    var buf   = frame.getBuffer();
    var doc   = buf.getDocument();
    var caret = buf.getCaret();
    var start = min( caret.getDot(), caret.getMark() );
    var end   = max( caret.getDot(), caret.getMark() );
    
    start = swing.text.Utilities.getRowStart( buf, start );
    
    var inclusive = start == end;
    
    while( (inclusive && (start <= end)) || (start < end) )
    {
      if( doc.getText( start, 2 ) != "//" )
      {
        doc.insertString( start, "//", null );
        
        // adjust end position to componsate for inserted indent:
        end += "//".length();
      }
      
      start = swing.text.Utilities.getRowEnd( buf, start ) + 1;
    }
  }
  
  /**
   * Uncomment selected text (or current line)
   */
  public function uncommentRegion()
  {
    var frame = EditorFrame.getCurrentFrame();
    var buf   = frame.getBuffer();
    var doc   = buf.getDocument();
    var caret = buf.getCaret();
    var start = min( caret.getDot(), caret.getMark() );
    var end   = max( caret.getDot(), caret.getMark() );
    
    start = swing.text.Utilities.getRowStart( buf, start );
    
    var inclusive = start == end;
    
    while( (inclusive && (start <= end)) || (start < end) )
    {
      if( doc.getText( start, 2 ) == "//" )
      {
        doc.remove( start, 2 );
        
        // adjust end position to componsate for removed indent:
        end -= "//".length();
      }
      
      start = swing.text.Utilities.getRowEnd( buf, start ) + 1;
    }
  }
  
  /**
   * Insert the specified open and close tags before and after the
   * cursor or selected text.  The cursor is positioned immediately
   * before the close tag.
   * 
   * @param open   the open tag
   * @param close  the close tag
   */
  public function insertTags( open, close )
  {
    var frame = EditorFrame.getCurrentFrame();
    var buf   = frame.getBuffer();
    var caret = buf.getCaret();
    var off   = min( caret.getDot(), caret.getMark() );
    var text  = buf.getSelectedText();
    
    if( text == null )
      text = "";
    
    // XXX maybe replaceSelection should always be atomic, but for
    // now we go thru the call gate here:
    buf.getDocument().performUndoableSequence( function() {
      buf.replaceSelection( open + text + close );
    } );
    caret.setDot( off + (open + text).length() );
  }
  
  public const var INDENT_SIZE = 2;
  public const var INDENT = "  ";
  
  /**
   * Get the {@link DocumentWriter} for this mode.  A mode may subclass
   * <code>DocumentWriter</code> to add additional functionality, but the 
   * constructor must still take same args as <code>DocumentWriter</code>
   * 
   * @return the <code>DocumentWriter</code>, or subclass, type
   */
  public function getDocumentWriterType()
  {
    return CModeDocumentWriter;
  }
  
  /**
   * A document writer that adds utilities for writing comment blocks,
   * etc.
   */
  protected function CModeDocumentWriter( frame, off, len )
    extends DocumentWriter( frame, off, len )
  {
    /**
     * Dump a (potentially) multi-line comment string.  This handles word-
     * wrap, etc.
     * 
     * @param str   the comment string
     */
    public function comment(str)
    {
      dumpln("/**");  // */
      
      var max = 72 - (2*ind);
      var stok = new java.util.StringTokenizer( str, " \n", true );
      var line = " * ";
      
      function flushline()
      {
        dumpln(line);
        line = " * ";
      }
      
      while( stok.hasMoreElements() )
      {
        if( line.length() > max )
          flushline();
        var token = stok.nextElement();
        if( token == "\n" )
          flushline();
        else if( (line != " * ") || (token != " ") )
          line += token;
      }
      flushline();
      
      dumpln(" */");
    }
  }
  
})();

/* setup any c-mode specific keybinding here */

cMode.addKeyFxn( "ctrl shift EQUALS", "increase-indent-level",
                 "increase the indent level of the selected text",
                 function(evt) {
                   cMode.increaseIndentLevel();
                 } );

cMode.addPopupMenuAction( "Increase Indent",
                          "Increase the indent level of selected text",
                          "shift_right.gif",
                          null,
                          function(evt) { cMode.increaseIndentLevel(); } );

cMode.addKeyFxn( "ctrl MINUS",
                 "decrease-indent-level",
                 "decrease the indent level of the selected text",
                 function(evt) {
                   cMode.decreaseIndentLevel();
                 } );

cMode.addPopupMenuAction( "Decrease Indent",
                          "Decrease the indent level of selected text",
                          "shift_left.gif",
                          null,
                          function(evt) { cMode.decreaseIndentLevel(); } );

cMode.addKeyFxn( "ctrl shift C",
                 "comment-region",
                 "comment out selected region or current line",
                 function(evt) {
                   cMode.commentRegion();
                 } );

cMode.addPopupMenuAction( "Comment Region",
                          "comment out selected region or current line",
                          "comment.gif",
                          null,
                          function(evt) { cMode.commentRegion(); } );


cMode.addKeyFxn( "ctrl shift U",
                 "uncomment-region",
                 "uncomment out selected region or current line",
                 function(evt) {
                   cMode.uncommentRegion();
                 } );

cMode.addPopupMenuAction( "Uncomment Region",
                          "uncomment out selected region or current line",
                          "uncomment.gif",
                          null,
                          function(evt) { cMode.uncommentRegion(); } );


cMode.addKeyFxn( "ctrl B",
                 "bold-tag",
                 "apply bold (&lt;b&gt;&lt;/b&gt;) tags to the selected text",
                 function(evt) { cMode.insertTags("<b>","</b>"); } );

cMode.addKeyFxn( "ctrl I",
                 "italic-tag",
                 "apply italic (&lt;i&gt;&lt;/i&gt;) tags to the selected text",
                 function(evt) { cMode.insertTags("<i>","</i>"); } );

cMode.addKeyFxn( "ctrl T",
                 "code-tag",
                 "apply code (&lt;code&gt;&lt;/code&gt;) tags to the selected text",
                 function(evt) { cMode.insertTags("<code>","</code>"); } );

//cMode.addKeyFxn( "ctrl shift U",
//                 "link-tag",
//                 "apply link ({@link }) tag to the selected text",
//                 function(evt) { cMode.insertTags("{@link ","}"); } );


/******************************************************************************
 * Newline-action, automatically indents to the same indent level as previous line.
 * Eventually this will do something cool, like syntax based indenting or something.
 */
cMode.addKeyFxn( "ENTER", swing.text.DefaultEditorKit.insertBreakAction, null, (function() {
    
    var ws = util.isa(" \t");
    
    return function(evt) {
      
      var frame = EditorFrame.getCurrentFrame();
      var buf   = frame.getBuffer();
      var data  = new DocumentArray( buf.getDocument() );
      
      // is this a legal offset into the document?
      function okOff(off)
      {
        return (0 <= off) && (off < data.length());
      }
      
      // is off a legal offset, and is data[off] == c?
      function check( off, c )
      {
        return (0 <= off) && (off < data.length()) && (data[off] == c);
      }
      
      // figure out indent for the line containing the specified offset:
      function findIndent(off)
      {
        off = swing.text.Utilities.getRowStart( buf, off );
        var str = "";
        while( okOff(off) && ws(data[off]) )
          str += data[off++];
        return str;
      }
      
      var str;
      var end;
      var start;
      
      /**
       * Set <code>start</code>, <code>end</code> based on the "cursor position"
       * offset passed in... also set <code>str</code> based on the amount of
       * whitespace from the current line.
       */
      function reset(off)
      {
        end = off;
        start = swing.text.Utilities.getRowStart( buf, end );
        
        str = findIndent(start);
        start  += str.length();
        
        /* Back up end to the last non-whitespace on the current line:
         */
        while( okOff(--end) && ws(data[end]) ) {}
      }
      
      reset( buf.getCaretPosition() );
      
      /************************************************************************
       * If it wasn't obvious, the order of the checks determines the
       * order of precedence of the rules:
       */
      var rules = [
        
        /**********************************************************************
         * handle multi-line comments, by inserting a "* " at the beginning 
         * of the line and possibly add extra space to line up the "*"'s
         */
        function() {
          if( ! cMode.inMultiLineComment() )
            return false;
          
          if( check( start, "/" ) )
            str += " ";
          str += "* ";
          return true;
        },
        
        /**********************************************************************
         * if last non-whitespace char is an open brace, then increase
         * indent by one level.
         */
        function() {
          if( ! (check( end, "{" ) || check( end, "(" )) )
            return false;
          
          // if next is close brace, insert EOL and indenting *after* cursor:
          if( check( end + 1, "}" ) || check( end + 1, ")" ) )
          {
            /* WARNING:  replaceSelection will screw up an offsets after the
             * cursor... we aren't looking at anything after the cursor besides
             * this one check, but it is something to be careful of if you are
             * mucking with this code... definately don't return false after
             * this point!
             */
            buf.replaceSelection("\n" + str);
            buf.setCaretPosition( buf.getCaretPosition() - 1 - str.length() );
          }
          
          str += cMode.INDENT;
          return true;
        },
        
        /**********************************************************************
         * if last non-whitespace char is a close paren, then revert
         * intent level to that of line with matching open paren.
         * 
         * if there is an unmatched open paren on the current line, then
         * increase the indent level.
         */
        function() {
          
          for( var i=end; i>=start; i-- )
          {
            if( check(i,")") )
            {
              var open = util.searchBackwardForOpenFrom( "(", ")", i, 200 );
              if( open == -1 )
                return false;
              
              reset(open);
              return callee();
            }
            else if( check(i,"(") )
            {
              // move forward to first non-ws character after open paren:
              while( check( ++i, " " ) ) {}
              
              // add appropriate number of spaces to line us up properly:
              var col = i - swing.text.Utilities.getRowStart( buf, i );
              str = "";
              for( var i=0; i<col; i++ )
                str += " ";
              
              return true;
            }
          }
          return false;
        },
        
        /**********************************************************************
         * if after a multi-line comment, back up indent level to match before
         * the start of the multi-line comment:
         */
        function() {
          if( !check( start, "*" ) )
            return false;
          
          str = str.substring( 0, max( str.length() - 1, 0 ) );
          return true;
        }
      ];
      
      for( var i=0; i<rules.length(); i++ )
        if( rules[i]() )
          break;
      
      buf.replaceSelection( "\n" + str );
    };
    
  })() );

/******************************************************************************
 * auto-insert-space-before-close-paren... if there is a space following the
 * matching open paren, then ensure there is one space preceeding the close 
 * paren.  The check is only made if there is only whitespace between the
 * cursor and the next close paren.
 */
cMode.addAnyKeyFxn( function(key) {
    
    var buf   = EditorFrame.getCurrentFrame().getBuffer();
    var doc   = buf.getDocument();
    var data  = new DocumentArray(doc);
    var off   = buf.getCaretPosition();
    
    function check( off, c )
    {
      return (0 <= off) && (off < data.length()) && (data[off] == c);
    }
    
    var keyChar = key.getKeyChar();
    
    if( !util.isValidChar(keyChar) )
      return;
    
    if( keyChar == " " )
    {
      if( check( off, " " ) && check( off+1, ")" ) )
        doc.remove( off, 1 );
    }
    else if( keyChar != ")" )
    {
      if( check( off, ")" )  )
      {
        // search backwards to find matching open paren:
        off = util.searchBackwardForOpenFrom( "(", ")", off );
        
        // if matching open paren is trailed by space, insert space
        // before close paren:
        if( check( off+1, " " ) )
        {
          buf.replaceSelection(" ");
          buf.setCaretPosition( buf.getCaretPosition() - 1 );
        }
      }
    }
    
  } );

/******************************************************************************
 * Setup TAB handler.  Increase indent level, according to this
 * algorithm:
 * <pre>
 *   if no whitespace between startLine(off)..off
 *     insert " "
 *     while (++off % indentSize) != 0 
 *       insert " "
 * </pre>
 */
cMode.addKeyFxn( "TAB", swing.text.DefaultEditorKit.insertTabAction, null, (function() {
    
    var ws = util.isa(" \t");
    
    return function(evt) {
      
        var frame = EditorFrame.getCurrentFrame();
        var buf   = frame.getBuffer();
        var doc   = buf.getDocument();
        var data  = new DocumentArray(doc);
        var caret = buf.getCaret();
        var off   = caret.getDot();
        
        if( off != caret.getMark() )
        {
          buf.replaceSelection("");
          return;
        }
        
        var start = swing.text.Utilities.getRowStart( buf, off );
        
        /* if no non-whitespace chars before cursor on this line,
         * then increase indent level.
         */
        for( var i=start; i<off; i++ )
          if( !ws(data[i]) )
            return;
        
        var str = " ";
        var idx = off - start;   // convert to column number
        while( (++idx % cMode.INDENT_SIZE) != 0 )
          str += " ";
      
      doc.insertString( off, str, null );
    };
  
  })() );

/******************************************************************************
 * Setup BACKSPACE handler.  Decrease indent level, according to this
 * algorithm:
 * <pre>
 *   if no whitespace between startLine(off)..off
 *     delete 1
 *     while (--off % indentSize) != 0 
 *       delete 1
 * </pre>
 */
cMode.addKeyFxn( "BACK_SPACE", swing.text.DefaultEditorKit.deletePrevCharAction, null, (function() {
    
    var ws = util.isa(" \t");
    
    return function(evt) {
        
        var frame = EditorFrame.getCurrentFrame();
        var buf   = frame.getBuffer();
        var doc   = buf.getDocument();
        var data  = new DocumentArray(doc);
        var caret = buf.getCaret();
        var off   = caret.getDot();
        
        if( off != caret.getMark() )
        {
          buf.replaceSelection("");
          return;
        }
        
        var start = swing.text.Utilities.getRowStart( buf, off );
        
        /* if no non-whitespace chars before cursor on this line,
         * then decrease indent level.
         */
        for( var i=start; i<off; i++ )
          if( !ws(data[i]) )
            return doc.remove( off-1, 1 );
        
        var len = 1;
        var idx = off - start;   // convert to column number
        while( (idx > 0) && (--idx % cMode.INDENT_SIZE) != 0 )
          len++;
    
        doc.remove( off-len, len );
      };
    
  })() );

/******************************************************************************
 */
cMode.addKeyFxn( "ctrl shift 8", "insert-comment-divider", 
                 "if in a multiline comment, insert * characters out to column 80",
                 function(evt) {
                   
                   if( cMode.inMultiLineComment() )
                   {
                     var buf   = EditorFrame.getCurrentFrame().getBuffer();
                     var off   = buf.getCaretPosition();
                     
                     var col = off - swing.text.Utilities.getRowStart( buf, off );
                     var str = "";
                     while( ++col < 80 )
                       str += "*";
                     
                     buf.replaceSelection(str);
                   }
                   
                 } );

/******************************************************************************
 * Setup key handlers to autocomplete braces/brackets/parents.  The open
 * and close strings should both be of length=1.
 * 
 * @param open     the opening string
 * @param close    the closing string, to automatically insert
 * @param name     name describing the key handler to install
 */
function installMatchers( open, close, name )
{
  cMode.addKeyFxn( "typed " + open, "insert-" + name + "-left", null, function(evt) {
      
      cMode.insertTags( open, close );
      
    } );
  
  cMode.addKeyFxn( "typed " + close, "insert-" + name + "-right", null, function(evt) {
      
      var frame = EditorFrame.getCurrentFrame();
      var buf   = frame.getBuffer();
      var doc   = buf.getDocument();
      var off   = buf.getCaretPosition();
      
      if( doc.getText(off,1) == close )
        buf.getCaret().setDot(off+1);
      else
        buf.replaceSelection(close);
      
    } );
}

installMatchers( "{", "}", "brace" );
installMatchers( "(", ")", "paren" );
installMatchers( "[", "]", "bracket" );


// handles block jumping up/down
function makeUpDownBlockJumper( inc, nrows, shift )
{
  function getRowStart( buf, off )
  {
    if( off < 0 )
      return 0;
    return swing.text.Utilities.getRowStart( buf, off );
  }
  
  function getRowEnd( buf, off )
  {
    var len = buf.getDocument().getLength();
    if( off >= len )
      return len - 1;
    return swing.text.Utilities.getRowEnd( buf, off );
  }
  
  var findNextLine;
  if( inc < 0 )
    findNextLine = function( buf, off ) {
      return getRowStart( buf, off ) - 1;
    };
  else
    findNextLine = function( buf, off ) {
      return getRowEnd( buf, off ) + 1;
    };
  
  return function(evt) {
      
      var buf = EditorFrame.getCurrentFrame().getBuffer();
      var off = buf.getCaretPosition();
      var len = buf.getDocument().getLength();
      
      // save current column for later:
      var col = off - getRowStart( buf, off );
      
      for( var i=0; i<nrows; i++ )
        off = findNextLine( buf, off );
      
      // we don't know where in the line we are:
      off = getRowStart( buf, off );
      var lastCol = getRowEnd( buf, off ) - off;
      col = (col > lastCol) ? lastCol : col;
      off += col;
      
      if( off < 0 )
        off = 0;
      else if( off >= len )
        off = len - 1;
      
      if(shift)
        buf.getCaret().moveDot(off);
      else
        buf.getCaret().setDot(off);
    };
}

// handles block jumping left/right.  Sort of like word jumping, but jumps
// matched: {} () []
function makeLeftRightBlockJumper( inc, shift )
{
  var SEPERATORS  = " ,.+-!%^&*/=\n\\";
  var JUMP_START  = ["(","[","{"];
  var JUMP_END    = [")","]","}"];
  var QUOTES      = ["\""];
  
  var ws = util.isa(SEPERATORS);
  var js;
  var je;
  var quote = util.isa(QUOTES);
  
  if( inc > 0 )
  {
    js = util.isa(JUMP_START);
    je = util.isa(JUMP_END);
  }
  else
  {
    js = util.isa(JUMP_END);
    je = util.isa(JUMP_START);
  }
  
  return function(evt) {
      
      // speedup:
      var search;
      {
        var doc = EditorFrame.getCurrentFrame().getBuffer().getDocument();
        var data = doc.getText( 0, doc.getLength() );
        
        search = function(match) {
          return util.search( match, off, data, inc );
        };
      }
      
      function okOff()
      {
        return (0 <= off) && (off < len);
      }
      
      var buf   = EditorFrame.getCurrentFrame().getBuffer();
      var off   = buf.getCaretPosition();
      var data  = new DocumentArray( buf.getDocument() );
      var len   = data.length();
      var level = 0;
      
      if( inc < 0 )
        off += inc;
      
      if( js(data[off]) )
      {
        level++;
        off += inc;
      }
      else
      {
        while( okOff() && ws(data[off]) )
          off += inc;
      }
      
      var inQuote = false;  // in quote after open/close quote before matching close/open quote
      
      while( okOff() )
      {
        // speedup:
if(false)
if( inc > 0 ) //  XXXX hack, something is broken!
        if( level > 0 )
          off = min( search(JUMP_START),
                     search(JUMP_END),
                     search(QUOTES) );
        
        // original logic:
        var c = data[off];
        
        //    if we encounter an unescaped quote, change quote state:
        if( quote(c) )
        {
          var tmpOff = off - 1;
          while( (tmpOff > 0) && (data[tmpOff] == "\\") )
            tmpOff--;
          var numSlash = off - 1 - tmpOff;
          var escaped  = (numSlash % 2) == 1;
          if( !escaped )
            inQuote = !inQuote;
        }
        
        //    if not in quote state, process jump-start/jump-end characters:
        if( !inQuote )
        {
          if( js(c) )
          {
            if( level <= 0 )
            {
              off -= inc;
              break;
            }
            level++;
          }
          else if( je(c) )
          {
            level--;
            if( level < 0 )
              off -= inc;
            if( level <= 0 )
              break;
          }
          else if( (level <= 0) && ws(c) )
          {
            off -= inc;
            break;
          }
        }
        
        off += inc;
      }
      
      if( inc > 0 )
        off += inc;
      
      if( okOff() )
      {
        if(shift)
          buf.getCaret().moveDot(off);
        else
          buf.getCaret().setDot(off);
      }
    };
}

cMode.addKeyFxn( "ctrl LEFT",  "caret-previous-block", "jump left",    makeLeftRightBlockJumper( -1, false ) );
cMode.addKeyFxn( "ctrl RIGHT", "caret-next-block",     "jump right",   makeLeftRightBlockJumper(  1, false ) );
cMode.addKeyFxn( "shift ctrl LEFT",  "selection-previous-block", null, makeLeftRightBlockJumper( -1, true ) );
cMode.addKeyFxn( "shift ctrl RIGHT", "selection-next-block",     null, makeLeftRightBlockJumper(  1, true ) );
cMode.addKeyFxn( "ctrl UP",    "caret-up-block",       "jump up",      makeUpDownBlockJumper( -1, 5, false ) );
cMode.addKeyFxn( "ctrl DOWN",  "caret-down-block",     "jump down",    makeUpDownBlockJumper(  1, 5, false ) );
cMode.addKeyFxn( "shift ctrl UP",    "selection-up-block",       null, makeUpDownBlockJumper( -1, 5, true ) );
cMode.addKeyFxn( "shift ctrl DOWN",  "selection-down-block",     null, makeUpDownBlockJumper(  1, 5, true ) );



/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */
