/******************************************************************************
 *     Copyright Texas Instruments 2004. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * The hooks database.  While hooks are conceptually related to mode, they
 * are stored external to the mode.  This is done to make it possible to
 * add a hook without actually causing the mode to be loaded (or even)
 * require the mode to exist yet.  This is mostly an advantage at startup,
 * because it lets us avoid loading a mode until it is actually needed, but
 * it also serves to decouple the system somewhat.
 */
public const var hooks = new '{
  
  /**
   * maps  mode-name => hook-table (where hook-table maps hook-name to hook-list)
   */
  private var modeTable = new java.util.Hashtable();
  
  private function getHookTable(modeName)
  {
    var hookTable = modeTable.get(modeName);
    if( hookTable == null )
    {
      hookTable = new java.util.Hashtable();
      modeTable.put( modeName, hookTable );
    }
    return hookTable;
  }
  
  /**
   * Add a hook function, that will be invoked when the named hook is run, 
   * such as "new-frame", and stuff... the parameters passed to the hook
   * function are determined by the hook (which is documented where??)
   * 
   * @param modeName   the mode the hook is applicable to
   * @param hookName   the hook name
   * @param fxn        the function to call
   */
  public function addHook( modeName, hookName, fxn )
  {
    var hookTable = getHookTable(modeName);
    var list = hookTable.get(hookName);
    
    if( list == null )
    {
      list = new java.util.LinkedList();
      hookTable.put( hookName, list );
    }
    
    list.add(fxn);
  }
  
  /**
   * Remove the specified hook function.
   * 
   * @param modeName   the mode the hook is applicable to
   * @param fxn        the function to remove
   */
  public function removeHook( modeName, fxn )
  {
    var hookTable = getHookTable(modeName);
    for( var list : hookTable.values() )
      list.remove(fxn);
  }
  
  /**
   * Run the specified hook.  This calls all the functions registered
   * with {@link #addHook} for the same hook <code>name</code>
   * <p>
   * This is only to be called by Mode#runHooks(), since this function
   * doesn't know who the parent mode is, and therefore can't run
   * parent hooks.
   * 
   * @param modeName   the mode the hook is applicable to
   * @param hookName   the hook name
   * @param ...        remaining args are passed to hook function
   */
  public function runHooks( modeName, hookName, args... )
  {
    var hookTable = getHookTable(modeName);
    
    var list = hookTable.get(hookName);
    if( list != null )
      for( var fxn : list.toArray() ) // toArray in case hook fxn add/removes hooks
        fxn.callAsFunction(args);
  }
  
  
}();