/*=============================================================================
 *     Copyright Texas Instruments 2002. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

mixin pkg.editor;

public const var util = new (function() {
  
  /**
   * A utility function that returns a function that determines if a character is a
   * member of the set.  The <code>set</code> is an array of characters (or a String).
   * 
   * @param set      set of characters
   * @return a function that takes a single character, and returns <code>true</code> iff
   * the character is a member of <code>set</code>
   */
  public function isa(set)
  {
    // make sure the set is an array of strings, rather than just a single string,
    // for better performance:
    {
      var tmp = [];
      for( var i=0; i<set.length(); i++ )
        tmp[i] = set[i];
      set = tmp;
    }
    
    return function(c) {
      for( var i=0; i<set.length(); i++ )
        if( c == set[i] )
          return true;
      return false;
    };
  }
  
  public static const var OTHER_LEGAL_CHARS = "~`!@#$%^&*()_-+={}[]\\|/:;\"\'<>,.?";
  
  /**
   * Is the specified character a "typeable" character?
   * 
   * @param c      a key-char (as in KeyStroke#getKeyChar)
   * @return a boolean
   */
  public function isValidChar(c)
  {
    return ( java.lang.Character.isLetterOrDigit(c) ||
             java.lang.Character.isWhitespace(c) || 
             (function() { 
               for( var i=0; i<OTHER_LEGAL_CHARS.length(); i++ )
                 if( c == OTHER_LEGAL_CHARS[i] )
                   return true;
               return false;
             })() );
  }
  
  /**
   * Prompt the user for a line number, and then go to that 
   * line
   */
  public function promptAndGotoLine()
  {
    try
    {
      var line = prompt("line");
      if( line != "" )
        EditorFrame.getCurrentFrame().
          scrollToLine( line.castToExactNumber() );
    }
    catch(e) { /* ignore bad input */ }
  }
  
  /**
   * Ask the user a multiple choice question.  Each of the
   * choices should be a one character mnemonic.
   * 
   * @param msg     the prompt message
   * @param choices the array of choices
   * @return the selected choice
   */
  public function promptChoice( msg, choices )
  {
    var fullMsg = msg + " (";
    for( var i=0; i<choices.length(); i++ )
    {
      if( i != 0 )
        fullMsg += "/";
      fullMsg += choices[i][0];
    }
    fullMsg += ")";
    
    while(true)
    {
      var choice = promptImpl( 1, fullMsg, null, null /*XXX should make completion set*/ );
      for( var i=0; i<choices.length(); i++ )
        if( choice == choices[i][0] )
          return choices[i];
    }
  }
  
  /**
   * Ask the user a yes/no question.
   * 
   * @param msg     the prompt message
   * @return <code>true</code> if user answered affirmative
   */
  public function promptYesNo(msg)
  {
    return promptChoice( msg, ["y","n"] ) == "y";
  }
  
  /**
   * Prompt for user input.
   * 
   * @param msg     the prompt message
   * @param args    (history (completionSet)?)?
   * @return the user input
   */
  public function prompt( msg, args... )
  {
    var history = promptHistory;
    var completionSet = null;
    if( args.length() > 0 )
      history = args[0];
    if( args.length() > 1 )
      completionSet = args[1];
    return promptImpl( -1, msg, history, completionSet );
  }
  
  var promptHistory = new History();
  
  /**
   * 
   * @param max     immediately return once user has typed in 
   *    this many chars, or <code>-1</code> for no limit
   * @param msg     the prompt message
   * @param history the history buffer, or <code>null</code>
   * @param cs      the completion set, or <code>null</code>
   * @return the user input
   */
  function promptImpl( max, msg, history, cs )
  {
    var eventQueue = new ti.swing.SwingQueue();
    var mode = new (function()
      extends EscapeMode() {
      
      var str = "";
      
      function handleChar(c)
      {
        if( c == "\n" )
        {
          if( str.length() > 2 )
            history.append(str);
          eventQueue.enqueue(str);
          str = "";
          leaveMode();
        }
        else if( (cs != null) && (c == "\t") )
        {
          replace( cs.getCommonCompletion(str) );
        }
        else
        {
          str += c;
          
          if( str.length() == max )
          {
            eventQueue.enqueue(str);
            str = "";
            leaveMode();
          }
          else
          {
            refresh();
          }
        }
      }
      
      function handleBackSpace()
      {
        if( str.length() > 0 )
        {
          str = str[0..str.length()-2];
          refresh();
        }
      }
      
      function replace(newStr)
      {
        if( newStr != null )
        {
          str = newStr;
          refresh();
        }
      }
      
      const var UP    = pkg.util.getKeyStroke("UP");
      const var DOWN  = pkg.util.getKeyStroke("DOWN");
      
      handleInvalidKey = (function(super) {
        return function(key) {
          
          if( history != null )
          {
            if( key == UP )
            {
              replace( history.getPrev() );
              return;
            }
            else if( key == DOWN )
            {
              replace( history.getNext() );
              return;
            }
          }
          
          super(key);
        };
      })(handleInvalidKey);
      
      function refresh()
      {
        frame.setStatusLineText(msg + ": " + str);
      }
      
      leaveMode = (function(super) {
        return function() {
          eventQueue.enqueue(null);
          super();
        };
      })(leaveMode);
      
      refresh();
      frame.setStatusLineIcon( pkg.util.getImageIcon("Help16.gif") );  // XXX need better icon!
      
    })();
    
    return eventQueue.dequeue();
  }
  
  /* normally I'd have searchForward call searchForwardFrom which would
   * call searchForwardFromTo, but in this case speed is more important
   * because these can be called for every keystroke
   */
  
  /**
   * Search forwards in the document from the current cursor position, 
   * returning the index of the first match after the current position 
   * in the document.  The <code>match</code> parameter can (currently)
   * be a string or an array of strings.  If an array, this will find
   * the first match to any of the strings.  Maybe at some point it
   * could be a regular expression.
   * 
   * @param match        the pattern to match
   * @return offset of next match, or -1 if none
   */
  public function searchForward(match)
  {
    var buf = EditorFrame.getCurrentFrame().getBuffer();
    var off = buf.getCaretPosition();
    var doc = buf.getDocument();
    return search( match, off, 0, doc.getLength(), doc, +1 );
  }
  
  /**
   * Search forward in the document from the specified position, returning
   * the index of the first match.
   * 
   * @param match        the pattern to match
   * @param off          the offset to search from
   * @return offset of next match, or -1 if none
   */
  public function searchForwardFrom( match, off )
  {
    var buf = EditorFrame.getCurrentFrame().getBuffer();
    var doc = buf.getDocument();
    return search( match, off, off, doc.getLength(), doc, +1 );
  }
  
  /**
   * Search forward in the document from the specified start position, to 
   * the specified end position, returning the index of the first match.
   * 
   * @param match        the pattern to match
   * @param start        the offset to search from (inclusive)
   * @param end          the offset to search to (exclusive)
   * @return offset of next match, or -1 if none
   */
  public function searchForwardFromTo( match, start, end )
  {
    var buf = EditorFrame.getCurrentFrame().getBuffer();
    return search( match, start, start, end, buf.getDocument(), +1 );
  }
  
  /**
   * Search backward in the current document from the current cursor postion.
   * 
   * @param match        the pattern to match
   * @return offset of match, or -1 if none
   */
  public function searchBackward(match)
  {
    var buf = EditorFrame.getCurrentFrame().getBuffer();
    var off = buf.getCaretPosition();
    var doc = buf.getDocument();
    return search( match, off, 0, doc.getLength(), doc, -1 );
  }
  
  /**
   * Search backward in the current document from the specified cursor postion.
   * 
   * @param match        the pattern to match
   * @param off          the offset to start searching from
   */
  public function searchBackwardFrom( match, off )
  {
    var buf = EditorFrame.getCurrentFrame().getBuffer();
    var doc = buf.getDocument();
    return search( match, off, 0, doc.getLength(), doc, -1 );
  }
  
  /**
   * Search backward in the document from the specified start position, to 
   * the specified end position, returning the index of the first match.
   * 
   * @param match        the pattern to match
   * @param start        the offset to search from (exclusive)
   * @param end          the offset to search to (inclusive)
   * @return offset of next match, or -1 if none
   */
  public function searchBackwardFromTo( match, start, end )
  {
    var buf = EditorFrame.getCurrentFrame().getBuffer();
    var doc = buf.getDocument();
    return search( match, end-1, start, doc.getLength(), doc, -1 );
  }
  
  /**
   * 
   * @param open
   * @param close
   * @param off
   * @param max       (optional)
   */
  public function searchBackwardForOpenFrom( open, close, off, args... )
  {
    var cnt = 1;
    var start = off;
    
    if( args.length() > 0 )
      function searchBackwardFrom( match, off ) {
        return searchBackwardFromTo( match, start - args[0], off+1 );
      }
    
    while( cnt > 0 )
    {
      var openOff  = searchBackwardFrom( open, off-1 );
      var closeOff = searchBackwardFrom( close, off-1 );
      
      if( openOff < closeOff )
      {
        cnt++;
        off = closeOff;
      }
      else
      {
        cnt--;
        off = openOff;
      }
    }
    
    return off;
  }
  
  /**
   * Implementing of search.
   */
  var search = oscript.swing.text.ODEUtils.search;
//    function search( str, off, start, end, doc, inc )
//    {
//    }
  
})();


/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */
