/*=============================================================================
 *     Copyright Texas Instruments 2003. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * A base class for all email attachments.  The derived class must implement
 * the <code>getEncodedBody()</code> method, which returns the body of the
 * attachment as a string, which is placed in the email message body.  If
 * the attachment is a binary attachment, the return value should have the
 * binary encoded as a string.
 * 
 * @param filename     the "suggested" filename of the attachment
 * @param contentType  the content type, ie. <code>"text/plain"</code>
 * @param encoding     the type of encoding, or <code>null</code> if none,
 *    ie. <code>"base64"</code>
 */
public function Attachment( filename, contentType, encoding )
{
  /**
   * Get the name of this attachment.
   * 
   * @return the filename (string)
   */
  public function getFilename()
  {
    return filename;
  }
  
  /**
   * Get the attachment as a file.  This just returns a file representation
   * of the un-decoded attachment body, so attachment types which encode
   * their payload probably need to re-implement this.
   * 
   * @return a file
   */
  public function getFile()
  {
    if( file == null )
      file = new pkg.fs.TempFile( getFilename(), getEncodedBody() );
    return file;
  }
  private var file = null;
  
  /**
   * The string version of the attachment is what gets embedded in the email
   * message itself, including the attachments sub-headers and encoded body,
   * but not the boundary-strings.
   * 
   * @return a string
   */
  public function castToString()
  {
    return ("Content-Type: " + contentType + "\n" +
            ((encoding != null) ? ("Content-Transfer-Encoding: " + encoding + "\n") : "") +
            "Content-Disposition: attachment; filename=\"" + filename + "\"\n" + 
            "\n" + getEncodedBody() + "\n");
  }
}




/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

