/*=============================================================================
 *     Copyright Texas Instruments 2000-2003.  All Rights Reserved.
 *   
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

mixin oscript.fs;


/**
 * This package provides access to the file system.  In the documentation of 
 * these methods and types, a "path" refers to a string path that represents
 * a file, and a "file" refers to a file object which is a handle to that
 * file.
 */
public var fs = new function() {
  
  /**
   * A file object, which is a handle to a file in the file system.
   * 
   * @param path   the path to the file
   */
  public const var File = new (function() {
    // this is actually just a wrapper for native code:
    public function callAsConstructor(args)
    {
      if( args.length() == 1 )
      {
        if( args[0] instanceof String )
          return AbstractFileSystem.resolve(args[0]);
        else if( args[0] instanceof AbstractFile )
          return AbstractFileSystem.resolve( args[0].getPath() );
      }
      
      throw new ProgrammingErrorException("wrong number of args");
    }
    
  })();
  
  /**
   * A virtual, read-only, temporary file.  Good for places where a file is 
   * needed, but doesn't exist.
   * 
   * @param name   the file name
   * @param body   the contents of the file
   */
  public function TempFile( name, body )
    extends oscript.fs.AbstractAbstractFile( name, true )
  {
    if( body == null )
      body = "";
    
    public function getInputStream()
    {
      return new java.io.StringBufferInputStream(body);
    }
  }
  
  /**
   * A jar-file-system is a filesystem representing the contents of a .jar
   * file.  This allows .jar files to be mount into the virtual filesystem,
   * so that script code can access the contents of a .jar file as if it 
   * were uncompressed.
   * 
   * @param root  the path to the .jar file
   */
  public var JarFileSystem = oscript.fs.JarFileSystem;
  
  /**
   * A local-file-system is the default filesystem type, allowing access to
   * files in the operating system's native filesystem.
   * 
   * @param root  the path to the root directory within the native filesystem
   */
  public var LocalFileSystem = oscript.fs.LocalFileSystem;
  
  /**
   * Mount a new filesystem.  A filesystem can be mounted at any
   * position within the virtual filesystem.  A mounted filesystem
   * could potentially eclipse a file.  If multiple filesystems
   * are mounted at the same point, the result is the union of all
   * those filesystems.
   * 
   * @param fs           the filesystem to mount
   * @param path         the path to the location to mount (ex: "/")
   */
  public function mount( fs, path )
  {
    AbstractFileSystem.mount( fs, path );
  }
  
  /**
   * Get the current working directory path.
   * 
   * @return the current working directory file object.
   */
  public function getCwd()
  {
    return new File( AbstractFileSystem.getCwd() );
  }
  
  /**
   * Set the current working directory path.
   * 
   * @param cwd  the new current working directory file or path
   */
  public function setCwd(cwd)
  {
    if( cwd instanceof AbstractFile )
      cwd = cwd.getPath();
    AbstractFileSystem.setCwd(cwd);
  }
  
  /**
   * Return the filename portion of the specified <code>path</code>
   * 
   * @param path   the path string
   * @return the filename portion of the path
   */
  public function basename(path)
  {
    return AbstractFileSystem.basename(path);
  }
  
  /**
   * Return the directory name portion of the specified <code>path</code>
   * 
   * @param path   the path string
   * @return the directory name portion of the path
   */
  public function dirname(path)
  {
    return AbstractFileSystem.dirname(path);
  }
  
  /**
   * Normalize the path.  This involves ensuring the path is
   * fully qualified (begins with "/"), and gets rid of any
   * extra "." and "/".
   * 
   * @param cwd   the (optional) cwd to normalize relative to
   * @param path  the string path to normalize
   */
  public function normalize(args...)
  {
    return AbstractFileSystem.normalize.callAsFunction(args);
  }
  
  /**
   * Get an iterator of the children of the specified file or path.  If the 
   * specified file or path is not a directory, this will return an empty 
   * iterator.
   * 
   * @param file  the file or path to get children of
   * @return Collection of AbstractFile
   */
  public function children(path)
  {
    return AbstractFileSystem.children(path);
  }
  
  /**
   * Try to resolve the specified path, returning a file object.  The
   * <code>path</code> can be absolute or relative.
   * 
   * @param path   the path to resolve
   * @return the file object... possibly file object can represent a non-
   *    existant file if there is no file at the specified path
   */
  public function resolve(path)
  {
    return AbstractFileSystem.resolve(path);
  }
  
}();


/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */
