/*=============================================================================
 *     Copyright Texas Instruments 2002. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * Container for input utilities and functions.  The thread group of the
 * current thread is used to determine which input stream to read from.
 * <p>
 * NOTE: because of the way the console works, these methods will never see 
 * the input until the user presses "RETURN"... this is because the console
 * handles the arrow keys, delete, backspace, etc until the user presses
 * RETURN, and then it pushes the data through the console's Reader, which
 * is what lets these methods see the input.
 */
public var input = new (function() {
  
  static public var WHITE_SPACE = " \t\n";
  
  /**
   * Used to map a thread-group to a thread-group-info object
   */
  var threadGroupInfoTable = new java.util.WeakHashMap();
  
  /**
   * This method identifies the calling thread's <code>ThreadGroup</code> 
   * and gets the corresponding <code>ThreadGroupInfo</code> object.
   * 
   * @param useDefault  should we use default <code>ThreadGroupInfo</code> 
   *    instead of creating a new one if one doesn't already exist for the
   *    current <code>ThreadGroup</code>
   * @return a <code>ThreadGroupInfo</code> object
   */
  function getThreadGroupInfo(useDefault)
  {
    var tg = java.lang.Thread.currentThread().getThreadGroup().getName();
    
    synchronized(threadGroupInfoTable)
    {
      var tgi = threadGroupInfoTable.get(tg);
      var itr;
      
      if( useDefault && (tgi == null) && (itr=threadGroupInfoTable.values().iterator()).hasNext() )
        tgi = itr.next();
      
      if( tgi == null )
      {
        tgi = new ThreadGroupInfo();
        threadGroupInfoTable.put( tg, tgi );
      }
      
      return tgi;
    }
  }
  
  /**
   * Sets the intput stream for the <code>ThreadGroup</code> of the current
   * thread.
   * 
   * @param in    the input stream to read from
   */
  public function initialize(in)
  {
    getThreadGroupInfo(false).initialize(in);
  }
  
  /**
   * Clears the input stream for the <code>ThreadGroup</code> of the
   * current thread.
   */
  public function dispose()
  {
    threadGroupInfoTable.remove( java.lang.Thread.currentThread().getThreadGroup().getName() );
  }
  
  /**
   * Read a character of input.
   * @return a string of length 1
   */
  public function readChar()
  {
    var val = getThreadGroupInfo(true).getIn().read();
    
    if( val == -1 )
      return null;
    
    // using java.lang.String(byte[]) constructor as sort of a hack to go
    // from an integer (containing a char) to a string:
    return new java.lang.String( [ val ] );
  }
  
  /**
   * Read the next whitespace seperated word.
   * @return a string
   * @see #WHITE_SPACE
   */
  public function readWord()
  {
    return readCond(isWhiteSpace);
  }
  
  /**
   * Read until the next end of line (ie. "\n").
   * @return a string
   */
  public function readLine()
  {
    return readCond(isEndOfLine);
  }
  
  // read until condition...
  function readCond(cond)
  {
    var str;
    var c;
    
    while( cond( c=readChar() ) ) {}
    
    str = c;
    
    while( !cond( c=readChar() ) )
      str += c;
    
    return str;
  }
  
  function isWhiteSpace(c)
  {
    for( var i=0; i<WHITE_SPACE.length(); i++ )
      if( WHITE_SPACE[i] == c )
        return true;
    return false;
  }
  
  function isEndOfLine(c)
  {
    return c == "\n";
  }
  
  var defaultIn = java.lang.System.in;
  
  function ThreadGroupInfo()
  {
    var in = defaultIn;
    
    public function initialize(in)
    {
      this.in = in;
    }
    
    public function getIn()
    {
      return in;
    }
  }
  
})();



/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */
