/*=============================================================================
 *     Copyright Texas Instruments 2002, 2003. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Run a listing's unit test.  This function is outside <code>osbook</code>
 * to prevent the unit test from having access to the internal state of 
 * osbook.
 * 
 * @param str     the test to run
 * @param error   the function(str) that the unit test can call to indicate 
 *   an error
 */
function runtest( str, error )
{
  eval str;
}


// make sure the system knows about some java packages that it might not
// otherwise know about:
pkg.system.declareJavaPackage("javax.xml");
pkg.system.declareJavaPackage("javax.xml.parsers");
pkg.system.declareJavaPackage("org.xml");
pkg.system.declareJavaPackage("org.xml.sax");
pkg.system.declareJavaPackage("org.xml.sax.helpers");

/**
 * A tool for parsing an osbook .xml file and generating .html output.  An
 * osbook file is basically an .html file with a few special element types:
 * <ul>
 *   <li> book     -  root element, must have "name" and "stylesheet" attrs
 *   <li> section  -  for defining a section/sub-section of the document
 *   <li> listing  -  a src code listing, optionally with test element
 *   <li> test     -  within a listing, a unit test for the listed code
 *   <li> console  -  shows interactions with the script console
 * </ul>
 */
public var osbook = new (function() {
  
  private const var indexOlTypes = [ "1", "A", "I", "a", "i" ];
  
  /**
   * Return an HTML document with the documentation generated from the
   * specified <code>file</code>.
   * 
   * @param file       the file to parse 
   * @return a string contaning the HTML document
   */
  public function parseToDocument(file)
  {
    file = pkg.cl.resolveFile(file);
    
    // create tmp directory, to save listings to in order to run the
    // listing's unit test:
    var tmpdir = new pkg.File("./tmp" + java.lang.Math.random());
    pkg.system.addScriptPath( tmpdir.getPath() );
    
    // we also need to track all the tmp files that get created, so
    // they can be deleted:
    var fileList = new java.util.LinkedList();
    
    try
    {
      tmpdir.mkdir();
      
      if( !file.exists() )
        throw new Exception("file doesn't exist: " + file);
      
      var head  = "";
      var index = "";
      var body  = "";
      var tail  = "";
      
      javax.xml.parsers.SAXParserFactory.newInstance().newSAXParser().parse(
        file.getInputStream(),
        new function() extends org.xml.sax.helpers.DefaultHandler() {
          
          // null while not in listing element, accumulated listing otherwise:
          var listing = null;
          var listingName = null;
          
          // the embedded test withing a listing element:
          var test = null;
          
          // the current section level:
          var currentSectionLevel = -1;
          
          // the previous section level, used to detect nesting:
          var lastStartSectionLevel = -1;
          var lastEndSectionLevel = -1;
          
          var currentIndexIndent = "";
          var currentIndexPrefix = "";
          var currentIndexCount  = -1;
          
          var currentStackIdx = 0;
          
          var savedIndexPrefixStack = [];
          var savedIndexIndentStack = [];
          var savedIndexCountStack  = [];
          const var indent = "&nbsp;&nbsp;&nbsp;";
          
          // to avoid namespace collisions with the anchor names:
          var currentSectionName = "";
          
          function currentIndex()
          {
            if( currentIndexCount == -1 )
              return "";
            return ((currentIndexPrefix != "") ? currentIndexPrefix + "." : "") + currentIndexCount;
          }
          
          /* 
           * The various handlers to handle character notification in
           * different states:
           */
          
          function bodyCharacterHandler( chars, off, len )
          {
            body += new java.lang.String( chars[off..off+len-1] );
          }
          
          function listingCharacterHandler( chars, off, len )
          {
            listing += new java.lang.String( chars[off..off+len-1] );
          }
          
          function testCharacterHandler( chars, off, len )
          {
            test += new java.lang.String( chars[off..off+len-1] );
          }
          
          
          /**
           * Receive notification of character data.
           */
          public var characters = bodyCharacterHandler;
          
          /**
           * Receive notification of a start tag.
           */
          public function startElement( namespace, name, qname, attrs )
          {
            if( qname == "book" )
            {
              head += "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 TRANSITIONAL//EN\">\n";
              head += "<!-- THIS FILE IS MACHINE GENERATED FROM: " + file.getName() + " -->\n";
              head += "<html><head>";
              head += "<link rel=\"stylesheet\" href=\"" + attrs.getValue("stylesheet") + "\" type=\"text/css\"/>";
              head += "<title>" + attrs.getValue("name") + "</title>";
              head += "</head><body><h1><a name=\"INDEX\">" + attrs.getValue("name") + "</a></h1>";
            }
            else if( qname == "section" )
            {
              var name   = attrs.getValue("name");
              var anchor = name;  // XXX need to take into account namespace!!
              
              ++currentSectionLevel;
              if( currentSectionLevel > lastStartSectionLevel )
              {
                ++currentStackIdx;
                
                savedIndexPrefixStack[currentStackIdx] = currentIndexPrefix;
                savedIndexIndentStack[currentStackIdx] = currentIndexIndent;
                savedIndexCountStack[currentStackIdx]  = currentIndexCount;
                
                currentIndexIndent += indent;
                currentIndexPrefix  = currentIndex();
                currentIndexCount   = 0;
              }
              
              ++currentIndexCount;
              
              var tmp = currentIndex();
              if( tmp.length() == 1 )
                tmp += "&nbsp;";
              
              index += "\n<br>" + currentIndexIndent + "<tt>" + tmp + "</tt>" + indent + "<a href=\"#" + anchor + "\">" + name + "</a>";
              
              body  += "<h" + (2+currentSectionLevel) + "><a name=\"" + anchor + "\" href=\"#INDEX\">" + currentIndex() + " " + name + "</a></h" + (2+currentSectionLevel) +">";
              
              lastStartSectionLevel = currentSectionLevel;
            }
            else if( qname == "listing" )
            {
              if( listing != null )
                throw new Exception("cannot have nested listings, listing=" + listing);
              listing = "";
              listingName = attrs.getValue("name");
              characters = listingCharacterHandler;
            }
            else if( qname == "test" )
            {
              if( listing == null )
                throw new Exception("test must be within listing");
              else if( test != null )
                throw new Exception("can only have one test per listing");
              test = "";
              characters = testCharacterHandler;
            }
            else if( qname == "console" )
            {
              body += "<div class=\"console\"><pre wrap>";
            }
            else if( qname == "osdoc:parseToBodyPart" )
            {
              var file = attrs.getValue("file");
              body += "<p>" + pkg.osdoc.parseToBodyParts(file).detail + "</p>";
            }
            else
            {
              var attrstr = "";
              for( var i=0; i<attrs.getLength(); i++ )
                attrstr += " " + attrs.getQName(i) + "=\"" + attrs.getValue(i) + "\"";
              body += "<" + qname + attrstr + ">";
            }
          }
          
          /**
           * Receive notification of an end tag.
           */
          public function endElement( namespace, name, qname )
          {
            if( qname == "book" )
            {
              tail += "</body></html>";
            }
            else if( qname == "section" )
            {
              --currentSectionLevel;
              if( currentSectionLevel < lastEndSectionLevel )
              {
                currentIndexPrefix = savedIndexPrefixStack[currentStackIdx];
                currentIndexIndent = savedIndexIndentStack[currentStackIdx];
                currentIndexCount  = savedIndexCountStack[currentStackIdx];
                --currentStackIdx;
              }
              lastEndSectionLevel = currentSectionLevel;
            }
            else if( qname == "listing" )
            {
              var str = "// this file saved as " + listingName + "\n" + listing;
              
              body += "<div class=\"file\"><pre wrap>";
              
              var file = new pkg.File( tmpdir + "/" + listingName );
              fileList.add(file);
              file.createNewFile();
              
              var ps = new java.io.PrintStream( file.getOutputStream(false), true );
              ps.print(str);
              ps.flush();
              ps.close();
              
              if( file.getExtension() == "os" )
                body += pkg.os2html.parseToBodyPart(file);
              else
                body += str;
              body += "</pre></div>";
              
              if( test != null )
              {
                var errcnt = 0;
                var errstr = "";
                runtest( test, function(str) {
                  errstr += "  " + str + "\n";
                  errcnt++;
                  return true;
                } );
                
                if( errcnt > 0 )
                {
                  errstr = ("errors in test for listing " + listingName + "\n" + 
                            errstr + 
                            errcnt + " errors");
                  writeln(errstr);
                  throw new Exception(errstr);
                }
                
                test = null;
              }
              
              listing = null;
              listingName = null;
              characters = bodyCharacterHandler;
            }
            else if( qname == "test" )
            {
              characters = listingCharacterHandler;
            }
            else if( qname == "console" )
            {
              body += "</pre></div>";
            }
            else
            {
              body += "</" + qname + ">";
            }
          }
          
        }()
      );
      
      return head + index + "<br><br>" + body + tail;
    }
    finally
    {
      pkg.system.removeScriptPath( tmpdir.getPath() );                
      for( var itr=fileList.iterator(); itr.hasNext(); )
        itr.next().delete();
      tmpdir.delete();
    }
  }
  
  /**
   * Parse an osbook file and write the generated HTML file
   * 
   * @param srcFile   the file to parse
   * @param outFile   the file to write output to
   */
  public function parseToFile( srcFile, outFile )
  {
    outFile = pkg.cl.resolveFile(outFile);
    
    if( !outFile.exists() )
      outFile.createNewFile();
    
    var out = outFile.getOutputStream(false);
    var ps = new java.io.PrintStream(out);
    ps.println( parseToDocument(srcFile) );
    ps.flush();
    ps.close();
    out.flush();
    out.close();
  }
  
})();


/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */
