/*=============================================================================
 *     Copyright Texas Instruments 2002. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * A tool for generating and displaying documentation from .os files.
 */
public var osdoc = new (function() {
  
  var detailVisitor = new oscript.visitor.ExtensibleVisitor();
  var indexVisitor = new oscript.visitor.ExtensibleVisitor();
  
  var st = oscript.syntaxtree;
  
  var headerBgColor = "#003060";
  var headerFgColor = "#ffffff";
  var bodyBgColor   = "#dddddd";
  
  /* 
   * Register visitor functions
   */
  
  /**
   * For each node token, store the specials, so they are accessible
   * to the other visitor functions.
   */
  detailVisitor.register( st.NodeToken, function( n, ctx ) {
    
    if( n.specialTokens != null )
      ctx.lastSpecialTokens = n.specialTokens;
    
  } );
  
  /**
   * <PRE>
   * f0 -> Permissions()
   * f1 -> "function"
   * f2 -> &lt;IDENTIFIER&gt;
   * f3 -> "("
   * f4 -> ( Arglist() )?
   * f5 -> ")"
   * f6 -> ( "extends" PrimaryExpressionWithTrailingFxnCallExpList() FunctionCallExpressionList() )?
   * f7 -> "{"
   * f8 -> Program(true)
   * f9 -> "}"
   * </PRE>
   */
  detailVisitor.register( st.FunctionDeclaration, function( n, ctx ) {
    
    // need to visit these to collect the specials:
    n.f0.accept( detailVisitor, ctx );
    n.f1.accept( detailVisitor, ctx );
    
    var name = n.f2.otokenImage;
    var anchor = ctx.makeAnchor(name);
    
    ctx.write("<tr><td colspan=2 bgColor=" + headerBgColor + "><font color=" + headerFgColor + "><b><a name=\"" + anchor + "\">" + name + "</a></b></font></td></tr>");
    
    if( ctx.lastSpecialTokens != null )
    {
      var argNames = n.f4.present() ? getArglist( n.f4.node ) : [];
      
      var str = "";
      
      for( var i=0; i<ctx.lastSpecialTokens.size(); i++ )
        str += ctx.lastSpecialTokens.elementAt(i).toString();
      
      var comment = Function.extractJavadocCommentBody( str, name, argNames );
      
      if( comment != null )
        ctx.write("<tr><td colspan=2 bgcolor=" + bodyBgColor + ">" + comment + "</td></tr>");
      
      ctx.lastSpecialTokens = null;
    }
    
    var childCtx = new ParseContext(anchor);
    n.f8.accept( detailVisitor, childCtx );
    
    var str = childCtx.castToString();
    if( str.length() > 0 )
    {
      ctx.write("<tr><td bgcolor=" + headerBgColor + " colspan=2><font color=" + headerFgColor + "><b><code>{</code></b></font></td></tr>");
      ctx.write("<tr><td bgcolor=" + headerBgColor + ">" + indent + "</td><td bgcolor=" + headerBgColor + "><table border=0 cellspacing=0 width=\"100%\">" + str + "</table></td></tr>");
      ctx.write("<tr><td bgcolor=" + headerBgColor + "  colspan=2><font color=" + headerFgColor + "><b><code>}</code></b></font></td></tr>");
      ctx.write("<tr><td bgcolor=" + bodyBgColor + " colspan=2>&nbsp;</td></tr>");
    }
    
  } );
  
  indexVisitor.register( st.FunctionDeclaration, function( n, ctx ) {
    
    var name = n.f2.otokenImage;
    var anchor = ctx.makeAnchor(name);
    
    ctx.write("<li> <a href=\"#" + anchor + "\">" + name + "</a>");
    
    var childCtx = new ParseContext(anchor);
    n.f8.accept( indexVisitor, childCtx );
    
    var str = childCtx.castToString();
    if( str.length() > 0 )
      ctx.write("<ul>" + str + "</ul>");
    
  } );
  
  var indent = "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
  
  /**
   * This expands an Arglist node, and returns an array of the names of the
   * <i>IDENTIFIER</i>s in the arglist.  The expansion of the Arglist node:
   * <PRE>
   * f0 -> Permissions(false)
   * f1 -> &lt;IDENTIFIER&gt;
   * f2 -> ( "," Permissions(false) &lt;IDENTIFIER&gt; )*
   * f3 -> ( "..." )?
   * </PRE>
   * 
   * @param n     the Arglist node
   * @return the arry of argument names
   */
  function getArglist(n)
  {
    var arglist = [ n.f1.tokenImage ];
    
    for( var i=0; i<n.f2.size(); i++ )
      arglist[i+1] = n.f2.elementAt(i).elementAt(2).tokenImage;
    
    return arglist;
  }
  
  /**
   * A parse-context has all the parsing state that needs to be accessed or
   * shared by the visitor functions.
   */
  function ParseContext(args...)
  {
    public var lastSpecialTokens = null;
    
    /**
     * The string buffer we are accumulating
     */
    var str = "";
    
    /**
     * Prefix that is prepended to anchor names.
     */
    var anchorPrefix = "";
    
    if( args.length() > 0 )
      anchorPrefix = args[0] + "-";
    
    /**
     * Given a function/variable name, generate an appropriate
     * anchor name.
     * 
     * @return name     the symbol name
     * @return an anchor string
     */
    public function makeAnchor(name)
    {
      return anchorPrefix + name;
    }
    
    /**
     * Write output to the document being generated.
     * 
     * @param str    the string to append to output
     */
    public function write(str)
    {
      this.str += str + "\n";
    }
    
    /**
     * @return the accumulated HTML as a string.
     */
    public function castToString()
    {
      return str;
    }
  }
  
  
  /**
   * Implements the parsing of a script file to a string that can be
   * part of an HTML document's body.
   * 
   * @param file       the file to parse
   * @return an object with an <code>index</code> property, containing
   *    an index string, and <code>detail</code> property, containing
   *    the body of the of docs
   */
  public function parseToBodyParts(file)
  {
    file = pkg.cl.resolveFile(file);
    
    if( !file.exists() )
      throw new Exception("file doesn't exist: " + file);
    
    var node = oscript.OscriptInterpreter.parse(file);
    
    // XXX prolly should pass the file to the ParseContext, or something
    // like that, so the file name can be encoded in the links between
    // the index and detail parts... this way docs for multiple src files
    // can be combined without risk of namespace clashes...
    
    var indexCtx = new ParseContext();
    indexVisitor.visit( node, indexCtx );
    
    var detailCtx = new ParseContext();
    detailVisitor.visit( node, detailCtx );
    
    return new '{
      public const var index  = "<ul>" + indexCtx + "</ul>";
      public const var detail = "<table border=0 cellspacing=0>" + detailCtx + "</table>";
    }();
  }
  
  /**
   * Return an HTML document with the documentation generated from the
   * specified <code>file</code>.
   * 
   * @param file       the file to parse 
   * @return a string contaning the HTML document
   */
  public function parseToDocument(file)
  {
    var parts = parseToBodyParts(file);
    return "<html><body bgcolor=" + bodyBgColor + ">" + "<h3>Index</h3>" + parts.index + "<h3>Detail</h3>" + parts.detail + "</body></html>";
  }
  
  /**
   * Parse a .os file and write the generated HTML document to file
   * 
   * @param srcFile   the file to parse
   * @param outFile   the file to write output to
   */
  public function parseToFile( srcFile, outFile )
  {
    outFile = pkg.cl.resolveFile(outFile);
    
    if( !outFile.exists() )
      outFile.createNewFile();
    
    var out = outFile.getOutputStream(false);
    var ps = new java.io.PrintStream(out);
    ps.println( parseToDocument(srcFile) );
    ps.flush();
    ps.close();
    out.flush();
    out.close();
  }
  
})();


/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */
