/*=============================================================================
 *     Copyright Texas Instruments 2002. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Container for output utilities and functions.  The thread-group of the
 * current thread is used to determine which output stream to use.
 */
public var output = new (function() {
  
  /**
   * Used to map a thread-group to a thread-group-info object
   */
  var threadGroupInfoTable = new java.util.WeakHashMap();
  
  /**
   * This method identifies the calling thread's <code>ThreadGroup</code> 
   * and gets the corresponding <code>ThreadGroupInfo</code> object.
   * 
   * @param useDefault  should we use default <code>ThreadGroupInfo</code> 
   *    instead of creating a new one if one doesn't already exist for the
   *    current <code>ThreadGroup</code>
   * @return a <code>ThreadGroupInfo</code> object
   */
  function getThreadGroupInfo(useDefault)
  {
    var tg = java.lang.Thread.currentThread().getThreadGroup().getName();
    
    synchronized(threadGroupInfoTable)
    {
      var tgi = threadGroupInfoTable.get(tg);
      var itr;
      
      if( useDefault && (tgi == null) && (itr=threadGroupInfoTable.values().iterator()).hasNext() )
        tgi = itr.next();
      
      if( tgi == null )
      {
        tgi = new ThreadGroupInfo();
        threadGroupInfoTable.put( tg, tgi );
      }
      
      return tgi;
    }
  }
  
  /**
   * Function to create a string with the specified foreground color attribute.
   * 
   * @param c      the foreground color
   * @param str    the string
   * @return a string with the appropriate embedded control characters
   */
  public var fgColor;
  
  /**
   * Function to create a string with the specified background color attribute.
   * 
   * @param c      the background color
   * @param str    the string
   * @return a string with the appropriate embedded control characters
   */
  public var bgColor;
  
  /**
   * Function to create a string with a hyperlink.  Each time the user clicks 
   * the hyperlink, the runnable is invoked.  
   * 
   * @param r            the runnable to invoke when user clicks link
   * @param str          the string to apply it to
   * @return a string with embedded control characters
   */
  public var hyperlink;

  try
  {
    fgColor = ti.swing.console.ColorInputHandler.fgColor;
    bgColor = ti.swing.console.ColorInputHandler.bgColor;
    hyperlink = ti.swing.console.ColorInputHandler.hyperlink;
  }
  catch(e)
  {
    fgColor = function(c, str){ return str; };
    bgColor = function(c, str){ return str; };
    hyperlink = function(c, str){ return str; };
  }
  
  /**
   * Sets the output stream for the <code>ThreadGroup</code> of the current
   * thread.
   * 
   * @param out   the print stream to direct all output to
   */
  public function initialize(out)
  {
    getThreadGroupInfo(false).initialize(out);
  }
  
  /**
   * Clears the output streams for the <code>ThreadGroup</code> of the
   * current thread.
   */
  public function dispose()
  {
    threadGroupInfoTable.remove( java.lang.Thread.currentThread().getThreadGroup().getName() );
  }
  
  /**
   * Write output to stdout.  This overrides the built-in method, so that the
   * output will go to the appropriate console.  And end of line character is
   * appended to the end of the output.
   * 
   * @param str    a string to write
   */
  public function writeln(str)
  {
    var out = getThreadGroupInfo(true).getOutputStream();
    out.println(str);
    out.flush();
  }
  
  /**
   * Write output to stdout.  This overrides the built-in method, so that the
   * output will go to the appropriate console.
   * 
   * @param str    a string to write
   */
  public function write(str)
  {
    var out = getThreadGroupInfo(true).getOutputStream();
    out.print(str);
    out.flush();
  }
  
  /**
   * deprecated, use pkg.log.error()
   */
  public function logError(args...)
  {
    pkg.log.error.callAsFunction(args);
  }
  
  /**
   * deprecated, use pkg.log.warning()
   */
  public function logWarning( args... )
  {
    pkg.log.warning.callAsFunction(args);
  }
  
  /**
   * deprecated, use pkg.log.info()
   */
  public function logInfoMsg( args... )
  {
    pkg.log.info.callAsFunction(args);
  }
    
  var defaultOut  = java.lang.System.out;
  
  /**
   * ThreadGroupInfo class.  An instance of this class is used to track all 
   * thread-group specific stuff, such as error/warning count, and the
   * error/warning/info output streams
   */
  function ThreadGroupInfo()
  {
    var out  = defaultOut;
    
    public function getOutputStream()
    {
      return out;
    }
    
    public function initialize(out)
    {
      this.out  = out;
    }
  }
  
})();



/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */
