/******************************************************************************
 *     Copyright Texas Instruments, Inc., 2002.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



/**
 * Has wrappers for registry methods, but when resource is uninstalled
 * automatically cleans up by unlinking created nodes and unsubscribing
 * subscribers.
 */
public const function RegistryResource(managed)
  extends ti.chimera.Resource(managed)
{
  
  /* XXX link and mkdir will recursively create parent directories, so
   * we need to track that in order to cleanup properly
   */
  
  public function mkdir(path)
  {
    pathList.add(path);
    registry.mkdir(path);
  }
  
  public function link( node, path )
  {
    pathList.add(path);
    registry.link( node, path );
  }
  
  public function unlink(path)
  {
    registry.unlink(path);
    pathList.remove(path);
  }
  
  public function subscribeToValue( path, contract, subscriber )
  {
    subscriberList.add(subscriber);
    registry.subscribeToValue( path, contract, subscriber );
  }
  
  public function unsubscribeFromValue(subscriber)
  {
    registry.unsubscribeFromValue(subscriber);
    subscriberList.remove(subscriber);
  }
  
  public function subscribeToCreation( path, subscriber )
  {
    subscriberList.add(subscriber);
    registry.subscribeToCreation( path, subscriber );
  }
  
  public function unsubscribeFromCreation(subscriber)
  {
    registry.unsubscribeFromCreation(subscriber);
    subscriberList.remove(subscriber);
  }
  
  public function subscribeToDeletion( path, subscriber )
  {
    subscriberList.add(subscriber);
    registry.subscribeToDeletion( path, subscriber );
  }
  
  public function unsubscribeFromDeletion(subscriber)
  {
    registry.unsubscribeFromDeletion(subscriber);
    subscriberList.remove(subscriber);
  }
  
  var pathList;
  var subscriberList;
  
  public function install()
  {
  }
  
  public function uninstall()
  {
    /* uninstall subscribers:
     */
    for( var itr=subscriberList.iterator(); itr.hasNext(); )
    {
      var subscriber = itr.next();
      registry.unsubscribeFromValue(subscriber);
      registry.unsubscribeFromCreation(subscriber);
      registry.unsubscribeFromDeletion(subscriber);
      itr.remove();
    }
    
    /* unlink nodes:
     */
    var error = false;
    while( pathList.size() > 0 )
    {
      var nremoved = 0;
      
      for( var itr=pathList.iterator(); itr.hasNext(); )
      {
        var path = itr.next();
        try {
          if( registry.exists(path) )
            registry.unlink(path);
          nremoved++;
          itr.remove();
        } catch(e) {
          if(error)
            java.lang.System.err.println("error unlinking " + path + ": pathList=" + pathList + ", e=" + e);
        }
      }
      
      if(error)
        break;
      
      if( nremoved == 0 )
        error = true;
    }
    
    reset();
  }
  
  function reset()
  {
    pathList = new java.util.LinkedList();
    subscriberList = new java.util.LinkedList();
  }
  
  reset();
  
}





/**
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */


