/******************************************************************************
 *     Copyright Texas Instruments, Inc., 2002.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



/**
 * Generate a sub-tree in the registry that is the result of applying a
 * specified function to the values stored in another sub-tree.  If the
 * <code>applyDirs</code> (apply fxn to directories) is <code>true</code>
 * then the function is also applied to the value of the directory node
 * (ie. the DirectoryTable), otherwise the directory structure will be
 * preserved and recursively the same function is applied to the sub-
 * directories.
 * 
 * @param srcPath     the path to the root node to apply function to.
 *    This should be a directory node
 * @param dstPath     the destination path
 * @param rr          the registry or registry resource
 * @param applyDirs   if <code>true</code>, apply <code>fxn</code> to
 *    directory nodes (nodes containing a DirectoryTable), otherwise
 *    the directory structure is preserved and the same function is
 *    applied to the children of the directory
 * @param fxn         the function to apply to the values of the
 *    nodes at the src path.  The value of the src node is passed
 *    to the function, and the return value is the value of the
 *    destination node
 */
public function applyFunction( srcPath, dstPath, rr, applyDirs, fxn )
{
  // the previous directory table:
  var oldDt = null;
  
  // maps child node to it's subscriber:
  var subscriberTable = new java.util.Hashtable();
  
  rr.subscribeToValue(
    srcPath, 
    new ti.chimera.registry.TypeNodeContract( ti.chimera.registry.DirectoryTable ),
    function( node, newDt ) {
      
      if( oldDt != null )
      {
        for( var removed : oldDt.notIn(newDt) )
        {
          var srcNode = oldDt.get(removed);
          srcNode.unsubscribe( subscriberTable.remove(srcNode) );
          rr.unlink( dstPath + "/" + removed );
        }
      }
      
      for( var added : newDt.notIn(oldDt) )
      {
        var srcNode = newDt.get(added);
        
        if( (srcNode.getValue() instanceof ti.chimera.registry.DirectoryTable) && !applyDirs )
        {
          applyFunction( srcPath + "/" + added, dstPath + "/" + added, rr, applyDirs, fxn );
        }
        else
        {
          var dstNode = new ti.chimera.registry.Node( null, null, null );
          
          rr.link( dstNode, dstPath + "/" + added );
          
          function subscriber( node, value )
          {
            dstNode.setValue( fxn( srcNode, dstNode, value ) );
          }
          
          srcNode.subscribe(subscriber);
          subscriberTable.put( srcNode, subscriber );
        }
      }
      
      oldDt = newDt;
    } );
}
