/*=============================================================================
 *     Copyright Texas Instruments 2004.  All Rights Reserved.
 *   
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



/**
 * package of runtime utility stuff
 */
public const var runtime = new function() {
  
  /* XXX maybe need something more sophisticated than this, such as logging
   * per thread/thread-group, but this will make things work for the IVM:
   */
  private var log = function(str) { /* ignore */ };
  
  /**
   * Execute a system command.
   * 
   * @param in           the input reader (to the process), or null if none
   * @param out          the output writer (from the process), or null if none
   * @param err          the error writer (from the process), or null if none
   * @param cmd          the command to execute
   * @return the return code from executing the command, an exact number
   */
  public function exec( in, out, err, cmd )
  {
    log("begin exec: \"" + cmd + "\"");
    
    var r = Runtime.getRuntime();
    var p = r.exec(cmd);
    
    linkInOut( p, in, out, err );
    p.waitFor();
    
    log("finish exec: \"" + cmd + "\" => " + p.exitValue());
    
    return p.exitValue();
  }
  
  
  /**
   * Execute a system command, returning the process object.  This allows for
   * more job control than the simpler {@link #exec} form of this command,
   * because caller can <code>destroy()</code>, <code>waitFor()</code> and
   * get the <code>exitValue()</code>.
   * 
   * @param in           the input reader (to the process), or null if none
   * @param out          the output writer (from the process), or null if none
   * @param err          the error writer (from the process), or null if none
   * @param cmd          the command to execute
   * @return a process object, which allows for further asynchronous querying
   *    and control of the process, with <code>exitValue</code>, <code>destroy</code>
   *    and <code>waitFor</code> methods.
   */
  public function execp( in, out, err, cmd )
  {
    log("begin execp: \"" + cmd + "\"");
    
    var r = Runtime.getRuntime();
    var p = r.exec(cmd);
    var t = new Thread( function() {
      linkInOut( p, in, out, err );
      p.waitFor();
      log("finish execp: \"" + cmd + "\" => " + p.exitValue());
    } );
    
    t.start();
    
    return new function() {
      
      /**
       * Get the exit value of the process, blocking until the exit value
       * is available
       * 
       * @return the exit code
       */
      public function exitValue()
      {
        return p.exitValue();
      }
      
      /**
       * Destroy the process, killing it prematurely if it has not already
       * completed.  (Note: the underlying Java API does not seem to necessarily
       * seem to work with all Java VMs)
       */
      public function destroy()
      {
        t.stop();
        p.destroy();
      }
      
      /**
       * Wait for the process to complete.  This will block until the process
       * has completed.
       */
      public function waitFor()
      {
        p.waitFor();
        t.join();
      }
      
    }();
  }
  
  /**
   * helper function to copy output from process's OutputStream to the provided
   * InputStream, and visa versa
   */
  private function linkInOut( p, in, out, err )
  {
    if( in != null )
    {
      var pout = p.getOutputStream();
      var c;
      
      while( (c = in.read()) != -1 )
        pout.write(c);
    }
    
    if( out != null )
    {
      var pin = p.getInputStream();
      var c;
      
      while( (c = pin.read()) != -1 )
        out.write(c);
      
      out.flush();
    }
    
    if( err != null )
    {
      var perr = p.getErrorStream();
      var c;
      
      while( (c = perr.read()) != -1 )
        err.write(c);
      
      err.flush();
    }
  }
  
  /**
   * For debugging <code>exec</code> and <code>execp</code> calls, you can
   * set a log function... at this point, not really indended for public
   * consumption (ie. may change in the future).
   * 
   * @param log   a <code>function(str)</code>
   */
  public function setLog(log)
  {
    this.log = log;
  }
  
}();

