/*=============================================================================
 *     Copyright Texas Instruments 2000-2004.  All Rights Reserved.
 *   
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * The "system" API is a collection of APIs that script code can use to
 * access the scripting engine, without being coupled to implementation
 * details of the scripting engine.
 * <p>
 * Note that many of these don't have to be functions, but could just
 * be variables... but the are functions to benefit from osdoc...
 * <p>
 * This is probably not (yet) complete....
 */
public var system = new function() {
  
  /**
   * Register a function-transformer object, which is responsible for
   * transforming a script function into a java object that is assignable
   * to a specified type.
   * 
   * @param ft    the <code>FunctionTransformer</code>
   * @see #FunctionTransformer
   */
  public function registerFunctionTransformer(ft)
  {
    oscript.data.JavaBridge.registerFunctionTransformer(ft);
  }
  
  /**
   * The base class for function tranformer objects.  A function transformer
   * must extend this base class, and implement the <code>transform</code>
   * method which takes as a parameter a function object, and returns a
   * object assignable to the specified <code>type</code>.
   * 
   * @param type   the type that the return value from <code>transform</code>
   *    must be assignable to
   */
  public function FunctionTransformer(type)
    extends oscript.data.JavaBridge.FunctionTransformer(type)
  {}
  
  /**
   * Add a specified path to the script-path, which is the set of locations
   * in the abstract-file-system which are checked when trying to resolve
   * a script file (as in <code>import</code> or through the script package
   * system).
   * 
   * @param path   the string path to add
   */
  public function addScriptPath(path)
  {
    oscript.OscriptInterpreter.addScriptPath(path);
  }
  
  /**
   * Remove a specified path from the script-path, which is the set of 
   * locations in the abstract-file-system which are checked when trying to 
   * resolve a script file (as in <code>import</code> or through the script 
   * package system).
   * 
   * @param path   the string path to add
   */
  public function removeScriptPath(path)
  {
    oscript.OscriptInterpreter.removeScriptPath(path);
  }
  
  /**
   * Return an iterator of entries in the script-path.  Each entry is a path
   * that is prefixed to a relative path passed to {@link #resolve} in the
   * process of trying to resolve a file.
   * 
   * @return an iterator of strings
   */
  public function getScriptPath()
  {
    return oscript.OscriptInterpreter.getScriptPath();
  }
  
  /**
   * Mount a .jar file under <code>/jar/file</code> and add to script path
   * @param file   the jar file to mount
   */
  public function mountJarFile(file)
  {
    oscript.OscriptInterpreter.mountJarFile(file);
  }
  
  /**
   * Resolve the specified file.  If <code>path</code> is a relative path,
   * then each of the entries in the script-path is searched in order until
   * the file is found.
   * 
   * @param path    the path to the file to resolve
   * @return the file that is found, or a file object representing a file
   *    that does not exist (ie. <code>! file.exists()</code>).  This method
   *    does not return <code>null</code>.
   */
  public function resolve(path)
  {
    return oscript.OscriptInterpreter.resolve( path, false );
  }
  
  /**
   * Used to declare the existance of a java package.  A hack to work around
   * the fact that the java vm cannot always tell us what packages exist.
   * 
   * @param javaPackageName   the java package name
   */
  public var declareJavaPackage = oscript.data.JavaPackageWrapper.declarePackage;
  
  /**
   * Create a package object, which has the scope of the caller as it's 
   * parent.  So the scope will work out as if the accessed members where 
   * declared in the scope of the caller.
   * <p>
   * This can be combined with a <code>mixin</code> to have the same effect 
   * as <code>import</code>, but the actual loading of the file is deferred 
   * until the symbol it provides is accessed.  For example:
   * <pre>
   *   import "foo/a.os";
   *   import "foo/b.os"
   * </pre>
   * can be replaced with:
   * <pre>
   *   mixin pkg.system.makePrivatePackage("/jar/ode/foo");
   * </pre>
   * note that since the files loaded by the package object have access to
   * any variable that is in scope at the point where this function is called
   * it is HIGHLY recommended that files loaded by the package be put in a
   * unique directory which is specified with an absolute path if possible.
   * This is to avoid inadvertantly using the package object to import a
   * different file.
   * 
   * @param path   the path to the directory containing the files that
   *    may be loaded by this package
   */
  public function makePrivatePackage(path)
  {
    var itr = currentStackFrame().iterator();
    
    itr.next();  // currentStackFrame()
    itr.next();  // makePrivatePackage()
    
    return new ScriptPackage( path, itr.next().getScope() );
  }
  
  /**
   * A function to declare some variable from any scope in any scope.  This 
   * relies on knowledge of internal interpreter APIs, so it is best if
   * all places that need to do something like this call this function, so
   * maintainence is easier if those internal APIs change.
   * 
   * @param isConst <code>true</code> if constant
   * @param name    the variable name
   * @param obj     the value to assign to the variable
   * @param scope   (optional) the scope to export the reference to,
   *    defaults to the global-scope.
   */
  public function export( isConst, name, obj, args... )
  {
//    function bogus() {} // to disable some optimizations, because compiler doesn't recognize __eval
    var scope = (args.length() >= 1) ? args[0] : oscript.OscriptInterpreter.getGlobalScope();
    oscript.OscriptInterpreter.__eval( (isConst ? "const " : "") + "var " + name + ";", scope );
    if( obj != null )
      oscript.OscriptInterpreter.__declareInScope( name, obj, scope );
  }
  
  /**
   * Access the stack-frame object for the current thread.  The stack-frame is
   * a chain of nodes, each with information about it's corresponding stack
   * frame, whose head is the most recent stack frame, and tail is the first
   * stack frame for the thread.  The stack-frame node has the following 
   * methods:
   * <ul>
   *   <li> iterator() =>  iterator of stack frame objects beginning
   *     with the current node
   *   <li> getLineNumber()
   *   <li> getFile()  =>  AbstractFile
   *   <li> getName()  =>  function name, or null
   * </ul>
   * 
   * @return the head of the stack-frame chain.
   */
  public function currentStackFrame()
  {
    // probably shouldn't let the general public see a raw (uncloned)
    // stack-frame... should remove topmost (ie current fxn) frame
    return oscript.util.StackFrame.currentStackFrame().clone();
  }
  
  /**
   * Dump the current stack backtrace
   * 
   * @param out  (optional) the output stream to dump to, defaults to stderr
   */
  public function dumpStack(args...)
  {
    var out = (args.length() > 0) ? args[0] : System.err;
    oscript.util.StackFrame.dumpStack(out);
  }
  
  /**
   * public wrapper for private APIs exposed for debuggers
   */
  public var debugger = new function() {
    
    private var getClass = oscript.data.Debugger._getClass;
    
    public var getMember = oscript.data.Debugger._getMember;
    public var memberSet = oscript.data.Debugger._memberSet;
    public var getPreviousScope = oscript.data.Debugger._getPreviousScope;
    
    /**
     * Get a description of the <code>scope</code>.  Return an
     * object with the following properties:
     * <table border=1>
     *   <tr><th>Property Name</th><th>Property Description</th></tr>
     *   <tr><td><tt>type</tt></td><td>string name scope's type, valid types 
     *      are "function", "object", "file", and "global"</td></tr>
     *   <tr><td><tt>text</tt></td><td>description of scope as appopriate for 
     *      the scope's type, ie function name, object type, file name, etc</td></tr>
     * </table>
     * The <tt>type</tt> field might be <tt>null</tt> for various minor types of
     * scopes that get created, such as for curly braces, for loops, etc.
     */
    public function getDescription(scope)
    {
      var c = getClass(scope);
      
      var text = null;
      var type = null;
      
      if( c == oscript.data.FunctionScope )
      {
        text = oscript.data.Debugger._getCallee(scope).getName();
        type = "function";
      }
      else if( c == oscript.data.ScriptObject )
      {
        text = oscript.data.Debugger._getType(scope).getName();
        type = "object";
      }
      else if( c == oscript.data.FileScope )
      {
        text = oscript.data.Debugger._getFile(scope).getName();
        type = "file";
      }
      else if( c == oscript.data.GlobalScope )
      {
        text = "global";
        type = "global";
      }
      else
      {
        text = ""+c;   // XXX for debug
        text = text.substring( 21, text.length() - 1 );
      }
      
      return new function( public text, public type ) {} ( text, type );
    }
    
  }();
  
  /**
   * This is the default comparision function for Array#sort()
   */
  public const var __defaultArraySortComparision = function( a, b ) {
    a = a.castToString();
    b = b.castToString();
    if( a < b )
      return -1;
    else if( a == b )
      return  0;
    else
      return +1;
  };
  
}();


/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */
