/******************************************************************************
 *     Copyright Texas Instruments, Inc., 2001-2002.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * The <code>ActionManager</code> is a utility class for managing actions that 
 * are possibly context sensitive.  The actions can be added to the global 
 * menubar, and also to a toolbar managed by this action-mgr, through a single 
 * interface, {@link #addGlobalAction}.  The context-sensitive actions are 
 * collectively enabled/disabled by {@link #setEnabled}.
 */
public function ActionManager()
{
  var contextSensitiveActionList = new java.util.LinkedList();
  var toolBar = new swing.JToolBar();
  var enabled = false;
  
  /**
   * Create an action, which can be context sensitive.  A non context sensitive
   * basically is the same as if {@link pkg.util.ActionAdapter} was created
   * directly.
   * 
   * @param contextSensitive   is this action context sensitive?
   * @param name         the name of the action
   * @param tip          a tool-tip string, or <code>null</code>
   * @param icon         the icon, or icon file path, or <code>null</code>
   * @param fxn          the function to call when the action happens
   */
  public function createAction( contextSensitive, name, tip, icon, fxn )
  {
    if( (icon != null) && (icon instanceof String) )
      icon = pkg.util.getImageIcon(icon);
    
    var a = new pkg.util.ActionAdapter( name, tip, icon, fxn );
    
    if(contextSensitive)
    {
      a.setEnabled(enabled);
      contextSensitiveActionList.add(a);
    }
    
    return a;
  }
  
  /**
   * @param contextSensitive   is this action context sensitive?
   * @param menuBarPath  the path to menubar item or <code>null</code>
   * @param name         the name of the action
   * @param tip          a tool-tip string, or <code>null</code>
   * @param smallIcon    the small icon (or file path) to use for menubar
   *    items, or <code>null</code>
   * @param bigIcon      the icon (or icon file path) to use for toolbar, 
   *    or <code>null</code>
   * @param accel        the accelerator key, a single character string
   * @param fxn          the function to call when the action happens
   */
  public function addGlobalAction( contextSensitive, menuBarPath, name, tip, smallIcon, bigIcon, accel, fxn )
  {
    if( menuBarPath != null )
    {
      var path = registry.dirname(menuBarPath);
      var name = registry.basename(menuBarPath);
      var action = createAction( contextSensitive, name, null, smallIcon, fxn );
      
      if( accel != null )
      {
        var ks = pkg.util.getAcceleratorKeyStroke(accel);
        action.putValue( swing.Action.ACCELERATOR_KEY, ks );
      }
      
      services["window manager"].addMenuBarItem( path, action );
    }
    
    if( name != null )
      toolBar.add( createAction( contextSensitive, name, tip, bigIcon, fxn ) );
  }
  
  /**
   * Enable or disable the context sensitive actions created by this
   * action-manager.
   * 
   * @param enabled      <code>true</code>/<code>false</code> to 
   *    enable/disable
   */
  public function setEnabled(enabled)
  {
    this.enabled = enabled;
    for( var itr=contextSensitiveActionList.iterator(); itr.hasNext(); )
      itr.next().setEnabled(enabled);
  }
  
  /**
   * Is this ActionManager currently enabled?
   */
  public function isEnabled()
  {
    return enabled;
  }
  
  /**
   * Get the toolbar managed by this action-manager
   */
  public function getToolBar()
  {
    return toolBar;
  }
}

/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

