/*=============================================================================
 *     Copyright Texas Instruments 2002. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * A <code>CompletionSet</code> is a utility for managing a set of keys that
 * can be possible completions for some string.
 */
public function CompletionSet()
{
  /**
   * Note current implementation isn't particularly sophisticated as far as
   * looking up completions.
   */
  var set = new java.util.TreeSet();
  
  /**
   * Add all in set as keys to the completion set.
   * 
   * @param set    the set of keys to add to the set
   */
  public function addAll(set)
  {
    for( var item : set )
      add(item);
  }
  
  /**
   * Add a key to the completion set.
   * 
   * @param key    the key to add to the set (a string)
   */
  public function add(key)
  {
    set.add(key);
  }
  
  /**
   * Remove a key from the completion set.
   * 
   * @param key    the key to remove from the set (a string)
   */
  public function remove(key)
  {
    set.remove(key);
  }
  
  /**
   * Get an iterator of the keys that are possible completions for the
   * specified string.
   * <p>
   * For example, if the set consisted of the following keys:
   * <ul>
   *   <li> <code>"abc"</code>
   *   <li> <code>"aBc"</code>
   *   <li> <code>"abcdef"</code>
   *   <li> <code>"def"</code>
   * </ul>
   * then the completions of <code>"a"</code> will be <code>{ "abc", 
   * "aBc", "abcdef" }</code> while the completions of <code>"aB"</code>
   * will be <code>{ "aBc" }</code>
   * 
   * @param str    the string that is possibly a partial key
   * @return an iterator of zero or more possible completions for the
   *    specified string
   */
  public function getCompletions(str)
  {
    return new (function() {
      
      var itr  = set.iterator();
      var val = null;
      
      public function hasNext()
      {
        refresh();
        return val != null;
      }
      
      public function next()
      {
        refresh();
        if( val == null )
          throw new Exception("no more elements");
        var retval = val;
        val = null;
        return retval;
      }
      
      function refresh()
      {
        while( (val == null) && itr.hasNext() )
        {
          val = itr.next();
          if( val.indexOf(str) != 0 )
            val = null;
        }
      }
      
      // so can be used in a foreach loop
      public function iterator()
      {
        return this;
      }
      
    })();
  }
  
  /**
   * Get the common completions key or partial key that completes the
   * specified string.  If there are one or more possible completions
   * for the specified string, the return value will be the common
   * prefix of all those completions (which means, by definition, that
   * if there is only one possible completion, that entire key will be
   * returned).  If there are no completions, then <code>null</code>
   * is returned.
   * <p>
   * For example, if the set consisted of the following keys:
   * <ul>
   *   <li> <code>"abc"</code>
   *   <li> <code>"aBc"</code>
   *   <li> <code>"abcdef"</code>
   *   <li> <code>"def"</code>
   * </ul>
   * the the common completion of <code>"ab"</code> will be <code>"abc"
   * </code> while the common completion of <code>"abc"</code> will be
   * <code>"abcdef"</code>.
   * 
   * @param str    the string that is possibly a partial key
   * @return the common completion or <code>null</code>
   */
  public function getCommonCompletion(str)
  {
    var itr = getCompletions(str);
    
    if( ! itr.hasNext() )
      return null;
    
    var commonCompletion = itr.next();
    
    while( itr.hasNext() )
    {
      var nextCompletion = itr.next();
      
      while( nextCompletion.indexOf(commonCompletion) != 0 )
        commonCompletion = commonCompletion.substring( 0, commonCompletion.length() - 1 );
    }
    
    return commonCompletion;
  }
  
  /**
   * Return a string representation of this object
   * 
   * @return a string
   */
  public function castToString()
  {
    return set.toString();
  }
  
}












