/******************************************************************************
 *     Copyright Texas Instruments 2004. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


mixin javax.swing;
mixin java.awt;

/**
 * A <code>DrawerBox</code> is an "outlook-bar" like component, consisting of
 * a set of "drawers" stacked vertically.  A single drawer is expanded at a
 * time, and that drawer's component is displayed.  Like with other containers
 * (such as a <code>JTabbedPane</code>), this component contained by the drawer
 * can be anything.
 */
public function DrawerBar()
  extends JPanel()
{
  /**
   * Use a custom layout manager to keep everything placed properly... with
   * the width of the container equal to the width of the widest child, and
   * all children the same width.
   */
  setLayout( new function() extends LayoutManager() {
    
    public function addLayoutComponent( name, c ) {}
    public function removeLayoutComponent(c) {}
    
    public function layoutContainer(container)
    {
      const var insets = container.insets();
      const var w = container.getSize().width;
      const var x = insets.left;
      var y = insets.top;
      
      const var count = container.getComponentCount();
      for( var i=0; i<count; i++ )
      {
        var comp = container.getComponent(i);
        if( comp.isVisible() )
        {
          var ch = comp.getPreferredSize().height;
          comp.setBounds( x, y, w, ch );
          y += ch;
        }
      }
    }
    
    public function preferredLayoutSize(container)
    {
      return layoutSize( container, function(comp) {
        return comp.getPreferredSize();
      } );
    }
    
    public function minimumLayoutSize(container)
    {
      return layoutSize( container, function(comp) {
        return comp.getMinimumSize();
      } );
    }
    
    /**
     * private utility for computing layout size based on size of children 
     * of the container
     * 
     * @param container  the container
     * @param getComponentSize  function(component) returning a 
     *    <code>Dimension</code>
     */
    private function layoutSize( container, getComponentSize )
    {
      var w = 0;
      var h = 0;
      
      const var count = container.getComponentCount();
      for( var i=0; i<count; i++ )
      {
        var comp = container.getComponent(i);
        if( comp.isVisible() )
        {
          var d = getComponentSize(comp);
          
          // height is sum of all component's heights:
          h += d.height;
          
          // width is max of all component's widths
          if( d.width > w )
            w = d.width;
        }
      }
      
      var insets = container.insets();
      w += insets.left + insets.right;
      h += insets.top + insets.bottom;
      
      return new Dimension( w, h );
    }
    
  }() );
  
  private var drawers = [];
  
  private function deselectAll()
  {
    for( var drawer : drawers )
      drawer.setSelected(false);
  }
  
  /**
   * Add a drawer.  The <code>fxn</code> takes no args, and should return a
   * <code>JComponent</code>.
   * 
   * @param text     text describing the drawer
   * @param icon     the icon to display, or <code>null</code>
   * @param fxn      the function returning the drawer contents
   */
  public function addDrawer( text, icon, fxn )
  {
    var idx = drawers.length();
    var selected = (idx == 0);
    drawers[idx] = new function() {
      
      /**
       * If this drawer is visible, this will be the component displayed in
       * the drawer (ie non-<code>null</code>)
       */
      private var component = null;
      
      /**
       * Note, to get the look we want, simulate a button with a JLabel
       * plus a mouse-listener
       */
      private var button = new JLabel( text, icon, SwingConstants.LEADING );
      
      button.addMouseListener( new function() extends java.awt.event.MouseAdapter() {
        public function mousePressed(evt)
        {
          setSelected( component == null );
        }
      }() );
      
      add(button);
      
      private var panel = new swing.JPanel( new BorderLayout() );
      panel.setVisible(false);
      add(panel);
      
      /**
       * Change whether this drawer is currently selected.
       */
      public function setSelected(selected)
      {
        // if new state is different from current state:
        if( selected != (component != null) )
        {
          if(selected)
          {
            deselectAll();
            component = fxn();
            panel.add(component);
            panel.setVisible(true);
          }
          else
          {
            panel.remove(component);
            panel.setVisible(false);
            component = null;
          }
        }
        
        button.setBorder( selected ? BorderFactory.createLoweredBevelBorder() : BorderFactory.createRaisedBevelBorder());
      }
      
      setSelected(selected);
      
    }();
  }
  
  
  private var _super_removeAll = removeAll;
  /**
   * xxx should override other remove type methods...
   */
  public function removeAll()
  {
    _super_removeAll();
    drawers = [];
    repaint();
  }
}



/* test code:
 */
//var db = new DrawerBar();
//
//db.addDrawer( "Foo", pkg.util.getImageIcon("breakpointglyph.gif"), function() { return new JList(["One","Two"]); } );
//db.addDrawer( "Bar", pkg.util.getImageIcon("pcglyph.gif"), function() { return new JList(["One","Two"]); } );
//db.addDrawer( "Foo", pkg.util.getImageIcon("breakpointglyph.gif"), function() { return new JTable(); } );
//db.addDrawer( "Foo", pkg.util.getImageIcon("breakpointglyph.gif"), function() { return new JList(["One","Two"]); } );
//
//var d = services["window manager"].getDialog("Test DrawerBar");
//d.getContentPane().add( new JScrollPane(db) );
//d.pack();
//d.center();

