/*=============================================================================
 *     Copyright Texas Instruments 2002. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * A parser that writes the syntax tree to the specified print stream/writer
 * (requires print(str)/println(str) methods).
 * 
 * @param specials    if <code>true</code> dump special tokens too
 * @param out         the print writer/stream
 */
public function FileDumperParser( specials, out )
{
  const var visitor = new oscript.visitor.ExtensibleVisitor();
  const var st = oscript.syntaxtree;
  
  var line = 0;
  var col  = 0;
  
  // hack:  better solution would be to give pkg.output a print/println:
  if( out == null )
    out = new (function() {
      public var print   = pkg.output.write;
      public var println = pkg.output.writeln;
      public function flush() {}
    })();
  
  
  /**
   * Visit the tree starting at the specified root
   */
  public function visit( node, arg )
  {
    return visitor.visit( node, arg );
  }
  
  /**
   * Parse and visit the specified file.
   */
  public function parse(file)
  {
    return visit( oscript.OscriptInterpreter.parse( new pkg.File(file) ), getArg() );
  }
  
  /**
   * Overload to produce arg passed in to visit() when parse()'ing 
   * a file.
   * 
   * @return some value
   */
  function getArg()
  {
    return null;
  }
  
  {
    /**
     * Utility function used by NodeToken handler
     */
    function writeToken(str)
    {
      for( var i=0; i<str.length(); ++i )
      {
        if( str[i] == "\n" )
        {
          line++;
          col = 1;
        }
        else
        {
          col++;
        }
        
        out.print( str[i] );
      }
      
      out.flush();
    }
    
    /**
     * The default handling for NodeToken is to dump to the output
     * stream.
     */
    visitor.register( st.NodeToken, function( n, arg ) {
      
      if( (n.beginLine == -1) || (n.beginColumn == -1) )
        return writeToken( n.tokenImage );
      
      // Handle specials:
      if( specials && (n.numSpecials() > 0) )
        for( var e=n.specialTokens.elements(); e.hasMoreElements(); )
          visit( e.nextElement(), arg );
      
      if( line < n.beginLine )
        for( col=1; line < n.beginLine; line++ )
          out.println("");
      
      for( ; col < n.beginColumn; col++ )
        out.print(" ");
      
      writeToken(n.tokenImage);
      
    } );
  }
}

