/*=============================================================================
 *     Copyright Texas Instruments, Inc., 2001-2002.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Parse a string and return a KeyStroke.
 * The string has the following syntax:
 * <pre>
 *    &lt;modifiers&gt;* (&lt;typedID&gt; | &lt;pressedReleasedID&gt;)
 *    modifiers := shift | control | meta | alt | button1 | button2 | button3
 *    typedID := typed &lt;typedKey&gt;
 *    typedKey := string of length 1 giving unicode character.
 *    pressedReleasedID := (pressed | released)? key
 * key := KeyEvent keycode name, i.e. the name following "VK_".
 * </pre>
 * If typed, pressed or released is not specified, pressed is assumed.
 * Here are some examples:
 * <pre>
 *     "INSERT" => new KeyStroke(0, KeyEvent.VK_INSERT);
 *     "control DELETE" => new KeyStroke(InputEvent.CTRL_MASK, KeyEvent.VK_DELETE);
 *     "alt shift X" => new KeyStroke(InputEvent.ALT_MASK | InputEvent.SHIFT_MASK, KeyEvent.VK_X);
 *     "alt shift released X" => new KeyStroke(InputEvent.ALT_MASK | InputEvent.SHIFT_MASK, KeyEvent.VK_X, true);
 *     "typed a" => new KeyStroke('a');
 * </pre>
 * This works similarly to javax.swing.KeyStroke.getKeyStroke(String) with the
 * exception that "control" (or "ctrl") is remapped to be the system menu 
 * shortcut key.
 */
public function getKeyStroke(str)
{
  var stok = new java.util.StringTokenizer(str);
  var released  = false;
  var typed     = false;
  var modifiers = 0;
  
  while( stok.hasMoreElements() )
  {
    var token = stok.nextElement();
    
    // first check for modifier tokens:
    if( modifierToMaskTable.get(token) != null )
      modifiers |= modifierToMaskTable.get(token);
    else if( token == "released" )
      released = true;
    else if( token == "pressed" )
      released = false;
    else if( token == "typed" )
      typed = true;
    // if the token isn't a modifier, then we are done parsing and
    // it is time to create the KeyStroke:
    else if(typed)
      return swing.KeyStroke.getKeyStroke( token[0], released );
    else
      return swing.KeyStroke.getKeyStroke( java.awt.event.KeyEvent.getMember("VK_" + token),
                                           modifiers,
                                           released );
  }
  
  throw new ProgrammingErrorException("bad KeyStroke: str=\"" + str + "\"");
}

/**
 * This table maps modifier key to a InputEvent.<XYZ>_MASK.
 */
var modifierToMaskTable = new java.util.Hashtable();

if( false && /*XXX*/ java.lang.System.getProperty("mrj.version") != null )
{
  modifierToMaskTable.put( "control", java.awt.event.InputEvent.META_MASK );
  modifierToMaskTable.put( "meta", java.awt.event.InputEvent.CTRL_MASK );
}
else
{
  modifierToMaskTable.put( "control", java.awt.event.InputEvent.CTRL_MASK );
  modifierToMaskTable.put( "meta", java.awt.event.InputEvent.META_MASK );
}

modifierToMaskTable.put( "ctrl", modifierToMaskTable.get("control") );

modifierToMaskTable.put( "shift", java.awt.event.InputEvent.SHIFT_MASK );
modifierToMaskTable.put( "alt", java.awt.event.InputEvent.ALT_MASK );
modifierToMaskTable.put( "button1", java.awt.event.InputEvent.BUTTON1_MASK );
modifierToMaskTable.put( "button2", java.awt.event.InputEvent.BUTTON2_MASK );
modifierToMaskTable.put( "button3", java.awt.event.InputEvent.BUTTON3_MASK );


/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

