/*=============================================================================
 *     Copyright Texas Instruments, Inc., 2002.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Utility for plugins or views to subscribe to a registry value.  If the
 * <code>obj</code> is a view, this will automatically install a close-
 * runnable to unsubscribe when the view is closed.  If the <code>obj</code>
 * is a plugin, this will install a managed resource to that plugin to
 * subscribe when the plugin becomes active, and unsubscribe when the
 * plugin becomes inactive.
 * 
 * @param obj          the view or plugin that the subscription
 *    lifecycle will be coupled to
 * @param path         the path to subscribe to
 * @param contract     the optional contract to subscribe with,
 *    or <code>null</code> (see {@link Registry#subscribeToValue})
 * @param subscriber   the node-subscriber
 */
public function subscribeToValue( obj, path, contract, subscriber )
{
  if( obj instanceof ti.chimera.Plugin )
  {
    obj.addResource( new (function() extends ti.chimera.Resource(true) {
        
        public function install()
        {
          registry.subscribeToValue( path, contract, subscriber );
        }
        
        public function uninstall()
        {
          registry.unsubscribeFromValue(subscriber);
        }
        
      })() );
  }
  else if( obj instanceof ti.chimera.View )
  {
    registry.subscribeToValue( path, contract, subscriber );
    obj.addCloseRunnable( function() { registry.unsubscribeFromValue(subscriber); } );
  }
  else
  {
    throw new ProgrammingErrorException("don't know how to lifecycle couple subscriber to: " + obj);
  }
}

/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

