/*=============================================================================
 *     Copyright Texas Instruments 2003. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * The data-manger is responsible for collecting data from an individual
 * screen, or state, in the wizard's sequence of screens.  Each one stores
 * data into it's own internal table, but requests to access data, if not
 * fulfilled by this object, are delegated to the parent.  In this way,
 * a collection of data-managers can act as a single data store, but when
 * it is necessary to roll-back (ie. user presses "Back" button), the
 * wizard can pop the head off the stack of data-managers in order to
 * get rid of changes made by the screen that we stepped backwards past.
 * 
 * @param prev   the previous data-manger, or <code>null</code>
 * @param notify the function() {...} to call when there are changed to
 *    this data structure.  Used by the {@link Wizard} to get notification
 *    of changes to the data by the panel
 */
public function DataManager( prev, notify )
{
  private var table = new java.util.Hashtable();
  
  /**
   * Store a value.  The <code>key</code> can refer to a "." delimited path,
   * in order to have a hierarchial data structure to the data collected
   * from the user.
   * 
   * @param key   the key (a string)
   * @param val   the value (any object)
   */
  public function put( key, val )
  {
    table.put( key, val );
    notify();
  }
  
  /**
   * Get the value associated with the specified key, or <code>null</code>
   * if there is no value associated with the key.
   * 
   * @param key   the key (a string)
   * @return the value, or <code>null</code>
   */
  public function get(key)
  {
    var val = table.get(key);
    
    if( (val == null) && (prev != null) )
      val = prev.get(key);
    
    return val;
  }
  
  /**
   * Return an iterator of sub-keys.  If <code>key</code> is <code>null</code>
   * then this returns an iterator of all top-level keys.  For example, if the
   * set includes the following keys:
   * <pre>
   *   a
   *   a.foo
   *   a.bar
   *   b.1
   *   b.2
   * </pre>
   * then the subkeys of <code>null</code> are <code>{a, b}</code>, and the
   * subkeys of <code>"a"</code> are <code>{foo, bar}</code>, and so on.
   * 
   * @param key   the root key to return subkeys relative to, or <code>null</code>
   * @return an iterator
   */
  public function subkeys(key)
  {
    if( key == null )
      key = "";
    else
      key += ".";
    
    var table = new java.util.Hashtable();
    populate(table);
    
    var e = table.keys();
    
    return new function() extends java.util.Iterator() {
      
      private var val = null;
      
      // set of values we've already returned, so we don't return
      // the same value twice:
      var valSet = new java.util.HashSet();
      
      private function refresh()
      {
        while( (val == null) && e.hasMoreElements() )
        {
          val = e.nextElement();
          
          if( val.indexOf(key) != 0 )
          {
            val = null;
            continue;
          }
          
          val = val.substring( key.length() );
          var idx = val.indexOf(".");
          if( idx != -1 )
            val = val.substring( 0, idx );
          
          if( !valSet.add(val) )
            val = null;
        }
      }
      
      public function hasNext()
      {
        refresh();
        return val != null;
      }
      
      public function next()
      {
        refresh();
        var val = this.val;
        this.val = null;
        return val;
      }
      
    }();
  }
  
  /**
   * note only public so this can be called by other instances of same class
   */
  public function populate(table)
  {
    if( prev != null )
      prev.populate(table);
    table.putAll( this.table );
  }
  
  // for debug:
  public function castToString()
  {
    var table = new java.util.Hashtable();
    populate(table);
    return table.toString();
  }
}

