/*=============================================================================
 *     Copyright Texas Instruments 2003. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * The generic wizard panel is a general purpose wizard panel for collecting
 * standard sorts of data.  To use the generic panel, use any combination
 * of these methods to add input components to the panel:
 * <ul>
 *   <li> addTextField
 * </ul>
 * 
 * @param dm         the data manager for this screen
 * @param title      the title for this screen, or <code>null</code>
 * @param desc       the description of this screen, or <code>null</code>
 * @param finish     can the "finish" button be enabled, if this panel is 
 *    valid (see {@link #isValid})
 * @param twocols    if <code>true</code>, use a two-column layout.  This
 *    is useful if all the contents of the panel are just text fields,
 *    but can look goofy if other types of input components are used
 */
public function GenericWizardPanel( dm, title, desc, finish, twocols )
  extends pkg.wizard.WizardPanel( dm, title, desc, finish )
{
  var panel;
  if( twocols )
    panel = new swing.JPanel( new java.awt.GridLayout( -1, 2 ) );
  else
    panel = new swing.Box( swing.BoxLayout.Y_AXIS );
  
  /**
   * The list of properties that must be collected by this screen.
   */
  private var propsList = new java.util.LinkedList();
  
  /**
   * Add to the list of properties collected by this panel.
   * 
   * @param prop   the property
   */
  protected function addProperty(prop)
  {
    propsList.add(prop);
  }
  
  /**
   * helper to install document listener on any text component.
   */
  private function addDocumentListener( prop, tc )
  {
    tc.getDocument().addDocumentListener(
      new function() extends swing.event.DocumentListener() {
        
        public function changedUpdate(evt)
        {
          dm.put( prop, tc.getText() );
        }
        
        public var removeUpdate = changedUpdate;
        public var insertUpdate = changedUpdate;
        
      }()
    );
  }
  
  /**
   * Add a one-line text field to this wizard panel.  The property
   * value will be the text in the text field.
   * 
   * @param prop     the property
   * @param name     the descriptive name, shown to user
   * @param default  the default value for the text field
   * @param required is this field required
   */
  public function addTextField( prop, name, default, required )
  {
    if(required)
      addProperty(prop);
    
    //panel.add( new swing.JLabel(name) );
    var lp = new swing.JPanel();
    lp.add( new swing.JLabel(name) );
    panel.add(lp);
    var tf = new swing.JTextField(20);
    addDocumentListener( prop, tf );
    panel.add(tf);
    
    if( default != null )
    {
      dm.put( prop, default );
      tf.setText(default);
    }
  }
  
  /**
   * Add a multi-line text area to this wizard panel.  The property
   * value will be the text in the text field.
   * 
   * @param prop     the property
   * @param name     the descriptive name, shown to user
   * @param default  the default value for the text field
   * @param required is this field required
   */
  public function addTextArea( prop, name, default, required )
  {
    if(required)
      addProperty(prop);
    
    var ta = new swing.JTextArea(3,50);
    ta.setLineWrap(true);
    ta.setWrapStyleWord(true);
    addDocumentListener( prop, ta );
    var sp = new swing.JScrollPane(ta);
    sp.setBorder( new swing.border.TitledBorder(name) );
    panel.add(sp );
    
    if( default != null )
    {
      dm.put( prop, default );
      ta.setText(default);
    }
  }
  
  /**
   * Add a checkbox to this wizard panel.  The property value will
   * be <code>true</code> if the checkbox is checked, otherwise it
   * will be <code>false</code>
   * 
   * @param prop     the property
   * @param name     the descriptive name, shown to user
   * @param default  the default value for the text field
   * @param required is this field required
   */
  public function addCheckBox( prop, name, default, required )
  {
    if(required)
      addProperty(prop);
    
    if( default == null )
      default = false;
    
    dm.put( prop, default );
    
    var cb = new swing.JCheckBox( name, default );
    
    cb.addActionListener( function(evt) {
      dm.put( tuple[0], cb.isSelected() );
    } );
    
    panel.add(cb);
  }
  
  /**
   * Add a group of radio button.  The property value will be the
   * selected value, from the array of all possible <code>values</code>
   * 
   * @param prop     the property
   * @param name     the descriptive name, shown to user
   * @param values   the array of all possible values
   * @param default  the default value for the text field
   * @param required is this field required
   */
  public function addRadioButtons( prop, name, values, default, required )
  {
    if(required)
      addProperty(prop);
    
    var buttonPanel = new swing.JPanel( new java.awt.GridLayout( 0, 1 ) );
    var buttonGroup = new swing.ButtonGroup();
    
    for( var j=0; j<values.length(); j++ )
    {
      var name     = values[j];
      var selected = name == default;
      
      var rb = new swing.JRadioButton( name, selected );
      
      if(selected)
        dm.put( prop, name );
      
      rb.addActionListener( function(evt) {
        dm.put( prop, name );
      } );
      
      buttonPanel.add(rb);
      buttonGroup.add(rb);
    }
    
    buttonPanel.setBorder( new swing.border.TitledBorder(name) );
    panel.add(buttonPanel);
  }
  
  add(panel);
  
  /**
   * Has the user entered all the required data for this panel?
   * 
   * @return <code>false</code> if user must enter more data, otherwise if
   *    <code>true</code> if the user can proceed to the next screen
   */
  public function isValid()
  {
    var val;
    for( var itr=propsList.iterator(); itr.hasNext(); )
      if( ((val = dm.get( itr.next() )) == null) || (val == "") )
        return false;
    return true;
  }
}

