/*=============================================================================
 *     Copyright Texas Instruments 2003. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * The Wizard is a special panel used to walk the user through a sequence 
 * of screens to collect data.  Each state is a function, or factory, that 
 * returns a {@link WizardPanel}.  This factory is passed an instance of a 
 * {@link DataManager} which is constructed by the wizard, and used by the 
 * panel to store data collected from the user.  The wizard itself handles 
 * management of these data managers in such a way that data added/overriden 
 * by a screen can be rolled-back if the user hits the "Back" button.
 * <p>
 * A simple wizard may just specify a fixed number of screens, which the 
 * wizard navigates the user through in sequence, or any given screen may,
 * by calling the wizard's {@link #setStateSequence} method, specify a new
 * next state, to form a dynamic sequence of states.
 * <p>
 * The wizard implementation should extend this class and implement the
 * {@link #doFinish} and {@link #doCancel} methods.  This base class will
 * handle the next and back buttons for you.  For example:
 * <pre>
 *   var wizard = new function() extends pkg.wizard.Wizard() {
 *     
 *     setImage("spiffy.gif");
 *     
 *     function doCancel()
 *     {
 *       dialog.setVisble(false);
 *     }
 *     
 *     function doFinish()
 *     {
 *       var dm = getDataManager();
 *       ... do something with data ...
 *       dialog.setVisible(false);
 *     }
 *     
 *     setStateSequence( [
 *       function(dm) { return new FirstScreen(dm); },
 *       function(dm) { return new SecondScreen(dm); },
 *       function(dm) { return new ThirdScreen(dm); }
 *     ] );
 *     
 *   }();
 *   
 *   dialog.getContentPane().add(wizard);
 *   dialog.setVisible(true);
 * </pre>
 * The call to {@link #setImage} is optional.
 */
public function Wizard()
  extends swing.JPanel( new java.awt.BorderLayout() )
{
  /**
   * In order to support rolling back (the back button) the wizard maintains
   * it's state in a state object.  Each state object has a link to the prev
   * state object, so rolling back is simply a matter of walking backwards
   * through the list.
   */
  private function State( public prev )
  {
    /**
     * The current wizard-panel.
     */
    public var panel = null;
    
    /**
     * The array of states (panel factory functions)
     */
    public var fxns = null;
    
    /**
     * The index into the array of states
     */
    public var fxnsIdx = 0;
  }
  
  private var state = null;
  
  /**
   */
  private var contentPanel = new swing.JPanel( new java.awt.BorderLayout() );
  add( java.awt.BorderLayout.CENTER, contentPanel );
  
  /* 
   * Build navigation button panel
   */
  private var navButtonPanel = new swing.JPanel();
  
  private var backButton = new swing.JButton("Back");
  backButton.addActionListener( function(evt) { doBack(); } );
  navButtonPanel.add(backButton);
  
  private var nextButton = new swing.JButton("Next");
  nextButton.addActionListener( function(evt) { doNext(); } );
  navButtonPanel.add(nextButton);
  
  private var cancelButton = new swing.JButton("Cancel");
  cancelButton.addActionListener( function(evt) { doCancel(); } );
  navButtonPanel.add(cancelButton);
  
  private var finishButton = new swing.JButton("Finish");
  finishButton.addActionListener( function(evt) { leaveState(); doFinish(); } );
  navButtonPanel.add(finishButton);
  
  add( java.awt.BorderLayout.SOUTH, navButtonPanel );
  
  /**
   * helper to refresh the state of the buttons in the navigation
   * button panel
   */
  private function refreshButtons()
  {
    if( (state != null) && (state.panel != null) )
    {
      var moreFxns = (state.fxnsIdx + 1) < state.fxns.length();
      
      backButton.setEnabled( state.prev != null );
      nextButton.setEnabled( state.panel.isValid() && moreFxns );
      finishButton.setEnabled( state.panel.isValid() && 
                               (state.panel.canFinish() || !moreFxns) );
    }
  }
  
  /**
   * helper to refresh current panel, and nav-bar buttons
   */
  private function refreshAll()
  {
    contentPanel.add( java.awt.BorderLayout.NORTH, state.panel );
    setActive( state.panel, true );
    refreshButtons();
  }
  
  /**
   * helper to activate/deactivate the panel component as we add/remove it
   */
  private function setActive( c, enabled )
  {
    if( c != null )
    {
      c.setVisible(enabled);
      c.setEnabled(enabled);
    }
  }
  
  /**
   * helper called when leaving the current state.
   */
  private function leaveState()
  {
    if( (state != null) && (state.panel != null) )
      state.panel.leavePanel();
  }
  
  /**
   * helper to transition to new state
   * 
   * @param fxns     the array of states, including future states
   * @param fxnsIdx  the index into fxns for the current state
   */
  private function nextState( fxns, fxnsIdx )
  {
    leaveState();
    
    var dm = getDataManager();
    
    if( state != null )
    {
      setActive( state.panel, false );
      contentPanel.remove( state.panel );
    }
    
    state = new State(state);
    
    state.fxns    = fxns;
    state.fxnsIdx = fxnsIdx;
    state.panel   = fxns[fxnsIdx]( new pkg.wizard.DataManager( dm, refreshButtons ) );
    
    refreshAll();
  }
  
  /**
   * Set the sequence of one or more states (screen factories) to step thru
   * in sequence, and transition to the first state in the specified sequence.  
   * This should be called at least once by the derived class in order to 
   * indicate the sequence of states.  More complex wizards that may require 
   * different next-states leading out of a specific screen may also use this 
   * to set the next state(s).  This operation completely replaces the current 
   * sequence of states, and resets the counter into the sequence to zero.  
   * However, it does <i>not</i> reset the history of previous states.
   * <p>
   * Note that history is maintained in such a way that the sequence of next
   * states, and the current position in the sequence, is restored as the
   * user steps back.
   * 
   * @param fxns  the sequence of factory functions
   */
  public function setStateSequence(fxns)
  {
    nextState( fxns, 0 );
  }
  
  /**
   * Handle the "next" button by going to the next state.
   */
  protected function doNext()
  {
    nextState( state.fxns, state.fxnsIdx + 1 );
  }
  
  /**
   * Handle the "back" button by stepping back to the previous
   * screen.
   */
  protected function doBack()
  {
    setActive( state.panel, false );
    contentPanel.remove( state.panel );
    state = state.prev;
    refreshAll();
  }
  
  /**
   * Subclasses should override this method to perform whatever processing
   * is needed as a result of the user pressing the "Cancel" button.
   */
  protected function doCancel() {}
  
  /**
   * Subclasses should override this method to perform whatever processing
   * is needed as a result of the user pressing the "Finish" button.
   */
  protected function doFinish() {}
  
  /**
   * Set the specfied image to appear along the left hand side of the wizard
   * screen.
   * 
   * @param img   an image or name of a image resource
   */
  public function setImage(img)
  {
    add( java.awt.BorderLayout.WEST, new swing.JLabel( pkg.util.getImageIcon(img) ) );
  }
  
  /**
   * Get the data-manager, which contains the data that has been collected
   * from the user so far.
   */
  public function getDataManager()
  {
    if( state == null )
      return null;
    return state.panel.getDataManager();
  }
  
  {
    var d = new java.awt.Dimension( 520, 360 );
    setSize(d);
    setPreferredSize(d);
    setMinimumSize(d);
    setMaximumSize(d);
  }
}
