/*=============================================================================
 *     Copyright Texas Instruments 2003. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * A wizard panel is a base class for "screens" in the wizard.  It must
 * define the method <code>isValid()</code> which should return a boolean
 * to indicate whether or not this screen has collected all the data that
 * the user is required to enter.
 * <p>
 * The derived class should only add() a single component to the panel,
 * and if it requires multiple components, it should encapsulate them
 * in a container of some sort (ie. panel, scroll-pane, etc.). 
 * 
 * @param dm         the data manager for this screen
 * @param title      the title for this screen, or <code>null</code>
 * @param desc       the description of this screen, or <code>null</code>
 * @param finish     can the "finish" button be enabled, if this panel
 *    is valid (see {@link #isValid})
 */
public function WizardPanel( dm, title, desc, finish )
  extends swing.JPanel( new java.awt.BorderLayout() )
{
  if( (title != null) || (desc != null) )
  {
    var titlePanel = new swing.JPanel( new java.awt.BorderLayout() );
    
    if( title != null )
    {
      var label = new swing.JLabel(title);
      label.setFont( new java.awt.Font("Helvetica", java.awt.Font.PLAIN, 24) );
      titlePanel.add( java.awt.BorderLayout.NORTH, label );
    }
    
    if( desc != null )
    {
      var label = new swing.JLabel("<html>" + desc + "</html>");
      titlePanel.add( java.awt.BorderLayout.CENTER, label );
    }
    
    add( java.awt.BorderLayout.NORTH, titlePanel );
  }
  
  /**
   * The derived class must override this method to return <code>true</code>
   * once all the required data has been collected.
   * 
   * @return <code>false</code> if user must enter more data, otherwise if
   *    <code>true</code> if the user can proceed to the next screen
   */
  public function isValid()
  {
    return false;
  }
  
  /**
   * Should the "finish" button be enabled when this panel is valid?
   */
  public function canFinish()
  {
    return finish;
  }
  
  /**
   * Get the data manager associated with this screen
   */
  public function getDataManager()
  {
    return dm;
  }
  
  /**
   * This method is called when the wizard is leaving this state (screen),
   * to provide a hook for the implementor to do something with the data
   * collected by this screen.
   */
  public function leavePanel()
  {}
}

